@@ -2,19 +2,9 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
@@ -27,6 +17,8 @@ static gchar *wc_next;
 
 struct cw *init_cw(){
     struct cw *cw;
+    time_t now;
+    struct tm tm;
 
     cw = g_new0(struct cw, 1);
     
@@ -36,6 +28,12 @@ struct cw *init_cw(){
     cw_next = NULL;
     wc_next = NULL;
     cw->latest = 0;
+    
+    
+    time(&now);
+    now -= 740 * 86400L; /* 740 days = two years + something */
+    gmtime_r(&now, &tm);
+    cw->minstamp = (tm.tm_year+1900)*10000 + (tm.tm_mon+1)*100 + tm.tm_mday;
 
     return cw;
 }
@@ -109,7 +107,7 @@ int load_cw_from_file(struct cw *cw, gchar *filename){
 
     f = fopen(filename, "rt");
     if (!f){
-        dbg("Can't open '%s'\n", filename);
+        /*dbg("Can't open '%s'\n", filename);*/
         return -1;
     }
 
@@ -195,8 +193,8 @@ void add_cw(struct cw *cw, gchar *call, gchar *wwl, gint stamp, gchar *qrv_str){
     }
     
     if (qrv_str){ /* can be NULL*/
-        for (c=qrv_str;*c!='\0';*c++){
-            *c=toupper(*c);
+        for (c=qrv_str;*c!='\0';c++){
+            *c=upcase(*c);
             cwi->qrv|=1<<(*c-'A');
         }
     }
@@ -277,6 +275,16 @@ gchar *find_wwl_by_call(struct cw *cw, gchar *call){
     return cwi->wwl0;
 }
 
+gchar *find_wwl_by_call_newer(struct cw *cw, gchar *call, int maxstamp){
+    struct cw_item *cwi;
+
+    cwi = t_hash_table_lookup(cw->cw, call);
+    if (!cwi) return NULL;
+    if (cwi->stamp0 > maxstamp) return cwi->wwl0;
+    if (cwi->stamp1 > maxstamp) return cwi->wwl1;
+    return NULL;
+}
+
 gchar *find_call_by_wwl(struct cw *cw, gchar *wwl){
     struct wc_item *wci;
 
@@ -347,59 +355,54 @@ int is_string_in_array(GPtrArray *arr, gchar *str){
 }
 
 
-int cmp_cw_qs(gpointer key, gpointer value, gpointer data){
-    gchar *c,*call;
+int cmp_cw_qs(gchar *call, struct cw_item *cwi, GPtrArray *result){
+    gchar *c;
     int ret=0, i;
     int qrb_int, qtf_int;
     char qrv_str[35];
+    int qrvfilter;
 
-    struct cw_item *cwi;
-    struct job *job;
-
-    job  = (struct job*)data;
-    call = (gchar*)key;
-    cwi  = (struct cw_item*)value;
-
+    qrvfilter = ctest ? ctest->qrv : -1; 
     
     c=qrv_str;
-    
     for (i=0;i<32;i++){
-        if ((cwi->qrv & ctest->qrv & (1<<i)) == 0) continue;
+        if ((cwi->qrv & qrvfilter & (1<<i)) == 0) continue;
         *c='A'+i;
         c++;
     }
     *c='\0';
+
     
-    if (my_strstr(call, job->str)){
+    if (my_strstr(call, gses->qs_str)){
         if (cwi->wwl0) {
             qrb_qtf_int(cwi->wwl0, &qrb_int, &qtf_int);
             c=g_strdup_printf("%-10s %-6s%5d/%3d %s", call, cwi->wwl0, qrb_int, qtf_int, qrv_str);
-            g_ptr_array_add(job->result, c);
+            g_ptr_array_add(result, c);
             if (cwi->wwl1){ /* wwl1 is only together with wwl0 */
                 qrb_qtf_int(cwi->wwl1, &qrb_int, &qtf_int);
                 c=g_strdup_printf("%-10s %-6s%5d/%3d %s", call, cwi->wwl1, qrb_int, qtf_int, qrv_str);
-                g_ptr_array_add(job->result, c);
+                g_ptr_array_add(result, c);
             }
         }else{
             c=g_strdup(call);
-            g_ptr_array_add(job->result, c);
+            g_ptr_array_add(result, c);
         }
         goto x;
     }
     
-    if (cwi->wwl0 && my_strstr(cwi->wwl0, job->str)){
+    if (cwi->wwl0 && my_strstr(cwi->wwl0, gses->qs_str)){
         qrb_qtf_int(cwi->wwl0, &qrb_int, &qtf_int);
         c=g_strdup_printf("%-10s %-6s%5d/%3d %s", call, cwi->wwl0, qrb_int, qtf_int, qrv_str);
      /*   dbg(",%s", c);*/
-        g_ptr_array_add(job->result, c);
+        g_ptr_array_add(result, c);
         goto x;
     }
     
-    if (cwi->wwl1 && my_strstr(cwi->wwl1, job->str)){
+    if (cwi->wwl1 && my_strstr(cwi->wwl1, gses->qs_str)){
         qrb_qtf_int(cwi->wwl1, &qrb_int, &qtf_int);
         c=g_strdup_printf("%-10s %-6s%5d/%3d %s", call, cwi->wwl1, qrb_int, qtf_int, qrv_str);
     /*    dbg(",%s", c);  */
-        g_ptr_array_add(job->result, c);
+        g_ptr_array_add(result, c);
         goto x;
     }
 x:;    
@@ -415,125 +418,90 @@ int compare_gstring(const void *a, const void *b){
     return (strcmp(*sa, *sb));
 }
 
+void get_cw_qs(gchar *str){
+    
+    qs_thread_kill();
+    
+    
+    if (strlen(str)<2) {
+        g_ptr_array_free_items(gses->qs);
+        return;
+    }
 
-
-
-typedef int (*GHFuncM) (gpointer key,gpointer value, gpointer user_data);
-
-void g_hash_table_foreach_max (THashTable *hash_table, GHFuncM func, gpointer user_data){
-    THashNode *node;
-    gint i;
-    int count;
-  
-    g_return_if_fail (hash_table != NULL);
-    g_return_if_fail (func != NULL);
-  
-    count=0;
-    for (i = 0; i < hash_table->size; i++)
-        for (node = hash_table->nodes[i]; node; node = node->next){
-            count++;
-            if (func(node->key, node->value, user_data)) goto x;
-        }
-x:;  
+    CONDGFREE(gses->qs_str);
+    gses->qs_str=g_strdup(str);
+    gses->qs_max_matches = term->y - QSONR_HEIGHT - 4 - cfg->loglines - 1;
+    if (aband) gses->qs_max_matches -= aband->spypeers->len;
+    if (ctest && ctest->bands->len==1) gses->qs_max_matches*=2;
+    if (!ctest) gses->qs_max_matches*=3;
+    qs_thread_create();
 }
 
-int search_in_cwdb_job (struct job *job){
+gpointer qs_thread_func(gpointer data){
+    char s[256];
+    GPtrArray *result;
     THashNode *node;
     gint i;
-    struct timeval timeout,tv;
-    int expire=0;
     int count;   
-    GHFuncM func;
-    
-    func = (GHFuncM) job->h_func_v;  
-  
-    /*dbg("search_in_cwdb_job\n");*/
-    /*ST_START;*/
-    if (!job->hash_table || !func) return 0;
-  
-    gettimeofday(&timeout, NULL);
-    timeout.tv_usec += 5000;
-    if (timeout.tv_usec<0){timeout.tv_usec+=1000000;timeout.tv_sec--;}
-        
-    count=0;
-    expire=0;
-    for (i = job->node_from; i < job->hash_table->size; i++){
-        for (node = job->hash_table->nodes[i]; node; node = node->next){
-            func(node->key, node->value, job);
-            
-            if (job->result->len >= job->max_matches) goto x;
-   
 
-            if (count++==200){
-                count=0;
-                gettimeofday(&tv, NULL);
-                if (tv.tv_sec>timeout.tv_sec) goto expire;
-                if (tv.tv_sec==timeout.tv_sec &&
-                    tv.tv_usec>timeout.tv_usec) goto expire;
-                
+    result=g_ptr_array_new();
+    
+    count=0;
+    for (i = 0; i < cw->cw->size; i++){
+        for (node = cw->cw->nodes[i]; node; node = node->next){
+            cmp_cw_qs(node->key, node->value, result);
+            
+            if (result->len >= gses->qs_max_matches) goto x;
+   
+            if (gses->qs_thread_break){
+                g_ptr_array_free_all(result);
+                return NULL;
             }
+
         }
     }
 x:; 
-    g_index_array_qsort((GIndexArray *)job->result, compare_gstring);
-    redraw_later(term); 
-    free_job(job);
-/*    ST_STOP;*/
-    return 0; 
+    g_index_array_qsort((GIndexArray *)result, compare_gstring); 
     
-expire:; 
-    g_index_array_qsort((GIndexArray *)job->result, compare_gstring);
-    redraw_later(term); 
-    job->node_from = i+1;
-    
-    register_job(job);
+    LOCK(gses->qs);
+    g_ptr_array_free_all(gses->qs);
+    gses->qs=result;
+    UNLOCK(gses->qs);
     
+    sprintf(s, "CWQS\n");
+    write(tpipe->threadpipe_write, s, strlen(s));
 
-/*    ST_STOP;*/
-    return 0; 
-    
+	return NULL;
+}
+
+void qs_thread_create(){
+    if (gses->qs_thread) raise(SIGSEGV);
+    gses->qs_thread_break=0;
+    gses->qs_thread=g_thread_create(qs_thread_func, (gpointer)NULL, TRUE, NULL);
+    if (!gses->qs_thread) raise(SIGSEGV);
 }
 
 
-
-GPtrArray *get_cw_qs(GPtrArray *result, gchar *str){
-    int len;
-    struct job *job;
-
-    g_ptr_array_free_items(result);
-    
-    abort_jobs();
-    
-    len = strlen(str);
-    if (len<2) return NULL;
-    
-    job=g_new0(struct job,1);
-    job->func = search_in_cwdb_job;
-    job->timeout.tv_sec = 0;
-    job->timeout.tv_usec = 1000;
-    job->max_matches = term->y - QSONR_HEIGHT - 4 - cfg->loglines - aband->spypeers->len -1;
-    if (ctest->bands->len==1) job->max_matches*=2;
-  /*  dbg("max_matches=%d\n", job->max_matches);*/
-    job->result = result;
-    job->str = str;
-    job->node_from = 0;
-    job->hash_table = cw->cw;
-    job->h_func_v = cmp_cw_qs;
-    
-    register_job(job);
-            
-    /* TODO */
-    
-    
-    return result;
+void qs_thread_join(){
+    if (!gses->qs_thread) return;
+    g_thread_join(gses->qs_thread);
+    gses->qs_thread=NULL;
 }
 
+void qs_thread_kill(){
+    if (!gses->qs_thread) return;
+    gses->qs_thread_break=1;
+    g_thread_join(gses->qs_thread);
+  /*  dbg("done\n");*/
+    gses->qs_thread=NULL;
+}
 
 void update_cw_from_band(struct cw *cw, struct band *band){
     int i;
     struct qso *q;
     gchar qrv_str[2];
 
+    stats_thread_join(band);
     qrv_str[0]=band->bandchar;
     qrv_str[1]='\0';
 
@@ -566,3 +534,85 @@ void update_cw_from_ctest(struct cw *cw, struct contest *ctest){
     }
     
 }
+
+
+
+int similar_calls(const char *call1, const char *call2, int factor, int thr, int p){
+    const char *c1, *c2;
+    int f1, f2, f3;
+
+    if (factor>thr) return 9;
+    c1=call1;
+    c2=call2;
+
+    while(*c1!='\0' && *c2!='\0'){
+
+        if (*c1==*c2){
+            c1++;
+            c2++;
+            continue;
+        }
+        f1 = similar_calls(c1+1, c2+1, factor+1, thr, 0); 
+        f2 = similar_calls(c1, c2+1, factor+2, thr, 0);
+        f3 = similar_calls(c1+1, c2, factor+2, thr, 0);
+        if (f1<f2){
+            if (f1<f3){
+                factor=f1;
+            }else{
+                if (f2<f3) factor=f2;
+                else factor=f3;
+            }
+        }else{
+            if (f2<f3) factor=f2;
+            else factor=f3;
+        }
+        break;
+
+    }
+
+    if (p) printf("similar %s %s = %d\n", call1, call2, factor);
+    /*else printf("          %s %s = %d\n", call1, call2, factor);*/
+    return factor;
+}
+
+
+int get_susp_ambiguous_call(struct cw *cw, char *call, char *wwl, GString *gs, int thr){
+    int i;
+    THashNode *node;
+    struct cw_item *cwi;
+
+    for (i = 0; i < cw->cw->size; i++){
+        for (node = cw->cw->nodes[i]; node; node = node->next){
+            cwi  = (struct cw_item *) node->value;
+            
+            if ((cwi->wwl0 && strcmp(cwi->wwl0, wwl)==0 && cwi->stamp0 > cw->minstamp) ||
+                (cwi->wwl1 && strcmp(cwi->wwl1, wwl)==0 && cwi->stamp1 > cw->minstamp)){
+                            
+                /* dbg("hit %s %s\n", node->key, wwl);*/
+                if (strcmp((gchar*)node->key, call)==0) continue;
+                if (similar_calls((gchar*)node->key, call, 0, thr, 0)<=thr){
+                    if (!gs) return 1;
+
+                    g_string_append(gs, node->key);
+                    g_string_append_c(gs, ' ');
+                }
+            }
+        }
+    }
+    return 0;
+}
+
+int cwdb_call_info(GString *gs, gchar *call, gchar *stroke){
+    struct cw_item *cwi;
+    char c[40];
+
+    strcpy(c, call);
+    strcat(c, stroke);
+    cwi = t_hash_table_lookup(cw->cw, c);
+    if (!cwi) return 0;
+    if (!cwi->wwl0) return 0;  /* wwl0 should be always defined */
+    g_string_sprintfa(gs, "C_W: %-9s %s %d", c, cwi->wwl0, cwi->stamp0);
+    if (cwi->wwl1) g_string_sprintfa(gs, ", %s %d", cwi->wwl1, cwi->stamp1);
+    g_string_append(gs, "\n");
+    return 1;
+}
