@@ -2,19 +2,9 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
@@ -155,7 +145,6 @@ int write_edi_header(struct band *b, struct config_band *confb, FILE *f){
     g_string_sprintfa(gs, "CToSc=%d\r\n",       b->stats->ntotal); 
     g_string_sprintfa(gs, "CODXC=%s;",  uc(safe_strncpy0(s, b->stats->odxcall, 20)));
     g_string_sprintfa(gs, "%s;%d\r\n",    uc(safe_strncpy0(s, b->stats->odxwwl,  20)), b->stats->odxqrb_int);  
-    g_string_sprintfa(gs, "MODESEP=%d\r\n", ctest->modesep);  
     
     ret=fprintf(f, "%s", gs->str) != gs->len;
     g_string_free(gs, TRUE);
@@ -301,6 +290,7 @@ int export_all_bands_edi(){
         band = g_ptr_array_index(ctest->bands, i);
         confb = get_config_band_by_bandchar(band->bandchar);
 
+        stats_thread_join(band);
         if (band->stats->nqsos <=0) continue;
 
         filename = g_strdup_printf("%s/%s%s.edi",
@@ -310,7 +300,7 @@ int export_all_bands_edi(){
         
         f=fopen(filename,"wb"); /* must be b for windoze */
         if (!f) {
-            if (!ignoreerror) { errbox(TEXT(T_CANT_WRITE), errno); ignoreerror=1;}
+            if (!ignoreerror) { errbox(VTEXT(T_CANT_WRITE), errno); ignoreerror=1;}
             g_free(filename);
             return -1;
         }
@@ -320,7 +310,7 @@ int export_all_bands_edi(){
         err|=write_edi_qsos          (band,confb,f);
         err|=fclose(f);
         if (err) {
-            if (!ignoreerror) { errbox(TEXT(T_CANT_WRITE), 0); ignoreerror=1; }
+            if (!ignoreerror) { errbox(VTEXT(T_CANT_WRITE), 0); ignoreerror=1; }
             g_free(filename);
             return -1;
         }
@@ -371,7 +361,7 @@ int save_all_bands_txt(int autosave){
     int ignoreerror=0;
     int floppy_mounted,nm_showed;
     
-    /*dbg("save_all_bands_txt(%d)\n", autosave);*/
+    dbg("save_all_bands_txt(%d)\n", autosave);
     
     if (!ctest) return -1;
     
@@ -381,7 +371,7 @@ int save_all_bands_txt(int autosave){
         for (j=0; j<2; j++){
             band = g_ptr_array_index(ctest->bands, i);
             confb = get_config_band_by_bandchar(band->bandchar);
-
+/*            dbg("  %c dirty=%d\n", band->bandchar, band->dirty_save); */
             if (autosave && !band->dirty_save) continue;
             
             if (j==0){
@@ -390,12 +380,13 @@ int save_all_bands_txt(int autosave){
                 if (autosave && autosave!=2) continue;
 				if (nm_showed) continue;
                 if (!floppy_mounted && !nm_showed) {
-                    log_addf(TEXT(T_NOT_MOUNTED_S), cfg->as_floppy_path);
+                    log_addf(VTEXT(T_NOT_MOUNTED_S), cfg->as_floppy_path);
                     nm_showed=1;
 					continue;
                 }
             }
             
+            stats_thread_join(band);
             dir=get_ctest_directory(j);
             if (!dir) continue;
             mkdir_p(dir, 0775);
@@ -403,15 +394,15 @@ int save_all_bands_txt(int autosave){
             filename = g_strdup_printf("%s/%c.txt", dir, band->bandchar);
             g_free(dir);
             
-            /*dbg("saving filename '%s'\n", filename);*/
+            dbg("  saving filename '%s'\n", filename);
             f=fopen(filename,"wb"); /* must be b for windoze */
             if (!f) {
                 gchar *c;
-                c = g_strdup_printf(TEXT(T_CANT_WRITE_S), filename);
+                c = g_strdup_printf(VTEXT(T_CANT_WRITE_S), filename);
                 log_addf(c);
                 g_free(c);
                 if (j==0){
-                    c = g_strdup_printf(TEXT(T_CANT_WRITE_S), filename);
+                    c = g_strdup_printf(VTEXT(T_CANT_WRITE_S), filename);
                 }else{
                     c = g_strdup_printf("Can't open file '%s' for writing\nTry to mount floppy or disable\nMenu->Setup->Autosave->Path To Floppy", filename);
                 }
@@ -428,15 +419,15 @@ int save_all_bands_txt(int autosave){
             err|=write_edi_qsos    (band,confb,f);
             err|=fclose(f);
             if (err) {    
-                log_addf(TEXT(T_CANT_WRITE_S), filename);
-                if (!ignoreerror) { errbox(TEXT(T_CANT_WRITE), 0); ignoreerror=1;}
+                log_addf(VTEXT(T_CANT_WRITE_S), filename);
+                if (!ignoreerror) { errbox(VTEXT(T_CANT_WRITE), 0); ignoreerror=1;}
                 g_free(filename);
                 continue;
             }
             band->dirty_save = 0;
             fprintf(band->swap, "ST %d\n", band->saveid);
             fflush(band->swap);
-            if (!autosave) log_addf(TEXT(T_SAVED_S), filename);
+            if (!autosave) log_addf(VTEXT(T_SAVED_S), filename);
             g_free(filename);
         }
         band->saveid++;
@@ -549,7 +540,7 @@ gchar *load_edi_remarks_until(FILE *f, gchar *until){
         if (ret) {
             tmp=ret;
             g_free(ret);
-            ret=g_strconcat(tmp," ",c);
+            ret=g_strconcat(tmp," ",c, NULL);
         }else{
             ret=g_strdup(c);
         }
@@ -702,11 +693,12 @@ void add_qsos_from_mem(struct band *b, GHashTable *hash, GPtrArray *qsos, gchar
         g_strfreev(items);
         compute_qrbqtf(q);
         add_qso(b, q);
-        update_stats(b->stats, b, q);
+        update_stats(b, b->stats, q);
         
         
 x:;
     }
+    minute_stats(b);
 /*    g_hash_table_foreach(ctest->bystamp, foreach_source_print, NULL);*/
 
 /*   don't needed   DIRTY_BAND(); */
@@ -751,7 +743,7 @@ int convert_bo_s_hash(GHashTable *hash, char *dir){
 
     c = g_hash_table_lookup(hash, "band");
     if (c){
-        tmpcb = get_config_band_by_bandchar(tolower(c[0]));
+        tmpcb = get_config_band_by_bandchar(lowcase(c[0]));
         if (tmpcb){
             if (tmpcb->pband && !g_hash_table_lookup(hash, "pband"))
                 g_hash_table_insert(hash, g_strdup("pband"), g_strdup(tmpcb->pband));
@@ -797,24 +789,28 @@ void load_contest_edi(gchar *date, int edi){
     static char ss[1024];
     gint saveid=-1;
     gchar *remarks;
+    time_t now;
+    struct tm tm;
+    int nowdate_int, ctestdate_int;
     
     file = NULL;
     datedir = NULL;
     
+	ST_START;
     c=g_strdup_printf("%s/tucnak/%s/desc", getenv("HOME"), date);
     descfile = fopen(c, "rt+");
     if (!descfile){
-        g_snprintf(ss,1000, TEXT(T_CANT_OPEN_S),c);
+        g_snprintf(ss,1000, VTEXT(T_CANT_OPEN_S),c);
         g_free(c);
-        msg_box(term, NULL, TEXT(T_ERROR), AL_CENTER, ss, NULL, 1, TEXT(T_OK), NULL, B_ENTER | B_ESC);
+        msg_box(NULL, VTEXT(T_ERROR), AL_CENTER, ss, NULL, 1, VTEXT(T_OK), NULL, B_ENTER | B_ESC);
         goto x;
     }
     
     if (lockf(fileno(descfile), F_TLOCK, 0)){ 
         fclose(descfile); descfile=NULL;
-        g_snprintf(ss,1000, TEXT(T_CANT_LOCK_S),c);
+        g_snprintf(ss,1000, VTEXT(T_CANT_LOCK_S),c);
         g_free(c);
-        msg_box(term, NULL, TEXT(T_ERROR), AL_CENTER, ss, NULL, 1, TEXT(T_OK), NULL, B_ENTER | B_ESC);
+        msg_box(NULL, VTEXT(T_ERROR), AL_CENTER, ss, NULL, 1, VTEXT(T_OK), NULL, B_ENTER | B_ESC);
         goto x;
     }
     g_free(c);
@@ -826,34 +822,35 @@ void load_contest_edi(gchar *date, int edi){
    
     file = g_strdup_printf("%s/log", datedir);
     load_fifo_from_file(glog, file,0);
-    fifo_addfq(glog, TEXT(T_LOADING_CTEST_S), date); 
+    fifo_addfq(glog, VTEXT(T_LOADING_CTEST_S), date); 
     CONDGFREE(file);
 
     c=g_strdup_printf("%s/tucnak/%s/log", getenv("HOME"), date);
     logfile = fopen(c, "at");
     if (!logfile){
-        g_snprintf(ss,1000, TEXT(T_CANT_OPEN_S),c);
+        g_snprintf(ss,1000, VTEXT(T_CANT_OPEN_S),c);
         g_free(c);
-        msg_box(term, NULL, TEXT(T_ERROR), AL_CENTER, ss, NULL, 1, TEXT(T_OK), NULL, B_ENTER | B_ESC);
+        msg_box(NULL, VTEXT(T_ERROR), AL_CENTER, ss, NULL, 1, VTEXT(T_OK), NULL, B_ENTER | B_ESC);
         goto x;
     }
+    g_free(c);
     setvbuf(logfile, NULL, _IONBF, 0);
-    fprintf(logfile, TEXT(T_LOADING_CTEST_S), date); fprintf(logfile, "\n");
+    fprintf(logfile, VTEXT(T_LOADING_CTEST_S), date); fprintf(logfile, "\n");
     
     
     file = g_strdup_printf("%s/talk", datedir);
     load_fifo_from_file(gtalk, file,1);
-    fifo_addfq(gtalk, TEXT(T_LOADING_CTEST_S), date);
+    fifo_addfq(gtalk, VTEXT(T_LOADING_CTEST_S), date);
     CONDGFREE(file);
     
     file = g_strdup_printf("%s/sked", datedir);
     load_fifo_from_file(gsked, file,1);
-    fifo_addfq(gsked, TEXT(T_LOADING_CTEST_S), date);
+    fifo_addfq(gsked, VTEXT(T_LOADING_CTEST_S), date);
     CONDGFREE(file);
     
-    n = scandir(datedir, &namelist, select_enable_all, alphasort);
+    n = tucnak_scandir(datedir, &namelist, select_enable_all, tucnak_alphasort);
     if (n<=0) {
-        errbox(TEXT(T_NO_FILES), 0);
+        errbox(VTEXT(T_NO_FILES), 0);
         fclose(descfile); 
         fclose(logfile); 
         goto x;
@@ -900,7 +897,7 @@ void load_contest_edi(gchar *date, int edi){
             gchar *c;
             c = g_hash_table_lookup(hash, "band");
             if (!c) goto destroy;
-            tmpcb = get_config_band_by_bandchar(tolower(c[0]));
+            tmpcb = get_config_band_by_bandchar(lowcase(c[0]));
             if (!tmpcb) goto destroy;
             pband = tmpcb->pband;
             if (!pband){
@@ -912,13 +909,14 @@ destroy:;
         }
 
         if (find_band_by_pband(pband)){
-            log_addf(TEXT(T_BAND_EXIST_SS), pband, file);
+            log_addf(VTEXT(T_BAND_EXIST_SS), pband, file);
             goto destroy;
         }
 
         if (!ctest) {
             init_ctest();
             load_ctest_from_mem(ctest, datedir, hash);
+            maps_reload();
         }
         
         b = init_band(NULL, hash);
@@ -936,7 +934,6 @@ destroy:;
             CONDGFREE(b->remarks);
             b->remarks=remarks;
         }      
-        /*recalc_stats(b->stats, b);*/
 
         CONDGFREE(file);
         
@@ -990,13 +987,13 @@ destroy:;
     }
     
     if (!ctest) {
-        errbox(TEXT(T_NO_BANDS), 0);
+        errbox(VTEXT(T_NO_BANDS), 0);
         fclose(descfile); 
         fclose(logfile); 
         goto x;
     }
     if (ctest->bands->len==0){
-        errbox(TEXT(T_NO_BANDS), 0);
+        errbox(VTEXT(T_NO_BANDS), 0);
         free_ctest();
         goto x;
     }
@@ -1007,6 +1004,17 @@ destroy:;
     ctest->cdate = g_strdup(date);
     c = index(ctest->cdate, '.');
     if (c) *c='\0';
+
+    now = time(NULL);
+    gmtime_r(&now, &tm);
+    nowdate_int = (tm.tm_year+1900)*10000 + (tm.tm_mon+1)*100 + tm.tm_mday;
+    ctestdate_int = atoi(ctest->cdate);
+    if (/* nowdate_int >= ctestdate_int &&*/ nowdate_int <= ctestdate_int + 2) 
+        ctest->recording = 1;
+    else
+        ctest->recording = 0;
+    dbg("ctestdate=%d nowdate=%d recording=%d\n", ctestdate_int, nowdate_int, ctest->recording);
+
     
     if (!edi_serid_used){
         /* only for backward compatibility with version without ser_id in edi file */
@@ -1021,11 +1029,11 @@ destroy:;
     if (cfg->startband){
         struct band *b;
         b=find_band_by_pband(cfg->startband);
-        activate_band(gses, b); /* b==NULL is handled by function */
+        activate_band(b); /* b==NULL is handled by function */
     }
-#ifdef HAVE_SDL		
-    gfx_reload(gfx);  
-#endif
+/*#ifdef HAVE_SDL		
+    maps_reload(); dela se pri activate_band 
+#endif*/
     net_send_operator();
 
 x: 
@@ -1033,12 +1041,12 @@ x:
         for (i=0; i<n; i++) free(namelist[i]);
         free(namelist);
     }
-
     CONDGFREE(datedir);   
     CONDGFREE(file);   
+	ST_STOP;
 }
 
-void import_edi(struct session *ses, char *filename){
+void import_edi(void *xxx, char *filename){
     FILE *f;
     GHashTable *hash;
     GPtrArray *qsos;
@@ -1048,7 +1056,7 @@ void import_edi(struct session *ses, char *filename){
     
     f = fopen(filename, "rt");
     if (!f) {
-        errbox(TEXT(T_CANT_OPEN_S), errno, filename);
+        errbox(VTEXT(T_CANT_OPEN_S), errno, filename);
         return;
     }
     hash = load_edi_until(f, NULL, "^\\[qsorecords;");
@@ -1060,7 +1068,7 @@ void import_edi(struct session *ses, char *filename){
     
     band=find_band_by_pband(pband);
     if (!band){
-        errbox(TEXT(T_BAND_NOT_OPEN), 0);
+        errbox(VTEXT(T_BAND_NOT_OPEN), 0);
         goto destroy;
     }
 
@@ -1068,8 +1076,8 @@ void import_edi(struct session *ses, char *filename){
     add_qsos_from_mem(band, hash, qsos, ctest->cdate);
 destroy:;    
     destroy_string_hash(hash);  
-    redraw_later(ses->term);
+    redraw_later();
 }
 
-void import_swap(struct session *ses, char *filename){
+void import_swap(void *xxx, char *filename){
 }
