@@ -3,19 +3,9 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
@@ -47,9 +37,9 @@ struct itrm {
     char *orig_title;
 };
 
-void free_trm(struct itrm *);
-void in_kbd(struct itrm *);
-void in_sock(struct itrm *);
+void free_trm(union cba_t);
+void in_kbd(union cba_t);
+void in_sock(union cba_t);
 
 struct itrm *ditrm = NULL;
 
@@ -60,7 +50,7 @@ int is_blocked()
 
 void free_all_itrms()
 {
-    if (ditrm) free_trm(ditrm);
+    if (ditrm) free_trm((union cba_t)ditrm);
 }
 
 void write_ev_queue(struct itrm *itrm)
@@ -68,11 +58,20 @@ void write_ev_queue(struct itrm *itrm)
     int l;
     if (!itrm->eqlen) internal("event queue empty");
     if ((l = write(itrm->sock_out, itrm->ev_queue, itrm->eqlen > 128 ? 128 : itrm->eqlen)) == -1) {
-        free_trm(itrm);
+        free_trm((union cba_t)itrm);
         return;
     }
+#ifdef HAVE_SDL    
+    if (sdl){
+        LOCK(sdl->eventpipestate);
+        sdl->eventpipestate+=l;
+/*        dbg("u %d\n", sdl->eventpipestate);*/
+        UNLOCK(sdl->eventpipestate);
+    }
+#endif
+    
     memmove(itrm->ev_queue, itrm->ev_queue + l, itrm->eqlen -= l);
-    if (!itrm->eqlen) set_handlers(itrm->sock_out, get_handler(itrm->sock_out, H_READ), NULL, get_handler(itrm->sock_out, H_ERROR), get_handler(itrm->sock_out, H_DATA));
+    if (!itrm->eqlen) set_handlers(itrm->sock_out, get_handler(itrm->sock_out, H_READ), NULL, get_handler(itrm->sock_out, H_ERROR), (union cba_t)(void*)get_handler(itrm->sock_out, H_DATA));
 }
 
 void queue_event(struct itrm *itrm, char *data, int len)
@@ -81,19 +80,33 @@ void queue_event(struct itrm *itrm, char *data, int len)
     if (!len) return;
     if (!itrm->eqlen && can_write(itrm->sock_out) && (w = write(itrm->sock_out, data, len)) <= 0) {
         /*free_trm(itrm);*/
-        register_bottom_half((void (*)(void *))free_trm, itrm);
+        register_bottom_half((void (*)(union cba_t))free_trm, (union cba_t)itrm);
         return;
     }
+    
+#ifdef HAVE_SDL    
+    if (sdl){
+        LOCK(sdl->eventpipestate);
+        sdl->eventpipestate+=w;
+/*        dbg("q %d\n", sdl->eventpipestate);    */
+        UNLOCK(sdl->eventpipestate);
+    }
+#endif
+        
     if (w < len) {
         char *c;
         if (!(c = mem_realloc(itrm->ev_queue, itrm->eqlen + len - w))) {
-            free_trm(itrm);
+            free_trm((union cba_t)itrm);
             return;
         }
         itrm->ev_queue = c;
         memcpy(itrm->ev_queue + itrm->eqlen, data + w, len - w);
         itrm->eqlen += len - w;
-        set_handlers(itrm->sock_out, get_handler(itrm->sock_out, H_READ), (void (*)(void *))write_ev_queue, (void (*)(void *))free_trm, itrm);
+        set_handlers(itrm->sock_out, 
+                (void (*)(union cba_t))get_handler(itrm->sock_out, H_READ), 
+                (void (*)(union cba_t))write_ev_queue, 
+                (void (*)(union cba_t))free_trm, 
+                (union cba_t)itrm);
     }
 }
 
@@ -138,11 +151,19 @@ void send_term_sequence(int h,int flags)
     }
 }
 
-void resize_terminal()
+void resize_terminal(union cba_t cba)
 {
     struct event ev = { EV_RESIZE, 0, 0, 0 };
     int x, y;
-    if (get_terminal_size(ditrm->std_out, &x, &y)) return;
+#ifdef HAVE_SDL    
+    if (sdl){
+        if (sdl_get_terminal_size(ditrm->std_out, &x, &y)) return;
+    }
+    else
+#endif
+    {
+        if (term_get_terminal_size(ditrm->std_out, &x, &y)) return;
+    }
     ev.x = x;
     ev.y = y;
     queue_event(ditrm, (char *)&ev, sizeof(struct event));
@@ -164,14 +185,24 @@ int setraw(int fd, struct termios *p)
     return 0;
 }
 
-void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in, void *init_string, int init_len)
+void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in)
 {
     int x, y;
     struct itrm *itrm;
     struct event ev = { EV_INIT, 80, 24, 0 };
     char *ts;
-    int xwin;
-    if (get_terminal_size(ctl_in, &x, &y)) {
+    int xwin, ret;
+
+#ifdef HAVE_SDL    
+    if (sdl){
+        ret=sdl_get_terminal_size(ctl_in, &x, &y);
+    }
+    else
+#endif
+    {
+        ret=term_get_terminal_size(ctl_in, &x, &y);
+    }
+    if (ret){
         error("ERROR: could not get terminal size");
         return;
     }
@@ -187,9 +218,11 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
     itrm->tm = -1;
     itrm->ev_queue = DUMMY;
     itrm->eqlen = 0;
-    if (ctl_in >= 0) setraw(ctl_in, &itrm->t);
-    set_handlers(std_in, (void (*)(void *))in_kbd, NULL, (void (*)(void *))free_trm, itrm);
-    if (sock_in != std_out) set_handlers(sock_in, (void (*)(void *))in_sock, NULL, (void (*)(void *))free_trm, itrm);
+    if (ctl_in >= 0) {
+        setraw(ctl_in, &itrm->t);
+        set_handlers(std_in, in_kbd, NULL, free_trm, (union cba_t)itrm);
+    }
+    if (sock_in != std_out) set_handlers(sock_in, in_sock, NULL, free_trm, (union cba_t)itrm);
     ev.x = x;
     ev.y = y;
     handle_terminal_resize(ctl_in, resize_terminal);
@@ -198,51 +231,30 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
     itrm->flags = 0;
     if (!(ts = getenv("TERM"))) ts = "";
     if ((xwin & ENV_TWIN) && !strcmp(ts,"linux")) itrm->flags |= USE_TWIN_MOUSE;
-    if (strlen(ts) >= MAX_TERM_LEN) queue_event(itrm, ts, MAX_TERM_LEN);
-    else {
-        char *mm;
-        int ll = MAX_TERM_LEN - strlen(ts);
-        queue_event(itrm, ts, strlen(ts));
-        if (!(mm = mem_alloc(ll))) {
-            free_trm(itrm);
-            return;
-        }
-        memset(mm, 0, ll);
-        queue_event(itrm, mm, ll);
-        mem_free(mm);
+    
+#ifdef HAVE_SDL    
+    if (!sdl){ 
+        itrm->mouse_h = handle_mouse(0, (void (*)(void *, char *, int))queue_event, itrm);
     }
-    if (!(ts = get_cwd()) && !(ts = stracpy(""))) goto neni_pamet;
-    if (strlen(ts) >= MAX_CWD_LEN) queue_event(itrm, ts, MAX_CWD_LEN);
-    else {
-        char *mm;
-        int ll = MAX_CWD_LEN - strlen(ts);
-        queue_event(itrm, ts, strlen(ts));
-        if (!(mm = mem_alloc(ll))) {
-            free_trm(itrm);
-            return;
-        }
-        memset(mm, 0, ll);
-        queue_event(itrm, mm, ll);
-        mem_free(mm);
+    else
+#endif
+    {
+        itrm->mouse_h = NULL;
     }
-    mem_free(ts);
-    neni_pamet:
-    queue_event(itrm, (char *)&xwin, sizeof(int));
-    queue_event(itrm, (char *)&init_len, sizeof(int));
-    queue_event(itrm, (char *)init_string, init_len);
-    itrm->mouse_h = handle_mouse(0, (void (*)(void *, char *, int))queue_event, itrm);
     itrm->orig_title = get_window_title();
-    save_screen();
-    send_init_sequence(std_out,itrm->flags);
-    set_window_title_(PACKAGE " " VERSION);
+    if (std_out>=0){
+        save_screen();
+        send_init_sequence(std_out,itrm->flags);
+    }
+    set_window_title(PACKAGE_NAME " " PACKAGE_VERSION);
 }
 
-void unblock_itrm_x(void *h)
+void unblock_itrm_x(union cba_t cba)
 {
-    close_handle(h);
+    close_handle(cba);
     if (!ditrm) return;
     unblock_itrm(0);
-    resize_terminal();
+    resize_terminal(CBA0);
 }
 
 int unblock_itrm(int fd)
@@ -256,7 +268,7 @@ int unblock_itrm(int fd)
     if (itrm->ctl_in >= 0 && setraw(itrm->ctl_in, NULL)) return -1;
     itrm->blocked = 0;
     send_init_sequence(itrm->std_out,itrm->flags);
-    set_handlers(itrm->std_in, (void (*)(void *))in_kbd, NULL, (void (*)(void *))free_trm, itrm);
+    set_handlers(itrm->std_in, in_kbd, NULL, free_trm, (union cba_t)itrm);
     handle_terminal_resize(itrm->ctl_in, resize_terminal);
     unblock_stdin();
     return 0;
@@ -275,27 +287,37 @@ void block_itrm(int fd)
     unhandle_terminal_resize(itrm->ctl_in);
     send_term_sequence(itrm->std_out,itrm->flags);
     tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
-    set_handlers(itrm->std_in, NULL, NULL, (void (*)(void *))free_trm, itrm);
+    set_handlers(itrm->std_in, NULL, NULL, free_trm, (union cba_t)itrm);
 }
 
-void free_trm(struct itrm *itrm)
+void free_trm(union cba_t cba)
 {
+    struct itrm *itrm;
+
+    itrm=cba.itrm;
     if (!itrm) return;
-    set_window_title_(itrm->orig_title);
+    set_window_title(itrm->orig_title);
     if (itrm->orig_title) mem_free(itrm->orig_title), itrm->orig_title = NULL;
     unhandle_terminal_resize(itrm->ctl_in);
-    send_term_sequence(itrm->std_out,itrm->flags);
+    if (itrm->std_out>=0)  send_term_sequence(itrm->std_out,itrm->flags);
     tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
-    restore_screen();
+    if (itrm->std_out>=0) restore_screen();
     if (itrm->mouse_h) unhandle_mouse(itrm->mouse_h);
-    set_handlers(itrm->std_in, NULL, NULL, NULL, NULL);
-    set_handlers(itrm->sock_in, NULL, NULL, NULL, NULL);
-    set_handlers(itrm->std_out, NULL, NULL, NULL, NULL);
-    set_handlers(itrm->sock_out, NULL, NULL, NULL, NULL);
+    if (itrm->std_in>=0)   set_handlers(itrm->std_in, NULL, NULL, NULL, CBA0);
+    if (itrm->sock_in>=0)  set_handlers(itrm->sock_in, NULL, NULL, NULL, CBA0);
+    if (itrm->std_out>=0)  set_handlers(itrm->std_out, NULL, NULL, NULL, CBA0);
+    if (itrm->sock_out>=0) set_handlers(itrm->sock_out, NULL, NULL, NULL, CBA0);
     if (itrm->tm != -1) kill_timer(itrm->tm);
     mem_free(itrm->ev_queue);
     mem_free(itrm);
     if (itrm == ditrm) ditrm = NULL;
+     itrm=NULL;
+}
+
+void itrm_safe_abort(){
+    struct itrm *itrm = ditrm;
+    if (!itrm) return;
+    tcsetattr(itrm->ctl_in, TCSANOW, &itrm->t);
 }
 
 void resize_terminal_x(char *text)
@@ -307,14 +329,14 @@ void resize_terminal_x(char *text)
     x = atoi(text);
     y = atoi(p);
     resize_window(x, y);
-    resize_terminal();
+    resize_terminal(CBA0);
 }
 
 void dispatch_special(char *text)
 {
     switch (text[0]) {
         case TERM_FN_TITLE:
-            set_window_title_(text + 1);
+            set_window_title(text + 1);
             break;
         case TERM_FN_RESIZE:
             resize_terminal_x(text + 1);
@@ -328,16 +350,19 @@ char buf[OUT_BUF_SIZE];
 /*udefine RD ({ char cc; if (p < c) cc = buf[p++]; else if ((hard_read(itrm->sock_in, &cc, 1)) <= 0) goto fr; cc; })*/
 #define RD(xx) { char cc; if (p < c) cc = buf[p++]; else if ((hard_read(itrm->sock_in, &cc, 1)) <= 0) goto fr; xx = cc; }
 
-void in_sock(struct itrm *itrm)
+void in_sock(union cba_t cba)
 {
     char *path, *delete;
     int pl, dl;
     char ch;
     int fg;
     int c, i, p;
+    struct itrm *itrm;
+
+    itrm=cba.itrm;
     if ((c = read(itrm->sock_in, buf, OUT_BUF_SIZE)) <= 0) {
         fr:
-        free_trm(itrm);
+        free_trm((union cba_t)itrm);
         return;
     }
     qwerty:
@@ -394,10 +419,10 @@ void in_sock(struct itrm *itrm)
         }
         mem_free(param);
         if (fg == 1) {
-            set_handlers(blockh, (void (*)(void *))unblock_itrm_x, NULL, (void (*)(void *))unblock_itrm_x, (void *)(vint)blockh);
+            set_handlers(blockh, unblock_itrm_x, NULL, unblock_itrm_x, (union cba_t)blockh);
             /*block_itrm(itrm->ctl_in);*/
         } else {
-            set_handlers(blockh, close_handle, NULL, close_handle, (void *)(vint)blockh);
+            set_handlers(blockh, close_handle, NULL, close_handle, (union cba_t)blockh);
         }
     }
     to_je_ale_hnus:
@@ -464,9 +489,9 @@ struct key os2xtd[256] = {
 /* 64 */
 {KBD_F6,0},{KBD_F7,0},{KBD_F8,0},{KBD_F9,0},{KBD_F10,0},{0,0},{0,0},{KBD_HOME,0},
 /* 72 */
-{KBD_UP,0},{KBD_PAGE_UP,0},{'-',KBD_ALT},{KBD_LEFT,0},{'5',0},{KBD_RIGHT,0},{'+',KBD_ALT},{KBD_END,0},
+{KBD_UP,0},{KBD_PGUP,0},{'-',KBD_ALT},{KBD_LEFT,0},{'5',0},{KBD_RIGHT,0},{'+',KBD_ALT},{KBD_END,0},
 /* 80 */
-{KBD_DOWN,0},{KBD_PAGE_DOWN,0},{KBD_INS,0},{KBD_DEL,0},{0,0},{0,0},{0,0},{0,0},
+{KBD_DOWN,0},{KBD_PGDN,0},{KBD_INS,0},{KBD_DEL,0},{0,0},{0,0},{0,0},{0,0},
 /* 88 */
 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{KBD_F1,KBD_CTRL},{KBD_F2,KBD_CTRL},
 /* 96 */
@@ -474,19 +499,19 @@ struct key os2xtd[256] = {
 /* 104 */
 {KBD_F1,KBD_ALT},{KBD_F2,KBD_ALT},{KBD_F3,KBD_ALT},{KBD_F4,KBD_ALT},{KBD_F5,KBD_ALT},{KBD_F6,KBD_ALT},{KBD_F7,KBD_ALT},{KBD_F8,KBD_ALT},
 /* 112 */
-{KBD_F9,KBD_ALT},{KBD_F10,KBD_ALT},{0,0},{KBD_LEFT,KBD_CTRL},{KBD_RIGHT,KBD_CTRL},{KBD_END,KBD_CTRL},{KBD_PAGE_DOWN,KBD_CTRL},{KBD_HOME,KBD_CTRL},
+{KBD_F9,KBD_ALT},{KBD_F10,KBD_ALT},{0,0},{KBD_LEFT,KBD_CTRL},{KBD_RIGHT,KBD_CTRL},{KBD_END,KBD_CTRL},{KBD_PGDN,KBD_CTRL},{KBD_HOME,KBD_CTRL},
 /* 120 */
 {'1',KBD_ALT},{'2',KBD_ALT},{'3',KBD_ALT},{'4',KBD_ALT},{'5',KBD_ALT},{'6',KBD_ALT},{'7',KBD_ALT},{'8',KBD_ALT},
 /* 128 */
-{'9',KBD_ALT},{'0',KBD_ALT},{'-',KBD_ALT},{'=',KBD_ALT},{KBD_PAGE_UP,KBD_CTRL},{KBD_F11,0},{KBD_F12,0},{0,0},
+{'9',KBD_ALT},{'0',KBD_ALT},{'-',KBD_ALT},{'=',KBD_ALT},{KBD_PGUP,KBD_CTRL},{KBD_F11,0},{KBD_F12,0},{0,0},
 /* 136 */
 {0,0},{KBD_F11,KBD_CTRL},{KBD_F12,KBD_CTRL},{KBD_F11,KBD_ALT},{KBD_F12,KBD_ALT},{KBD_UP,KBD_CTRL},{'-',KBD_CTRL},{'5',KBD_CTRL},
 /* 144 */
 {'+',KBD_CTRL},{KBD_DOWN,KBD_CTRL},{KBD_INS,KBD_CTRL},{KBD_DEL,KBD_CTRL},{KBD_TAB,KBD_CTRL},{0,0},{0,0},{KBD_HOME,KBD_ALT},
 /* 152 */
-{KBD_UP,KBD_ALT},{KBD_PAGE_UP,KBD_ALT},{0,0},{KBD_LEFT,KBD_ALT},{0,0},{KBD_RIGHT,KBD_ALT},{0,0},{KBD_END,KBD_ALT},
+{KBD_UP,KBD_ALT},{KBD_PGUP,KBD_ALT},{0,0},{KBD_LEFT,KBD_ALT},{0,0},{KBD_RIGHT,KBD_ALT},{0,0},{KBD_END,KBD_ALT},
 /* 160 */
-{KBD_DOWN,KBD_ALT},{KBD_PAGE_DOWN,KBD_ALT},{KBD_INS,KBD_ALT},{KBD_DEL,KBD_ALT},{0,0},{KBD_TAB,KBD_ALT},{KBD_ENTER,KBD_ALT},{0,0},
+{KBD_DOWN,KBD_ALT},{KBD_PGDN,KBD_ALT},{KBD_INS,KBD_ALT},{KBD_DEL,KBD_ALT},{0,0},{KBD_TAB,KBD_ALT},{KBD_ENTER,KBD_ALT},{0,0},
 /* 168 */
 {0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},{0,0},
 /* 176 */
@@ -530,8 +555,8 @@ int process_queue(struct itrm *itrm)
                 case 'e': ev.x = KBD_END; break;
                 case 'H':
                 case 0: ev.x = KBD_HOME; break;
-                case 'I': ev.x = KBD_PAGE_UP; break;
-                case 'G': ev.x = KBD_PAGE_DOWN; break;
+                case 'I': ev.x = KBD_PGUP; break;
+                case 'G': ev.x = KBD_PGDN; break;
 
                  /* BSD cons25 */         
  /*               case 'M': ev.x = KBD_F1; break;*/
@@ -552,8 +577,8 @@ int process_queue(struct itrm *itrm)
                     case 247: ev.x = KBD_INS; break;
                     case 214: ev.x = KBD_HOME; break;
                     case 220: ev.x = KBD_END; break;
-                    case 216: ev.x = KBD_PAGE_UP; break;
-                    case 222: ev.x = KBD_PAGE_DOWN; break;
+                    case 216: ev.x = KBD_PGUP; break;
+                    case 222: ev.x = KBD_PGDN; break;
                     case 249: ev.x = KBD_DEL; break;
                     } break;
                 case '~': switch (v) {
@@ -561,8 +586,8 @@ int process_queue(struct itrm *itrm)
                     case 2: ev.x = KBD_INS; break;
                     case 3: ev.x = KBD_DEL; break;
                     case 4: ev.x = KBD_END; break;
-                    case 5: ev.x = KBD_PAGE_UP; break;
-                    case 6: ev.x = KBD_PAGE_DOWN; break;
+                    case 5: ev.x = KBD_PGUP; break;
+                    case 6: ev.x = KBD_PGDN; break;
                     case 7: ev.x = KBD_HOME; break;
                     case 8: ev.x = KBD_END; break;
                     case 11: ev.x = KBD_F1; break;
@@ -662,10 +687,10 @@ int process_queue(struct itrm *itrm)
     ev.x = itrm->kqueue[0];
     l2:
     /*if (ev.x == 1) ev.x = KBD_HOME;
-    if (ev.x == 2) ev.x = KBD_PAGE_UP;
+    if (ev.x == 2) ev.x = KBD_PGUP;
     if (ev.x == 4) ev.x = KBD_DEL;
     if (ev.x == 5) ev.x = KBD_END;
-    if (ev.x == 6) ev.x = KBD_PAGE_DOWN;*/
+    if (ev.x == 6) ev.x = KBD_PGDN;*/
     if (ev.x == 8) ev.x = KBD_BS;
     if (ev.x == 9) ev.x = KBD_TAB;
     if (ev.x == 10) ev.x = KBD_ENTER, ev.y = KBD_CTRL;
@@ -688,24 +713,27 @@ int process_queue(struct itrm *itrm)
         memmove(itrm->kqueue, itrm->kqueue + el, itrm->qlen -= el);
     }
     end:
-    if (itrm->qlen < IN_BUF_SIZE) set_handlers(itrm->std_in, (void (*)(void *))in_kbd, NULL, (void (*)(void *))free_trm, itrm);
+    if (itrm->qlen < IN_BUF_SIZE) set_handlers(itrm->std_in, in_kbd, NULL, free_trm, (union cba_t)itrm);
     return el;
     ret:
-    itrm->tm = install_timer_(ESC_TIMEOUT, (void (*)(void *))kbd_timeout, itrm);
+    itrm->tm = install_timer(ESC_TIMEOUT, (void (*)(union cba_t))kbd_timeout, (union cba_t)itrm);
     return 0;
 }
 
-void in_kbd(struct itrm *itrm)
+void in_kbd(union cba_t cba)
 {
     int r;
+    struct itrm *itrm;
+
+    itrm=cba.itrm;
     if (itrm->tm != -1) kill_timer(itrm->tm), itrm->tm = -1;
     if (itrm->qlen >= IN_BUF_SIZE) {
-        set_handlers(itrm->std_in, NULL, NULL, (void (*)(void *))free_trm, itrm);
+        set_handlers(itrm->std_in, NULL, NULL, free_trm, (union cba_t)itrm);
         while (process_queue(itrm));
         return;
     }
     if ((r = read(itrm->std_in, itrm->kqueue + itrm->qlen, IN_BUF_SIZE - itrm->qlen)) <= 0) {
-        free_trm(itrm);
+        free_trm((union cba_t)itrm);
         return;
     }
     if ((itrm->qlen += r) > IN_BUF_SIZE) {
