@@ -2,24 +2,17 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
 #include "header.h"
 
+/*#include <unistd.h>
+#include <sys/types.h>*/
+int fileno(FILE *stream);
 
 char tname[MAX_STR_LEN],pcall[MAX_STR_LEN],pclub[MAX_STR_LEN];
 char pwwlo[MAX_STR_LEN],pexch[MAX_STR_LEN],tdate[MAX_STR_LEN];
@@ -32,7 +25,7 @@ char excbonu_str[MAX_STR_LEN], excmult_str[MAX_STR_LEN];
 char dxcbonu_str[MAX_STR_LEN], dxcmult_str[MAX_STR_LEN];
 int rstused;
 char default_rst[MAX_STR_LEN],default_rs[MAX_STR_LEN];
-int modesep;
+char operator[MAX_STR_LEN];
 
 /*int qsop_method=1, total_method=1;*/
 
@@ -191,13 +184,12 @@ void band_fn(struct dialog_data *dlg)
 
 
                                                    
-
 /* xxx is char *band */
-void band_settings(struct terminal *term, void *xxx, void *from_ctest)
+void band_settings(union cba_t cba, int from_ctest)
 {
     struct dialog *d;
     int i,found;
-    char *pband=(char *)xxx;
+    char *pband=cba.charp;
     struct config_band *confb=NULL;
     struct band *band=NULL;
     static char ss[1024];
@@ -262,11 +254,11 @@ void band_settings(struct terminal *term, void *xxx, void *from_ctest)
     d->title = ss;
     d->fn = band_fn;
     if (confb){
-        g_snprintf(ss,1000,TEXT(T_BAND_SETTINGS_SC), upcase(confb->bandchar),pband);
+        g_snprintf(ss,1000,VTEXT(T_BAND_SETTINGS_SC), upcase(confb->bandchar),pband);
         d->refresh = (void (*)(void *))refresh_band_confb;
         d->refresh_data = confb;
     }else{
-        g_snprintf(ss,1000,TEXT(T_BAND_SETTINGS_SC), upcase(band->bandchar),pband);
+        g_snprintf(ss,1000,VTEXT(T_BAND_SETTINGS_SC), upcase(band->bandchar),pband);
         d->refresh = (void (*)(void *))refresh_band;
         d->refresh_data = band;
     }
@@ -343,27 +335,27 @@ void band_settings(struct terminal *term, void *xxx, void *from_ctest)
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
-    d->items[i].text = TEXT(T_OK);
+    d->items[i].text = VTEXT(T_OK);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
-    d->items[i].text = TEXT(T_CANCEL);
+    d->items[i].text = VTEXT(T_CANCEL);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
                                
 
 int dlg_band_settings_confb(struct dialog_data *dlg, struct dialog_item_data *di){
     /*dbg("dlg_band_settings %p %p \n",dlg,di);*/
-    band_settings(dlg->win->term, di->item->text, NULL); 
+    band_settings((union cba_t)di->item->text, 0); 
     return 0;
 }
 
 int dlg_band_settings(struct dialog_data *dlg, struct dialog_item_data *di){
     /*dbg("dlg_band_settings %p %p \n",dlg,di);*/
-    band_settings(dlg->win->term, di->item->text, (void*)43); 
+    band_settings((union cba_t)di->item->text, 1); 
     return 0;
 }
 
@@ -404,7 +396,6 @@ char *responsible_op_msg[] = {
 };
 void responsible_op_fn(struct dialog_data *dlg)
 {
-    struct terminal *term = dlg->win->term;
     int max = 0, min = 0;
     int w, rw;
     int y = -1;
@@ -477,7 +468,7 @@ void responsible_op_fn(struct dialog_data *dlg)
 
 
 
-void responsible_op(struct terminal *term, void *xxx, void *yyy)
+void responsible_op(void)
 {
     struct dialog *d;
     int i;
@@ -495,7 +486,7 @@ void responsible_op(struct terminal *term, void *xxx, void *yyy)
 
     if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
-    d->title = TEXT(T_RESPOP);
+    d->title = VTEXT(T_RESPOP);
     d->fn = responsible_op_fn;
     d->refresh = (void (*)(void *))refresh_responsible_op;
     
@@ -539,25 +530,25 @@ void responsible_op(struct terminal *term, void *xxx, void *yyy)
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
-    d->items[i].text = TEXT(T_OK);
+    d->items[i].text = VTEXT(T_OK);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
-    d->items[i].text = TEXT(T_CANCEL);
+    d->items[i].text = VTEXT(T_CANCEL);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
                                
 
 int dlg_responsoble_op(struct dialog_data *dlg, struct dialog_item_data *di){
-    responsible_op(dlg->win->term, NULL, NULL); 
+    responsible_op(); 
     return 0;
 }
 
-void menu_responsible_op(struct terminal *term, void *xxx, struct session *ses){
-    responsible_op(term, NULL, NULL); 
+void menu_responsible_op(union cba_t cba){
+    responsible_op(); 
 } 
 
 /******************** QSOP METHOD, TOTAL METHOD  ******************/
@@ -590,15 +581,15 @@ char *total_method_msg[]={
 };
 
 
-void qsop_method_func (struct terminal *term, void *ip, void *d){
+void qsop_method_func (union cba_t cba){
     int active;
 
-    active=(vint)ip;
+    active=cba.int_;
     if (active<1 || active>MAX_QSOP_METHOD) return;
     qsop_method = active;
     /*safe_strncpy0(qsop_method_str, qsop_method_msg[qsop_method], MAX_STR_LEN);*/
-    safe_strncpy0(qsop_method_str,_(CTEXT(T_QP0+qsop_method),term),MAX_STR_LEN);
-    redraw_later(term);
+    safe_strncpy0(qsop_method_str,_(CTEXT(T_QP0+qsop_method)),MAX_STR_LEN);
+    redraw_later();
 }
 
 int dlg_qsop_method(struct dialog_data *dlg, struct dialog_item_data *di){
@@ -607,24 +598,23 @@ int dlg_qsop_method(struct dialog_data *dlg, struct dialog_item_data *di){
     
     if (!(mi = new_menu(1))) return 0;
     for (i = 1; i <= MAX_QSOP_METHOD; i++) {
-        add_to_menu(&mi, qsop_method_msg[i], "", "", MENU_FUNC qsop_method_func, (void *)(vint)i, 0);
+        add_to_menu(&mi, qsop_method_msg[i], "", "", MENU_FUNC qsop_method_func, (union cba_t)i, 0);
     }
     sel = qsop_method-1;
     if (sel < 0) sel = 0;
     if (sel>=MAX_QSOP_METHOD) sel=0;
-    do_menu_selected(term, mi, (void *)(vint)qsop_method, sel);
+    do_menu_selected(mi, (union cba_t)qsop_method, sel);
     return 0;
 }
 
-void total_method_func (struct terminal *term, void *ip, void *d){
+void total_method_func (union cba_t cba){
     int active;
 
-    active=(vint)ip;
+    active=cba.int_;
     if (active<1 || active>MAX_TOTAL_METHOD) return;
     total_method = active;
-/*    safe_strncpy0(total_method_str, total_method_msg[total_method], MAX_STR_LEN);*/
-    safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method),term),MAX_STR_LEN);
-    redraw_later(term);
+    safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method)),MAX_STR_LEN);
+    redraw_later();
     return;
 }
 
@@ -634,37 +624,38 @@ int dlg_total_method(struct dialog_data *dlg, struct dialog_item_data *di){
     
     if (!(mi = new_menu(1))) return 0;
     for (i = 1; i <= MAX_TOTAL_METHOD; i++) {
-        add_to_menu(&mi, total_method_msg[i], "", "", MENU_FUNC total_method_func, (void *)(vint)i, 0);
+        add_to_menu(&mi, total_method_msg[i], "", "", MENU_FUNC total_method_func, (union cba_t)i, 0);
     }
     sel = total_method-1;
     if (sel < 0) sel = 0;
     if (sel>=MAX_TOTAL_METHOD) sel=0;
-    do_menu_selected(term, mi, (void *)(vint)total_method, sel);
+    do_menu_selected(mi, (union cba_t)total_method, sel);
     return 0;
 }
 
 
 /******************** WIZZ *******************************************/
 
-void wizz_func (struct terminal *term, void *ip, void *d){
+void wizz_func (union cba_t cba){
     int active;
     struct wizz_item *wi;
     time_t now;
-    struct tm *utc;
+    struct tm utc;
 
-    active=(vint)ip;
+    active=cba.int_;
     if (active<0 || active>=wizz->items->len) return;
     wi = get_wizz(wizz, active);
 
     time(&now);
-    utc=gmtime(&now);
+    gmtime_r(&now, &utc);
 
     safe_strncpy0(tname, wi->tname,100);
     safe_strncpy0(pcall, cfg->pcall, 20);uc(pcall);
     safe_strncpy0(pclub, cfg->pclub, 20);uc(pclub);
+    safe_strncpy0(operator, cfg->operator, 20);uc(operator);
     safe_strncpy0(pwwlo, cfg->pwwlo, 9); uc(pwwlo);
     safe_strncpy0(pexch, wi->pexch, 9); uc(pexch);
-    g_snprintf(tdate, MAX_STR_LEN, "%d%02d%02d", 1900+utc->tm_year, 1+utc->tm_mon, utc->tm_mday);
+    g_snprintf(tdate, MAX_STR_LEN, "%d%02d%02d", 1900+utc.tm_year, 1+utc.tm_mon, utc.tm_mday);
     qsoused=wi->qsoused;
     g_snprintf(qsomult_str,10, "%d", wi->qsomult);
     wwlused=wi->wwlused;
@@ -683,40 +674,40 @@ void wizz_func (struct terminal *term, void *ip, void *d){
     
     qsop_method=wi->qsop_method;
     /*safe_strncpy0(qsop_method_str,qsop_method_msg[qsop_method],MAX_STR_LEN);*/
-    safe_strncpy0(qsop_method_str,_(CTEXT(T_QP0+qsop_method),term),MAX_STR_LEN);
+    safe_strncpy0(qsop_method_str,_(CTEXT(T_QP0+qsop_method)),MAX_STR_LEN);
     total_method=wi->total_method;
-    safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method),term),MAX_STR_LEN);
+    safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method)),MAX_STR_LEN);
     /*safe_strncpy0(total_method_str,total_method_msg[total_method],MAX_STR_LEN);*/
     
     /* wwltype */
     /* minqsop, maxqsop */
     
-    contest_options1(term, TEXT(T_NEW_CTEST), NULL);
+    contest_options1(VTEXT(T_NEW_CTEST), 0);
 }
 
 
 struct menu_item no_wizz[] = {
-    {CTEXT(T_NO_CTESTS), "", M_BAR, NULL, NULL, 0, 0},
-    {NULL, NULL, 0, NULL, NULL, 0, 0}
+    {CTEXT(T_NO_CTESTS), "", M_BAR, NULL, {NULL}, 0, 0},
+    {NULL, NULL, 0, NULL, {NULL}, 0, 0}
 };
 
 
-void menu_wizz(struct terminal *term, void *xxx, void *yyy){
+void menu_wizz(union cba_t cba){
     int i;
     struct menu_item *mi;
     struct wizz_item *wi;
     
     if (!wizz->items->len){
-        do_menu_(term, no_wizz, gses);
+        do_menu(no_wizz, CBA0);
         return;
     }
     
     if (!(mi = new_menu(1))) return;
     for (i = 0; i < wizz->items->len; i++) {
         wi = get_wizz(wizz, i);
-        add_to_menu(&mi, wi->tname, "", "", MENU_FUNC wizz_func, (void *)(vint)i, 0);
+        add_to_menu(&mi, wi->tname, "", "", MENU_FUNC wizz_func, (union cba_t)i, 0);
     }
-    do_menu_selected(term, mi, NULL, 0);
+    do_menu_selected(mi, CBA0, 0);
 }
 
 /******************** CONTEST OPTIONS 1 *******************************/
@@ -766,7 +757,7 @@ void update_desc(struct contest *ctest){
         setvbuf(ctest->descfile, NULL, _IONBF, 0);
         fseek(ctest->descfile, 0, SEEK_SET);
         if (ftruncate(fileno(ctest->descfile), 0)){
-            errbox(TEXT(T_CANT_TRUNC), errno);
+            errbox(VTEXT(T_CANT_TRUNC), errno);
             return;
         }
         fprintf(ctest->descfile,"%s %s %s\n", ctest->cdate, uc(pcall), ctest->tname);    
@@ -777,19 +768,23 @@ void update_desc(struct contest *ctest){
 void refresh_contest_options1(void *xxx)
 {
     int isnew=0;
+    int updloc;
     /*dbg("refresh_contest_options1\n");*/
 
     update_desc(ctest);
     
     if (!ctest) isnew=1;
+    updloc=strcmp(cfg->pwwlo, pwwlo);
     
     STORE_STR_FS_UC(cfg,pcall); 
     STORE_STR_FS_UC(cfg,pclub);
+    /*STORE_STR_FS_UC(cfg,operator);*/
     STORE_STR_FS_UC(cfg,pwwlo);
     STORE_STR_FS   (cfg,default_rst);
     STORE_STR_FS   (cfg,default_rs);
     
     if (init_ctest()) return;
+    if (ctest->pwwlo) updloc=strcasecmp(ctest->pwwlo, pwwlo);
     
     STORE_STR_FS   (ctest,tname);
     STORE_STR_FS_UC(ctest,pcall);
@@ -824,16 +819,13 @@ void refresh_contest_options1(void *xxx)
     STORE_INT (ctest,rstused);
     STORE_INT (ctest,qsop_method);
     STORE_INT (ctest,total_method);
-    STORE_INT (ctest,modesep);
     
-    set_ctest_title(term);
+    set_ctest_title();
 
     if (!isnew){               /* contest options from menu */
         gchar *descfile;
         
-        recalc_all_qrbqtf(ctest);
-        recalc_all_stats(ctest);
-        save_all_bands_txt(0);
+        /*save_all_bands_txt(0);*/
 
         descfile=g_strdup_printf("%s/desc", ctest->directory);
         save_desc_to_file(NULL);
@@ -841,10 +833,15 @@ void refresh_contest_options1(void *xxx)
 
         net_send_ac();
         net_send_operator();
-#ifdef HAVE_SDL     
-        gfx_update_qth();
-        invalidate_cache(aband);
-#endif      
+#ifdef HAVE_SDL
+        if (updloc){
+            recalc_all_qrbqtf(ctest);
+            recalc_all_stats(ctest);
+            map_recalc_cors();
+            maps_reload();
+        }
+#endif   
+        save_all_bands_txt(0);
         return;
     }
     
@@ -859,7 +856,7 @@ void refresh_contest_options1(void *xxx)
     if (cfg->startband){
         struct band *b;
         b=find_band_by_pband(cfg->startband);
-        activate_band(gses, b); /* b==NULL is handled by function */
+        activate_band(b); /* b==NULL is handled by function */
     }
     
 }
@@ -886,7 +883,6 @@ char *contest_options1_msg[] = {
     CTEXT(T_DEFRST), 
     CTEXT(T_QP_CALC),
     CTEXT(T_TO_CALC),
-    CTEXT(T_MODESEP),
     "","","","","","","","","","","","","","","","","","","","","","","","","","",
     "", /* OK */        /* 21 */
     "", /* Cancel */
@@ -902,7 +898,7 @@ void contest_options1_fn(struct dialog_data *dlg)
     int firstbandbutt, numofbands;
 
     /*dbg("contest_options1_fn()\n");*/
-    firstbandbutt=22;
+    firstbandbutt=21;
     for (numofbands=0; numofbands<26; numofbands++ ) {
         if (! dlg->items[firstbandbutt+numofbands].item) break;
         if (dlg->items[firstbandbutt+numofbands].item->type!= D_BUTTON) break;
@@ -932,9 +928,6 @@ void contest_options1_fn(struct dialog_data *dlg)
     max_group_width(term, contest_options1_msg +19, dlg->items +19, 2, &max);
     min_group_width(term, contest_options1_msg +19, dlg->items +19, 2, &min);
     
-    max_group_width(term, contest_options1_msg + 21, dlg->items + 21, 1, &max);
-    min_group_width(term, contest_options1_msg + 21, dlg->items + 21, 1, &min);
-
     max_group_width(term, contest_options1_msg +firstbandbutt, dlg->items +firstbandbutt, numofbands, &max);
     min_group_width(term, contest_options1_msg +firstbandbutt, dlg->items +firstbandbutt, numofbands, &min);
     
@@ -962,7 +955,6 @@ void contest_options1_fn(struct dialog_data *dlg)
     dlg_format_group(NULL, term, contest_options1_msg +14, dlg->items +14, 2, 0, &y, w, &rw);
     dlg_format_group(NULL, term, contest_options1_msg +16, dlg->items +16, 3, 0, &y, w, &rw);
     dlg_format_group1(NULL, term, contest_options1_msg+19, dlg->items +19, 2, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, contest_options1_msg +21, dlg->items +21, 1, 0, &y, w, &rw);
     y++;
     dlg_format_group1(NULL, term, contest_options1_msg +firstbandbutt, dlg->items +firstbandbutt, numofbands, 0, &y, w, &rw);
     y++;
@@ -988,7 +980,6 @@ void contest_options1_fn(struct dialog_data *dlg)
     dlg_format_group(term, term, contest_options1_msg +14, dlg->items +14, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group(term, term, contest_options1_msg +16, dlg->items +16, 3, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group1(term, term, contest_options1_msg+19, dlg->items +19, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, contest_options1_msg +21, dlg->items +21, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     y++;
     dlg_format_group1(term, term, contest_options1_msg +firstbandbutt, dlg->items +firstbandbutt, numofbands, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     y++;
@@ -996,7 +987,7 @@ void contest_options1_fn(struct dialog_data *dlg)
 }
 
     
-void contest_options1(struct terminal *term, char *title, void *from_ctest)
+void contest_options1(char *title, int from_ctest)
 {
     struct dialog *d;
     int i,j;
@@ -1025,6 +1016,11 @@ void contest_options1(struct terminal *term, char *title, void *from_ctest)
     d->items[i].data = pclub;
     d->items[i].maxl = 15;
     
+    /*d->items[++i].type = D_FIELD;
+    d->items[i].dlen = 20;
+    d->items[i].data = operator;
+    d->items[i].maxl = 15; */
+    
     d->items[++i].type = D_FIELD;
     d->items[i].dlen = 9;
     d->items[i].data = pwwlo;
@@ -1133,12 +1129,6 @@ void contest_options1(struct terminal *term, char *title, void *from_ctest)
     d->items[i].fn   = dlg_total_method;
     d->items[i].text = total_method_str;
 
-    d->items[++i].type = D_CHECKBOX;
-    d->items[i].gid = 0;
-    d->items[i].gnum = 1;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data=(char *)&modesep;
-    
 
     /*    dbg("firstband=%d\n",i+1);*/
     if (!from_ctest){
@@ -1174,35 +1164,35 @@ void contest_options1(struct terminal *term, char *title, void *from_ctest)
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
-    d->items[i].text = TEXT(T_OK);
+    d->items[i].text = VTEXT(T_OK);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
-    d->items[i].text = TEXT(T_CANCEL);
+    d->items[i].text = VTEXT(T_CANCEL);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = 0;                         
     d->items[i].fn = dlg_edi_prop;
-    d->items[i].text = TEXT(T_EDI_PROP);
+    d->items[i].text = VTEXT(T_EDI_PROP);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = 0;
     d->items[i].fn = dlg_responsoble_op;
-    d->items[i].text = TEXT(T_RESPOP2);
+    d->items[i].text = VTEXT(T_RESPOP2);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
 
 
-void contest_options1_from_menu(struct terminal *term, void *xxx, void *yyy)
+void contest_options1_from_menu(union cba_t cba)
 {
     time_t now;
-    struct tm *utc;
+    struct tm utc;
 
     time(&now);
-    utc=gmtime(&now);
+    gmtime_r(&now, &utc);
 
     /*dbg("contest_options1\n");*/
     safe_strncpy0(tname,"",100);
@@ -1210,7 +1200,7 @@ void contest_options1_from_menu(struct terminal *term, void *xxx, void *yyy)
     safe_strncpy0(pclub, cfg->pclub, 20);uc(pclub);
     safe_strncpy0(pwwlo, cfg->pwwlo, 9); uc(pwwlo);
     safe_strncpy0(pexch, cfg->pexch, 9); uc(pexch);
-    g_snprintf(tdate, MAX_STR_LEN, "%d%02d%02d", 1900+utc->tm_year, 1+utc->tm_mon, utc->tm_mday);
+    g_snprintf(tdate, MAX_STR_LEN, "%d%02d%02d", 1900+utc.tm_year, 1+utc.tm_mon, utc.tm_mday);
     qsoused=1;
     safe_strncpy(qsomult_str,"1", 10);
     wwlused=1;
@@ -1225,32 +1215,24 @@ void contest_options1_from_menu(struct terminal *term, void *xxx, void *yyy)
     safe_strncpy0(default_rs, cfg->default_rs, 5);    uc(default_rs);
     safe_strncpy0(default_rst, cfg->default_rst, 5);  uc(default_rst);
     qsop_method=cfg->qsop_method % MAX_QSOP_METHOD;
-    safe_strncpy0(qsop_method_str,_(CTEXT(T_QP0+qsop_method),term),MAX_STR_LEN);
+    safe_strncpy0(qsop_method_str,_(CTEXT(T_QP0+qsop_method)),MAX_STR_LEN);
     /*safe_strncpy0(qsop_method_str,qsop_method_msg[qsop_method],MAX_STR_LEN);*/
     total_method=cfg->total_method % MAX_TOTAL_METHOD;
-    safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method),term),MAX_STR_LEN);
+    safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method)),MAX_STR_LEN);
     /*safe_strncpy0(total_method_str,total_method_msg[total_method],MAX_STR_LEN);*/
-    modesep=0;
    
-    contest_options1(term, TEXT(T_NEW_CTEST), NULL);
+    contest_options1(VTEXT(T_NEW_CTEST), 0);
 }
                                
 
-void contest_options1_from_ctest(struct terminal *term, void *xxx, void *yyy)
+void contest_options1_from_ctest(union cba_t cba)
 {
-    time_t now;
-    struct tm *utc;
-
-    time(&now);
-    utc=gmtime(&now);
-
     /*dbg("contest_options1\n");*/
     safe_strncpy0(tname, ctest->tname,100);
     safe_strncpy0(pcall, ctest->pcall, 20);uc(pcall);
     safe_strncpy0(pclub, ctest->pclub, 20);uc(pclub);
     safe_strncpy0(pwwlo, ctest->pwwlo, 9); uc(pwwlo);
     safe_strncpy0(pexch, ctest->pexch, 9); uc(pexch);
-/*  g_snprintf(tdate, MAX_STR_LEN, "%d%02d%02d", 1900+utc->tm_year, 1+utc->tm_mon, utc->tm_mday);*/
     qsoused=ctest->qsoused;
     g_snprintf(qsomult_str,10, "%d", ctest->qsomult);
     wwlused=ctest->wwlused;
@@ -1266,17 +1248,16 @@ void contest_options1_from_ctest(struct terminal *term, void *xxx, void *yyy)
     safe_strncpy0(default_rst, cfg->default_rst, 5);  uc(default_rst);
     
     qsop_method=ctest->qsop_method;
-    safe_strncpy0(qsop_method_str,_(CTEXT(T_QP0+qsop_method),term),MAX_STR_LEN);
+    safe_strncpy0(qsop_method_str,_(CTEXT(T_QP0+qsop_method)),MAX_STR_LEN);
     /*safe_strncpy0(qsop_method_str,qsop_method_msg[qsop_method],MAX_STR_LEN);*/
     total_method=ctest->total_method;
-    safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method),term),MAX_STR_LEN);
+    safe_strncpy0(total_method_str,_(CTEXT(T_TP0+total_method)),MAX_STR_LEN);
     /*safe_strncpy0(total_method_str,total_method_msg[total_method],MAX_STR_LEN);*/
-    modesep=ctest->modesep;
     
-    contest_options1(term, TEXT(T_CONTEST_OP), (void *)42);
-#ifdef HAVE_SDL 
-    gfx_update_qth();
-#endif  
+    contest_options1(VTEXT(T_CONTEST_OP), 1);
+/*#ifdef HAVE_SDL 
+    maps_reload();
+#endif  */
 }
                                
 
@@ -1288,33 +1269,51 @@ char pwwlo[MAX_STR_LEN],pexch[MAX_STR_LEN];
 char default_rst[MAX_STR_LEN],default_rs[MAX_STR_LEN];
 */  
 char padr1[MAX_STR_LEN],padr2[MAX_STR_LEN];
+int global_operator;
 
 void refresh_contest_def(void *xxx)
 {
+    int updloc;
 /*    dbg("refresh_contest_def\n");*/
-    STORE_STR(cfg,pcall);
-    STORE_STR(cfg,pclub);
-    STORE_STR(cfg,pwwlo);
+    updloc=strcmp(cfg->pwwlo, pwwlo);
+    STORE_STR_UC(cfg,pcall);
+    STORE_STR_UC(cfg,pclub);
+    STORE_STR_UC(cfg,operator);
+    STORE_INT(cfg, global_operator);
+    STORE_STR_UC(cfg,pwwlo);
     STORE_STR(cfg,padr1);
     STORE_STR(cfg,padr2);
-    STORE_STR(cfg,pexch);
-    STORE_STR(cfg,default_rs);
-    STORE_STR(cfg,default_rst);
+    STORE_STR_UC(cfg,pexch);
+    STORE_STR_UC(cfg,default_rs);
+    STORE_STR_UC(cfg,default_rst);
 #ifdef HAVE_SDL
-    gfx_update_qth();
+    if (updloc) maps_reload();
 #endif
+	if (first_contest_def){
+		if (strlen(cfg->pcall)==0 || strlen(cfg->pwwlo)==0){
+/*			register_bottom_half(contest_def, CBA0);*/
+            register_bottom_half(destroy_terminal, CBA0);
+
+		}else{
+			first_contest_def=0;
+			menu_save_rc(CBA0);
+		}
+	}
+    net_send_operator();
 }
 
 char *contest_def_msg[] = {
     CTEXT(T_CALLSIGN2),  /* 0 */
     CTEXT(T_CLUB),
+    CTEXT(T_OPERATOR),
+    CTEXT(T_GLOBAL_OPERATOR2),
     CTEXT(T_YOURWWL),
-    CTEXT(T_PADR1),
-    CTEXT(T_PADR2),
-    CTEXT(T_YOUREXC),  /* 5 */
-    CTEXT(T_RS),
-    CTEXT(T_RST),  
-    "", /* OK */        /* 8 */
+    CTEXT(T_PADR1),    /* 5 */  
+    CTEXT(T_PADR2),              
+    CTEXT(T_YOUREXC),            
+    CTEXT(T_RS),       
+    CTEXT(T_RST),       /* 9 */   
+    "", /* OK */       
     "", /* Cancel */
 };
 void contest_def_fn(struct dialog_data *dlg)
@@ -1328,21 +1327,23 @@ void contest_def_fn(struct dialog_data *dlg)
     min_group_width(term, contest_def_msg + 0, dlg->items + 0, 1, &min);
     max_group_width(term, contest_def_msg + 1, dlg->items + 1, 1, &max);
     min_group_width(term, contest_def_msg + 1, dlg->items + 1, 1, &min);
-    max_group_width(term, contest_def_msg + 2, dlg->items + 2, 1, &max);
-    min_group_width(term, contest_def_msg + 2, dlg->items + 2, 1, &min);
-    max_group_width(term, contest_def_msg + 3, dlg->items + 3, 1, &max);
-    min_group_width(term, contest_def_msg + 3, dlg->items + 3, 1, &min);
+    max_group_width(term, contest_def_msg + 2, dlg->items + 2, 2, &max);
+    min_group_width(term, contest_def_msg + 2, dlg->items + 2, 2, &min);
     max_group_width(term, contest_def_msg + 4, dlg->items + 4, 1, &max);
     min_group_width(term, contest_def_msg + 4, dlg->items + 4, 1, &min);
     max_group_width(term, contest_def_msg + 5, dlg->items + 5, 1, &max);
     min_group_width(term, contest_def_msg + 5, dlg->items + 5, 1, &min);
-    max_group_width(term, contest_def_msg + 6, dlg->items + 6, 1, &max);
+    max_group_width(term, contest_def_msg + 6, dlg->items + 5, 1, &max);
     min_group_width(term, contest_def_msg + 6, dlg->items + 6, 1, &min);
     max_group_width(term, contest_def_msg + 7, dlg->items + 7, 1, &max);
     min_group_width(term, contest_def_msg + 7, dlg->items + 7, 1, &min);
+    max_group_width(term, contest_def_msg + 8, dlg->items + 8, 1, &max);
+    min_group_width(term, contest_def_msg + 8, dlg->items + 8, 1, &min);
+    max_group_width(term, contest_def_msg + 9, dlg->items + 9, 1, &max);
+    min_group_width(term, contest_def_msg + 9, dlg->items + 9, 1, &min);
     
-    max_buttons_width(term, dlg->items + 8, 2, &max);
-    min_buttons_width(term, dlg->items + 8, 2, &min);
+    max_buttons_width(term, dlg->items + 10, 2, &max);
+    min_buttons_width(term, dlg->items + 10, 2, &min);
     
     w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
     if (w > max) w = max;
@@ -1354,14 +1355,15 @@ void contest_def_fn(struct dialog_data *dlg)
     y ++;
     dlg_format_group(NULL, term, contest_def_msg + 0, dlg->items + 0, 1, 0, &y, w, &rw);
     dlg_format_group(NULL, term, contest_def_msg + 1, dlg->items + 1, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, contest_def_msg + 2, dlg->items + 2, 1, 0, &y, w, &rw);
-    dlg_format_group(NULL, term, contest_def_msg + 3, dlg->items + 3, 1, 0, &y, w, &rw);
+    dlg_format_group(NULL, term, contest_def_msg + 2, dlg->items + 2, 2, 0, &y, w, &rw);
     dlg_format_group(NULL, term, contest_def_msg + 4, dlg->items + 4, 1, 0, &y, w, &rw);
     dlg_format_group(NULL, term, contest_def_msg + 5, dlg->items + 5, 1, 0, &y, w, &rw);
     dlg_format_group(NULL, term, contest_def_msg + 6, dlg->items + 6, 1, 0, &y, w, &rw);
     dlg_format_group(NULL, term, contest_def_msg + 7, dlg->items + 7, 1, 0, &y, w, &rw);
+    dlg_format_group(NULL, term, contest_def_msg + 8, dlg->items + 8, 1, 0, &y, w, &rw);
+    dlg_format_group(NULL, term, contest_def_msg + 9, dlg->items + 9, 1, 0, &y, w, &rw);
     y++;
-    dlg_format_buttons(NULL, term, dlg->items +8, 2, 0, &y, w, &rw, AL_LEFT);
+    dlg_format_buttons(NULL, term, dlg->items +10, 2, 0, &y, w, &rw, AL_LEFT);
     
     
     w = rw;
@@ -1375,26 +1377,29 @@ void contest_def_fn(struct dialog_data *dlg)
     y++;
     dlg_format_group(term, term, contest_def_msg + 0, dlg->items + 0, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group(term, term, contest_def_msg + 1, dlg->items + 1, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, contest_def_msg + 2, dlg->items + 2, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
-    dlg_format_group(term, term, contest_def_msg + 3, dlg->items + 3, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
+    dlg_format_group(term, term, contest_def_msg + 2, dlg->items + 2, 2, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group(term, term, contest_def_msg + 4, dlg->items + 4, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group(term, term, contest_def_msg + 5, dlg->items + 5, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group(term, term, contest_def_msg + 6, dlg->items + 6, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     dlg_format_group(term, term, contest_def_msg + 7, dlg->items + 7, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
+    dlg_format_group(term, term, contest_def_msg + 8, dlg->items + 8, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
+    dlg_format_group(term, term, contest_def_msg + 9, dlg->items + 9, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
     y++;
-    dlg_format_buttons(term, term, dlg->items +8, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+    dlg_format_buttons(term, term, dlg->items +10, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
     
 }
 
 
 
-void contest_def(struct terminal *term, void *xxx, void *yyy)
+void contest_def(union cba_t cba)
 {
     struct dialog *d;
     int i;
 
     safe_strncpy0(pcall,cfg->pcall,MAX_STR_LEN); uc(pcall);
     safe_strncpy0(pclub,cfg->pclub,MAX_STR_LEN); uc(pclub);
+    safe_strncpy0(operator,cfg->operator,MAX_STR_LEN); uc(operator);
+    global_operator=cfg->global_operator;
     safe_strncpy0(pwwlo,cfg->pwwlo,MAX_STR_LEN); uc(pwwlo);
     safe_strncpy0(padr1,cfg->padr1,MAX_STR_LEN);
     safe_strncpy0(padr2,cfg->padr2,MAX_STR_LEN);
@@ -1405,7 +1410,7 @@ void contest_def(struct terminal *term, void *xxx, void *yyy)
 
     if (!(d = mem_alloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
-    d->title = TEXT(T_CTEST_DEF);
+    d->title = VTEXT(T_CTEST_DEF);
     d->fn = contest_def_fn;
     d->refresh = (void (*)(void *))refresh_contest_def;
     
@@ -1422,46 +1427,61 @@ void contest_def(struct terminal *term, void *xxx, void *yyy)
     
     d->items[i=2].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
+    d->items[i].data = operator;
+    d->items[i].maxl = 15;
+    
+    d->items[i=3].type = D_CHECKBOX;  
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data=(char *)&global_operator;
+    
+    d->items[i=4].type = D_FIELD;
+    d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = pwwlo;
     d->items[i].maxl = 15;
     
-    d->items[i=3].type = D_FIELD;
+    d->items[i=5].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = padr1;
     d->items[i].maxl = 25;
     
-    d->items[i=4].type = D_FIELD;
+    d->items[i=6].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = padr2;
     d->items[i].maxl = 25;
     
-    d->items[i=5].type = D_FIELD;
+    d->items[i=7].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = pexch;
     d->items[i].maxl = 15;
     
-    d->items[i=6].type = D_FIELD;
+    d->items[i=8].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = default_rs;
     d->items[i].maxl = 5;
     
-    d->items[i=7].type = D_FIELD;
+    d->items[i=9].type = D_FIELD;
     d->items[i].dlen = MAX_STR_LEN;
     d->items[i].data = default_rst;
     d->items[i].maxl = 5;
     
-    d->items[++i].type = D_BUTTON;
+    d->items[++i].type = D_BUTTON;   /* 10 */
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
-    d->items[i].text = TEXT(T_OK);
+    d->items[i].text = VTEXT(T_OK);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ESC;
-    d->items[i].fn = cancel_dialog;
-    d->items[i].text = TEXT(T_CANCEL);
+	if (first_contest_def){
+    	d->items[i].fn = ok_dialog;
+	}else{	
+	    d->items[i].fn = cancel_dialog;
+	}
+    d->items[i].text = VTEXT(T_CANCEL);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
                                
 
@@ -1530,7 +1550,7 @@ void edi_prop_fn(struct dialog_data *dlg)
 }
 
 
-void edi_prop(struct terminal *term, void *xxx, void *yyy)
+void edi_prop(union cba_t cba)
 {
     struct dialog *d;
     int i;
@@ -1541,7 +1561,7 @@ void edi_prop(struct terminal *term, void *xxx, void *yyy)
 
     if (!(d = mem_alloc(sizeof(struct dialog) + 20 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 20 * sizeof(struct dialog_item));
-    d->title = TEXT(T_CTEST_DEF);
+    d->title = VTEXT(T_CTEST_DEF);
     d->fn = edi_prop_fn;
     d->refresh = (void (*)(void *))refresh_edi_prop;
     
@@ -1559,19 +1579,19 @@ void edi_prop(struct terminal *term, void *xxx, void *yyy)
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
-    d->items[i].text = TEXT(T_OK);
+    d->items[i].text = VTEXT(T_OK);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
-    d->items[i].text = TEXT(T_CANCEL);
+    d->items[i].text = VTEXT(T_CANCEL);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
                                
 int dlg_edi_prop(struct dialog_data *dlg, struct dialog_item_data *di){
-    edi_prop(dlg->win->term, NULL, NULL); 
+    edi_prop(CBA0); 
     return 0;
 }
 
