@@ -2,19 +2,9 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
@@ -27,28 +17,15 @@
 gint mode;
 char mode_str[EQSO_LEN];
 
-char *mode_msg[]={
-    "unspecified",
-    "SSB",
-    "CW",
-    "SSBs CWr",
-    "CWs SSBr",
-    "AM",
-    "FM",
-    "RTTY",
-    "SSTV",
-    "ATV"   /* 9 */
-};
-
-void mode_func (struct terminal *term, void *amode, void *d){
+void mode_func (union cba_t cba){
     int active;
 
-    active=(vint)amode;
+    active=cba.int_;
     if (active<0 || active>MAX_MODES) return;
     mode = active;
     safe_strncpy0(mode_str,mode_msg[mode],EQSO_LEN);
     /*safe_strncpy0(mode_str,_(CTEXT(T_QP0+qsop_method),term),MAX_STR_LEN);*/
-    redraw_later(term);
+    redraw_later();
 }
 
 int dlg_mode(struct dialog_data *dlg, struct dialog_item_data *di){
@@ -57,12 +34,12 @@ int dlg_mode(struct dialog_data *dlg, struct dialog_item_data *di){
     
     if (!(mi = new_menu(1))) return 0;
     for (i = 1; i <= MAX_MODES; i++) {
-        add_to_menu(&mi, mode_msg[i], "", "", MENU_FUNC mode_func, (void *)(vint)i, 0);
+        add_to_menu(&mi, mode_msg[i], "", "", MENU_FUNC mode_func, (union cba_t)i, 0);
     }
     sel = mode-1;
     if (sel < 0) sel = 0;
     if (sel>=MAX_MODES) sel=0;
-    do_menu_selected(term, mi, (void*)(vint)mode, sel);
+    do_menu_selected(mi, (union cba_t)mode, sel);
     return 0;
 }
 
@@ -127,13 +104,13 @@ void refresh_edit_qso(struct qso *q)
     g_hash_table_foreach(ctest->bystamp, foreach_source_qsort_by_stamp, NULL);
    
     compute_qrbqtf(q);
-    DIRTY_BAND(aband);
+    dirty_band(aband);
     write_qso_to_swap(aband, q);
     replicate_qso(NULL, q);
-    recalc_stats(aband->stats, aband);
-    check_autosave();
+    recalc_stats(aband);
+    /*check_autosave();*/
 #ifdef HAVE_SDL
-	gfx_reload(gfx);
+	maps_reload();
 #endif	
     dump_qso(q, "refresh_edit_qso after");
     dump_all_sources(ctest);
@@ -249,7 +226,7 @@ void edit_qso_fn(struct dialog_data *dlg)
     
 }
 
-void edit_qso(struct terminal *term, struct qso *qso, void *yyy)
+void edit_qso(struct qso *qso)
 {
     struct dialog *d;
     int i;
@@ -257,9 +234,6 @@ void edit_qso(struct terminal *term, struct qso *qso, void *yyy)
     
 
 /*    dbg("edit_qso\n");*/
-    
-    if (!qso) return;
-    
     dump_qso(qso, "edit_qso");
     dump_all_sources(ctest);
     
@@ -283,24 +257,24 @@ void edit_qso(struct terminal *term, struct qso *qso, void *yyy)
     safe_strncpy0(oorstr,     qso->rstr,     EQSO_LEN);
     safe_strncpy0(oorsts,     qso->rsts,     EQSO_LEN);
 
-    g_snprintf(ocallsign, OEQSO_LEN, "%s %-15s", TEXT(T_CALLSIGN2), callsign);
-    g_snprintf(orstr    , OEQSO_LEN, "%s %-15s", TEXT(T_RSTR)     , rstr    );
-    g_snprintf(oqsonrr  , OEQSO_LEN, "%s %-15s", TEXT(T_QSONRR)   , qsonrr  );
-    g_snprintf(oexc     , OEQSO_LEN, "%s %-15s", TEXT(T_OPTEXCH)  , exc     );
-    g_snprintf(olocator , OEQSO_LEN, "%s %-15s", TEXT(T_WWL)      , locator );
-    g_snprintf(omode_str, OEQSO_LEN, "%s %-15s", TEXT(T_MODE)     , mode_str);
-    g_snprintf(orsts    , OEQSO_LEN, "%s %-15s", TEXT(T_RSTS)     , rsts    );
-    g_snprintf(oqsonrs  , OEQSO_LEN, "%s %-15s", TEXT(T_QSONRS)   , qsonrs  );
-    g_snprintf(odate_str, OEQSO_LEN, "%s %-15s", TEXT(T_DATE)     , date_str);
-    g_snprintf(otime_str, OEQSO_LEN, "%s %-15s", TEXT(T_TIME)     , time_str);
-    g_snprintf(ooperator, OEQSO_LEN, "%s %-15s", TEXT(T_OPERATOR) , operator);
-    g_snprintf(oremark  , OEQSO_LEN, "%s %-15s", TEXT(T_REMARK)   , remark  );
+    g_snprintf(ocallsign, OEQSO_LEN, "%s %-15s", VTEXT(T_CALLSIGN2), callsign);
+    g_snprintf(orstr    , OEQSO_LEN, "%s %-15s", VTEXT(T_RSTR)     , rstr    );
+    g_snprintf(oqsonrr  , OEQSO_LEN, "%s %-15s", VTEXT(T_QSONRR)   , qsonrr  );
+    g_snprintf(oexc     , OEQSO_LEN, "%s %-15s", VTEXT(T_OPTEXCH)  , exc     );
+    g_snprintf(olocator , OEQSO_LEN, "%s %-15s", VTEXT(T_WWL)      , locator );
+    g_snprintf(omode_str, OEQSO_LEN, "%s %-15s", VTEXT(T_MODE)     , mode_str);
+    g_snprintf(orsts    , OEQSO_LEN, "%s %-15s", VTEXT(T_RSTS)     , rsts    );
+    g_snprintf(oqsonrs  , OEQSO_LEN, "%s %-15s", VTEXT(T_QSONRS)   , qsonrs  );
+    g_snprintf(odate_str, OEQSO_LEN, "%s %-15s", VTEXT(T_DATE)     , date_str);
+    g_snprintf(otime_str, OEQSO_LEN, "%s %-15s", VTEXT(T_TIME)     , time_str);
+    g_snprintf(ooperator, OEQSO_LEN, "%s %-15s", VTEXT(T_OPERATOR) , operator);
+    g_snprintf(oremark  , OEQSO_LEN, "%s %-15s", VTEXT(T_REMARK)   , remark  );
     
     
     if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
     /*d->title = "Edit QSO";*/
-    g_snprintf(ss,1000,TEXT(T_EDIT_QSO_CD), upcase(aband->bandchar), atoi(qso->qsonrs));
+    g_snprintf(ss,1000,VTEXT(T_EDIT_QSO_CD), upcase(aband->bandchar), atoi(qso->qsonrs));
     d->title = ss;
     d->fn = edit_qso_fn;
     d->refresh = (void (*)(void *))refresh_edit_qso;
@@ -369,15 +343,15 @@ void edit_qso(struct terminal *term, struct qso *qso, void *yyy)
     d->items[++i].type = D_BUTTON;      /* 13 */
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
-    d->items[i].text = TEXT(T_OK);
+    d->items[i].text = VTEXT(T_OK);
     
     d->items[++i].type = D_BUTTON;      /* 14 */
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
-    d->items[i].text = TEXT(T_CANCEL);
+    d->items[i].text = VTEXT(T_CANCEL);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
                                
 
@@ -387,7 +361,7 @@ void edit_qso(struct terminal *term, struct qso *qso, void *yyy)
 
 char qrg_str[EQSO_LEN];
 int we_call;
-char pband[EQSO_LEN]="Select"; /* TODO TEXT(T_SELECT) */
+char pband[EQSO_LEN]="Select"; /* TODO VTEXT(T_SELECT) */
 /*, time_str[EQSO_LEN], callsign[EQSO_LEN]*/;
 /*char locator[EQSO_LEN], remark[MAX_STR_LEN]; */
                        
@@ -398,15 +372,15 @@ void refresh_sked(struct qso *q)
     GString *gs;
     char s[10], *c;
     time_t now;
-    struct tm *utc;
+    struct tm utc;
 
     time(&now);
-    utc=gmtime(&now);
+    gmtime_r(&now, &utc);
     
     dbg("refresh_sked\n");
     gs = g_string_new("SK ");
 
-    g_string_sprintfa(gs, "%d.%02d;", utc->tm_hour, utc->tm_min);
+    g_string_sprintfa(gs, "%d.%02d;", utc.tm_hour, utc.tm_min);
     g_string_append(gs, fixsemi(aband->operator));
     g_string_append_c(gs,';');
     safe_strncpy0(s, aband->pband, 10); 
@@ -516,7 +490,7 @@ void sked_fn(struct dialog_data *dlg)
     
 }
 
-void sked(struct terminal *term, void *xxx, void *yyy){
+void sked(void){
     struct dialog *d;
     int i;
     struct band *band;
@@ -540,7 +514,7 @@ void sked(struct terminal *term, void *xxx, void *yyy){
 
     if (!(d = mem_alloc(sizeof(struct dialog) + 15 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 15 * sizeof(struct dialog_item));
-    d->title = TEXT(T_SEND_SKED);
+    d->title = VTEXT(T_SEND_SKED);
     d->fn = sked_fn;
     d->refresh = (void (*)(void *))refresh_sked;
     d->refresh_data = NULL;
@@ -582,64 +556,64 @@ void sked(struct terminal *term, void *xxx, void *yyy){
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
-    d->items[i].text = TEXT(T_OK);
+    d->items[i].text = VTEXT(T_OK);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
-    d->items[i].text = TEXT(T_CANCEL);
+    d->items[i].text = VTEXT(T_CANCEL);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
                                
-void sked_from_tmpqso(struct terminal *term, struct tmpqso *tmpqso, void *yyy)
+void sked_from_tmpqso(struct tmpqso *tmpqso)
 {
     time_t now;
-    struct tm *utc;
+    struct tm utc;
 
     time(&now);
     now+=cfg->skedshift*60;
-    utc=gmtime(&now);
+    gmtime_r(&now, &utc);
            
     strcpy(qrg_str,"");
     we_call=0;
     /* pband */
-    g_snprintf(time_str, EQSO_LEN, "%d.%02d", utc->tm_hour, utc->tm_min);
+    g_snprintf(time_str, EQSO_LEN, "%d.%02d", utc.tm_hour, utc.tm_min);
     safe_strncpy0(callsign, tmpqso->callsign, EQSO_LEN); uc(callsign);
     safe_strncpy0(locator,  tmpqso->locator,  EQSO_LEN); uc(locator);
     strcpy(remark, "");
 
-    sked(term, NULL, yyy);
+    sked();
 }
 
-void sked_from_qso(struct terminal *term, struct qso *qso, void *yyy)
+void sked_from_qso(struct qso *qso)
 {
     time_t now;
-    struct tm *utc;
+    struct tm utc;
 
     time(&now);
     now+=cfg->skedshift*60;
-    utc=gmtime(&now);
+    gmtime_r(&now, &utc);
            
     strcpy(qrg_str,"");
     we_call=0;
     /* pband */
-    g_snprintf(time_str, EQSO_LEN, "%d.%02d", utc->tm_hour, utc->tm_min);
+    g_snprintf(time_str, EQSO_LEN, "%d.%02d", utc.tm_hour, utc.tm_min);
     safe_strncpy0(callsign, qso->callsign, EQSO_LEN); uc(callsign);
     safe_strncpy0(locator,  qso->locator,  EQSO_LEN); uc(locator);
     strcpy(remark, "");
 
-    sked(term, NULL, yyy);
+    sked();
 }
 
 
 
-void sked_pband_func (struct terminal *term, void *ip, void *d){
+void sked_pband_func (union cba_t cba){
     struct band *band;
     int active;
 
-    active=(vint)ip;
+    active=cba.int_;
     dbg("sked_pband_func(%d)\n", active);
     if (active<0 || active>=ctest->bands->len) return;
     
@@ -647,10 +621,10 @@ void sked_pband_func (struct terminal *term, void *ip, void *d){
     band = g_ptr_array_index(ctest->bands, pband_int);
     safe_strncpy0(pband, band->pband, EQSO_LEN);
  	safe_strncpy0(qrg_str, band->skedqrg, EQSO_LEN);
-    redraw_later(term);
+    redraw_later();
 }
 
-void sked_pband(struct terminal *term, struct session *ses, char *ptr)                   
+void sked_pband(int apband_int)                   
 {                                                                                             
     int i, sel;
     struct menu_item *mi;
@@ -659,16 +633,16 @@ void sked_pband(struct terminal *term, struct session *ses, char *ptr)
     if (!(mi = new_menu(1))) return;
     for (i = 0; i < ctest->bands->len; i++) {
         band = (struct band *)g_ptr_array_index(ctest->bands, i);
-        add_to_menu(&mi, band->pband, "", "", MENU_FUNC sked_pband_func, (void *)(vint)i, 0);
+        add_to_menu(&mi, band->pband, "", "", MENU_FUNC sked_pband_func, (union cba_t)i, 0);
     }
-    sel = (vint) ptr;
+    sel = apband_int;
     if (sel < 0) sel = 0;
     if (sel>=ctest->bands->len) sel=0;
-    do_menu_selected(term, mi, ptr, sel);
+    do_menu_selected(mi, CBA0, sel);
 }
 
 
 int dlg_pband(struct dialog_data *dlg, struct dialog_item_data *di){
-    sked_pband(dlg->win->term, NULL, (char *)(vint)pband_int); 
+    sked_pband(pband_int); 
     return 0;
 }
