@@ -2,19 +2,9 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
@@ -85,9 +75,9 @@ int dlg_clear_tmpqsos(struct dialog_data *dlg, struct dialog_item_data *di){
     return cancel_dialog(dlg,di);
 }
 
+/* todo prototyp */
 int dlg_edit_qso(struct dialog_data *dlg, struct dialog_item_data *di){
-
-    edit_qso(dlg->win->term, (struct qso *)dlg->dlg->refresh_data, NULL);
+    edit_qso((struct qso *)dlg->dlg->refresh_data);
 
     clear_tmpqsos(aband);
     return cancel_dialog(dlg,di);
@@ -98,7 +88,7 @@ int dlg_mark_as_error(struct dialog_data *dlg, struct dialog_item_data *di){
     
     qso = (struct qso*) dlg->dlg->refresh_data;
     qso_mark_as_error(aband, atoi(qso->qsonrs)-1);
-    recalc_stats(aband->stats, aband);
+    recalc_stats(aband);
     
     aband->dupe_in_tmpqso = 0;
     ADD_TMPQSO_STRING(aband, callsign, qso->callsign, 1, ucallsign);
@@ -126,7 +116,7 @@ int dlg_store_as_dupe(struct dialog_data *dlg, struct dialog_item_data *di){
 }
 
 
-void duplicate_callsign(struct terminal *term, struct qso *qso, void *yyy)
+void duplicate_callsign(struct qso *qso)
 {
     struct dialog *d;
     int i;
@@ -134,7 +124,7 @@ void duplicate_callsign(struct terminal *term, struct qso *qso, void *yyy)
 
     if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
-    d->title = TEXT(T_DUPE_QSO);
+    d->title = VTEXT(T_DUPE_QSO);
     d->fn = duplicate_callsign_fn;
     d->refresh = (void (*)(void *))dlg_clear_tmpqsos;
     d->refresh_data = (void *)qso;
@@ -142,71 +132,175 @@ void duplicate_callsign(struct terminal *term, struct qso *qso, void *yyy)
     d->items[i=0].type = D_BUTTON;
     d->items[i].gid  = B_ENTER;
     d->items[i].fn   = dlg_clear_tmpqsos; 
-    d->items[i].text = TEXT(T_CLEAR_THIS_QSO);
+    d->items[i].text = VTEXT(T_CLEAR_THIS_QSO);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid  = B_ENTER;
     d->items[i].fn   = dlg_edit_qso; 
-    d->items[i].text = TEXT(T_EDIT_OLD_QSO);
+    d->items[i].text = VTEXT(T_EDIT_OLD_QSO);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid  = B_ENTER;
     d->items[i].fn   = dlg_mark_as_error; 
-    d->items[i].text = TEXT(T_MARK_OLD_QSO);
+    d->items[i].text = VTEXT(T_MARK_OLD_QSO);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid  = B_ENTER;
     d->items[i].fn   = dlg_store_as_dupe; 
-    d->items[i].text = TEXT(T_STORE_AS_DUPE);
+    d->items[i].text = VTEXT(T_STORE_AS_DUPE);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
                                
 
 
 /**************************** EDIT ***********************************/
 
-void menu_add_error(struct terminal *term, void *xxx, struct session *ses){
+void menu_add_error(union cba_t cba){
     add_error(aband, "");
 } 
 
 /*********************** CHOP - Change operator *************************/
 
-void chop(struct session *ses, char *op){
+static char op[EQSO_LEN];
+static int global_operator;
+
+char *chop_msg[] = {
+    CTEXT(T_CALLSIGN), /* 0 */
+    CTEXT(T_GLOBAL_OPERATOR),
+    "", /* OK */  /* 9 */
+    "", /* Cancel */
+};
+
+void chop(struct band *band, char *op){
     gchar *c;
-
-    if (!op) return;
-
     
+    g_free(band->operator);
+    band->operator = fixsemi(g_strdup(op));
+    c = g_strconcat("O ", op, NULL);
+    add_swap(band, c);
+    g_free(c);  
+    wkd_tmpqso(band, WT_OPERATOR, op);
+}
+
+void refresh_chop(void *xxx){
+    int i;
+
     uc(op);
-    if (!regcmp(op,"^([0-9][A-Z]|[A-Z]{1,2}[0-9]?)[0-9][A-Z]{1,3}$")==0){
-        errbox(TEXT(T_BAD_CALL), 0);
+    if (!regcmp(op,"^([0-9][A-Z]|[A-Z]{1,2}[0-9]?)[0-9]{1,4}[A-Z]{1,4}$")==0){
+        errbox(VTEXT(T_BAD_CALL), 0);
         return;
     }
-        
-    g_free(aband->operator);
-    aband->operator = fixsemi(g_strdup(op));
-    c = g_strconcat("O ", op, NULL);
-    add_swap(aband, c);
-    g_free(c);  
-    wkd_tmpqso(aband, WT_OPERATOR, op);
-    redraw_later(ses->term);
+    cfg->global_operator = global_operator;
+    
+    if (global_operator){
+        for (i=0; i<ctest->bands->len; i++){
+            struct band *b;
+
+            b=(struct band *)g_ptr_array_index(ctest->bands, i);
+            chop(b, op);
+        }
+    }else{
+        chop(aband, op);
+    }
+    
+    redraw_later();
     
 }
 
-void menu_chop(struct terminal *term, void *d, struct session *ses){
-    static char op[EQSO_LEN];
+void chop_fn(struct dialog_data *dlg)
+{
+    struct terminal *term = dlg->win->term;
+    int max = 0, min = 0;
+    int w, rw;
+    int y = -1;
+
+    max_group_width(term, chop_msg + 0, dlg->items + 0, 1, &max);
+    min_group_width(term, chop_msg + 0, dlg->items + 0, 1, &min);
+    max_group_width(term, chop_msg + 1, dlg->items + 1, 1, &max);
+    min_group_width(term, chop_msg + 1, dlg->items + 1, 1, &min);
+    max_buttons_width(term, dlg->items + 2, 2, &max);
+    min_buttons_width(term, dlg->items + 2, 2, &min);
+    
+    w = dlg->win->term->x * 9 / 10 - 2 * DIALOG_LB;
+    if (w > max) w = max;
+    if (w < min) w = min;
+    if (w > dlg->win->term->x - 2 * DIALOG_LB - 8 ) w = dlg->win->term->x - 2 * DIALOG_LB - 8;
+    if (w < 1) w = 1;
+    
+    rw = 0;
+    y ++;
+    dlg_format_group(NULL, term, chop_msg + 0, dlg->items + 0, 1, 0, &y, w, &rw);
+    y++;
+    dlg_format_group(NULL, term, chop_msg + 1, dlg->items + 1, 1, 0, &y, w, &rw);
+    y++;
+    dlg_format_buttons(NULL, term, dlg->items + 2, 2, 0, &y, w, &rw, AL_LEFT);
+    
+    
+    w = rw;
+    dlg->xw = w + 2 * DIALOG_LB;
+    dlg->yw = y + 2 * DIALOG_TB;
+
+    
+    center_dlg(dlg);
+    draw_dlg(dlg);
+    y = dlg->y + DIALOG_TB;
+    y++;
+    dlg_format_group(term, term, chop_msg + 0, dlg->items + 0, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
+    y++;
+    dlg_format_group(term, term, chop_msg + 1, dlg->items + 1, 1, dlg->x + DIALOG_LB, &y, w, AL_LEFT);
+    y++;
+    dlg_format_buttons(term, term, dlg->items + 2, 2, dlg->x + DIALOG_LB, &y, w, NULL, AL_LEFT);
+    
+}
+
+
+void menu_chop(union cba_t cba){
+    struct dialog *d;
+    int i;
     
     if (!ctest || !aband) return;
     
     fixsemi(aband->operator);
     safe_strncpy0(op, aband->operator, EQSO_LEN);
+    global_operator=cfg->global_operator;
 
-    input_field(ses->term, NULL, TEXT(T_CHOP), TEXT(T_N_CALLSIGN), 
-               TEXT(T_OK), TEXT(T_CANCEL), ses, 
+/*    input_field(NULL, VTEXT(T_CHOP), VTEXT(T_N_CALLSIGN), 
+               VTEXT(T_OK), VTEXT(T_CANCEL), NULL, 
                NULL, 20, op, 0, 0, NULL,
-               (void (*)(void *, char *)) chop, NULL);
+               (void (*)(void *, char *)) chop, NULL);*/
+    
+    if (!(d = mem_alloc(sizeof(struct dialog) + 10 * sizeof(struct dialog_item)))) return;
+    memset(d, 0, sizeof(struct dialog) + 10 * sizeof(struct dialog_item));
+    d->title = VTEXT(T_CHOP);
+    d->fn = chop_fn;
+    d->refresh = (void (*)(void *))refresh_chop;
+    
+    d->items[i=0].type = D_FIELD;
+    d->items[i].dlen = EQSO_LEN;
+    d->items[i].data = op;
+    
+    d->items[++i].type = D_CHECKBOX;
+    d->items[i].gid = 0;
+    d->items[i].gnum = 1;
+    d->items[i].dlen = sizeof(int);
+    d->items[i].data = (void *)&global_operator;
+    
+
+    d->items[++i].type = D_BUTTON;
+    d->items[i].gid = B_ENTER;
+    d->items[i].fn = ok_dialog;
+    d->items[i].text = VTEXT(T_OK);
+    
+    d->items[++i].type = D_BUTTON;
+    d->items[i].gid = B_ESC;
+    d->items[i].fn = cancel_dialog;
+    d->items[i].text = VTEXT(T_CANCEL);
+    
+    d->items[++i].type = D_END;
+    do_dialog(d, getml(d, NULL));
+
 
 }
 /*********************** SKEDQRG - Change QRG for sked *************************/
@@ -224,10 +318,10 @@ void skedqrg(struct session *ses, char *qrg){
     c = g_strdup_printf("RG %s;%s\n",aband->pband,qrg);
     rel_write_all(c);
     g_free(c);  
-    redraw_later(ses->term);
+    redraw_later();
 }
 
-void menu_skedqrg(struct terminal *term, void *d, struct session *ses){
+void menu_skedqrg(union cba_t cba){
     static char qrg[20];
     
     if (!ctest || !aband) return;
@@ -235,8 +329,8 @@ void menu_skedqrg(struct terminal *term, void *d, struct session *ses){
     fixsemi(aband->skedqrg);
     safe_strncpy0(qrg, aband->skedqrg, 20);
 
-    input_field(ses->term, NULL, TEXT(T_SKED_QRG), TEXT(T_N_SKED_QRG), 
-               TEXT(T_OK), TEXT(T_CANCEL), ses, 
+    input_field(NULL, VTEXT(T_SKED_QRG), VTEXT(T_N_SKED_QRG), 
+               VTEXT(T_OK), VTEXT(T_CANCEL), NULL, 
                NULL, 20, qrg, 0, 0, NULL,
                (void (*)(void *, char *)) skedqrg, NULL);
 
@@ -244,7 +338,7 @@ void menu_skedqrg(struct terminal *term, void *d, struct session *ses){
 
 /******************** FILL OPERATORS **************************/
 
-void menu_fillop(struct terminal *term, void *d, struct session *ses){
+void menu_fillop(union cba_t cba){
     struct band *b;
     struct qso *q;
     int i,j,k;
@@ -284,24 +378,25 @@ void menu_fillop(struct terminal *term, void *d, struct session *ses){
         if (gs2->len) g_string_erase(gs2,0,1);
         
         g_free(b->mope1);
-        b->mope1=gs1->str;
-        g_string_free(gs1,FALSE);
+        b->mope1=g_strdup(gs1->str);
+        g_string_free(gs1,TRUE);
         
         g_free(b->mope2);
-        b->mope2=gs2->str;
-        g_string_free(gs2,FALSE);
+        b->mope2=g_strdup(gs2->str);
+        g_string_free(gs2,TRUE);
     }
+    save_all_bands_txt(0);
 }
 
 /******************** RECALC ALL QRB and QTF  **************************/
 
-void menu_recalc_qrb(struct terminal *term, void *d, struct session *ses){
+void menu_recalc_qrb(union cba_t cba){
 	if (ctest) recalc_all_qrbqtf(ctest);
 }
 
 /******************** UPDATE C_W ******************************/
 
-void menu_cw_update_contest(struct terminal *term, void *d, struct session *ses){
+void menu_cw_update_contest(union cba_t cba){
     gchar *s;
     int ret;
     
@@ -309,14 +404,14 @@ void menu_cw_update_contest(struct terminal *term, void *d, struct session *ses)
     s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL);
     ret=save_cw_into_file(cw, s);        
     if (ret){
-        errbox(TEXT(T_CANT_WRITE), ret);
+        errbox(VTEXT(T_CANT_WRITE), ret);
     }else{
-        log_addf(TEXT(T_SAVED_S), s);
+        log_addf(VTEXT(T_SAVED_S), s);
     }
     g_free(s);
 }
 
-void menu_cw_update_band(struct terminal *term, void *d, struct session *ses){
+void menu_cw_update_band(union cba_t cba){
     gchar *s;
     int ret;
     
@@ -324,14 +419,14 @@ void menu_cw_update_band(struct terminal *term, void *d, struct session *ses){
     s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL);
     ret=save_cw_into_file(cw, s);        
     if (ret){
-        errbox(TEXT(T_CANT_WRITE), ret);
+        errbox(VTEXT(T_CANT_WRITE), ret);
     }else{
-        log_addf(TEXT(T_SAVED_S), s);
+        log_addf(VTEXT(T_SAVED_S), s);
     }
     g_free(s);
 }
 
-void menu_import_ebw(struct terminal *term, void *d, struct session *ses){
+void menu_import_ebw(union cba_t cba){
     gchar *s;
     int ret;
     
@@ -340,17 +435,17 @@ void menu_import_ebw(struct terminal *term, void *d, struct session *ses){
     s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL);
     ret=save_cw_into_file(cw, s);        
     if (ret){
-        errbox(TEXT(T_CANT_WRITE), ret);
+        errbox(VTEXT(T_CANT_WRITE), ret);
     }else{
-        log_addf(TEXT(T_SAVED_S), s);
+        log_addf(VTEXT(T_SAVED_S), s);
     }
 
     s = g_strconcat(getenv("HOME"), "/tucnak/tucnaknames", NULL);
     ret=save_namedb_into_file(namedb, s);        
     if (ret){
-        errbox(TEXT(T_CANT_WRITE), ret);
+        errbox(VTEXT(T_CANT_WRITE), ret);
     }else{
-        log_addf(TEXT(T_SAVED_S), s);
+        log_addf(VTEXT(T_SAVED_S), s);
     }
 
 
@@ -482,7 +577,7 @@ void autosave_fn(struct dialog_data *dlg)
     
 }
 
-void menu_autosave(struct terminal *term, void *xxx, void *yyy)
+void menu_autosave(union cba_t cba)
 {
     struct dialog *d;
     int i;
@@ -501,7 +596,7 @@ void menu_autosave(struct terminal *term, void *xxx, void *yyy)
 
     if (!(d = mem_alloc(sizeof(struct dialog) + 55 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 55 * sizeof(struct dialog_item));
-    d->title = TEXT(T_AUTOSAVE_OPTS);
+    d->title = VTEXT(T_AUTOSAVE_OPTS);
     d->fn = autosave_fn;
     d->refresh = (void (*)(void *))refresh_autosave;
 
@@ -556,15 +651,15 @@ void menu_autosave(struct terminal *term, void *xxx, void *yyy)
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
-    d->items[i].text = TEXT(T_OK);
+    d->items[i].text = VTEXT(T_OK);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
-    d->items[i].text = TEXT(T_CANCEL);
+    d->items[i].text = VTEXT(T_CANCEL);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
 
 
@@ -583,34 +678,34 @@ void refresh_add_sw(struct session *ses){
 
     switch(asw_type){
         case SWT_PIPE: /* shell */
-            sw = new_subwin(ses, SWT_PIPE, TEXT(T_SW_SHELL), sw_pipe_enter);
+            sw = new_subwin(SWT_PIPE, VTEXT(T_SW_SHELL), sw_pipe_enter);
             break;
         case SWT_QSOS: 
-            sw = new_subwin(ses, SWT_QSOS, TEXT(T_SW_QSOS), NULL);
+            sw = new_subwin(SWT_QSOS, VTEXT(T_SW_QSOS), NULL);
             break;
         case SWT_LOG: 
-            sw = new_subwin(ses, SWT_LOG, TEXT(T_SW_LOG), NULL);
+            sw = new_subwin(SWT_LOG, VTEXT(T_SW_LOG), NULL);
             break;
         case SWT_TALK: 
-            sw = new_subwin(ses, SWT_TALK, TEXT(T_SW_TALK), NULL);
+            sw = new_subwin(SWT_TALK, VTEXT(T_SW_TALK), NULL);
             break;
         case SWT_SKED: 
-            sw = new_subwin(ses, SWT_SKED, TEXT(T_SW_SKED), NULL);
+            sw = new_subwin(SWT_SKED, VTEXT(T_SW_SKED), NULL);
             break;
         case SWT_UNFI: 
-            sw = new_subwin(ses, SWT_UNFI, TEXT(T_SW_UNFI), NULL);
+            sw = new_subwin(SWT_UNFI, VTEXT(T_SW_UNFI), NULL);
             break;
         case SWT_DXC: 
-/*            sw = new_subwin(ses, SWT_DXC, TEXT(T_SW_DXC), NULL);*/
+/*            sw = new_subwin(SWT_DXC, VTEXT(T_SW_DXC), NULL);*/
             break;
         case SWT_STAT: 
-            sw = new_subwin(ses, SWT_STAT, TEXT(T_SW_STAT), NULL);
+            sw = new_subwin(SWT_STAT, VTEXT(T_SW_STAT), NULL);
             break;
         case SWT_SWAP: 
-/*            sw = new_subwin(ses, SWT_SWAP, TEXT(T_SW_STAT), NULL);*/
+/*            sw = new_subwin(SWT_SWAP, VTEXT(T_SW_STAT), NULL);*/
             break;
-        case SWT_PIPE+100: /* other */
-            sw = new_subwin(ses, SWT_PIPE, TEXT(T_SW_PIPE), sw_pipe_enter);
+        case SWT_MAP:
+            sw = new_subwin(SWT_MAP, VTEXT(T_SW_MAP), NULL);
             break;
             
     }
@@ -627,7 +722,7 @@ char *add_sw_msg[] = {
     CTEXT(T_DXC),
     CTEXT(T_STAT),
     CTEXT(T_SWAP),
-    CTEXT(T_PIPE),               
+    CTEXT(T_MAP),               
     CTEXT(T_CMD),  /* 10 */
     CTEXT(T_RESPAWN),
     "", /* OK */  /* 12 */
@@ -721,7 +816,7 @@ void add_sw_fn(struct dialog_data *dlg)
     
 }
 
-void menu_add_subwin(struct terminal *term, void *xxx, struct session *ses){
+void menu_add_subwin(union cba_t cba){
 
     struct dialog *d;
     int i;
@@ -733,10 +828,10 @@ void menu_add_subwin(struct terminal *term, void *xxx, struct session *ses){
     
     if (!(d = mem_alloc(sizeof(struct dialog) + 15 * sizeof(struct dialog_item)))) return;
     memset(d, 0, sizeof(struct dialog) + 15 * sizeof(struct dialog_item));
-    d->title = TEXT(T_ADD_SUBWIN);
+    d->title = VTEXT(T_ADD_SUBWIN);
     d->fn = add_sw_fn;
     d->refresh = (void (*)(void *))refresh_add_sw;
-    d->refresh_data = (void *)ses;
+    d->refresh_data = (void *)gses; /*todo */
 
     
     d->items[i=0].type = D_CHECKBOX;  /* 0 */
@@ -795,7 +890,7 @@ void menu_add_subwin(struct terminal *term, void *xxx, struct session *ses){
     
     d->items[++i].type = D_CHECKBOX;
     d->items[i].gid = 1;
-    d->items[i].gnum = SWT_PIPE+100;
+    d->items[i].gnum = SWT_MAP;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (void *)&asw_type;
     
@@ -810,30 +905,30 @@ void menu_add_subwin(struct terminal *term, void *xxx, struct session *ses){
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ENTER;
     d->items[i].fn = ok_dialog;
-    d->items[i].text = TEXT(T_OK);
+    d->items[i].text = VTEXT(T_OK);
     
     d->items[++i].type = D_BUTTON;
     d->items[i].gid = B_ESC;
     d->items[i].fn = cancel_dialog;
-    d->items[i].text = TEXT(T_CANCEL);
+    d->items[i].text = VTEXT(T_CANCEL);
     
     d->items[++i].type = D_END;
-    do_dialog_(term, d, getml(d, NULL));
+    do_dialog(d, getml(d, NULL));
 }
 
 
 /********************* CLOSE SUBWIN ***********************/
 
-void menu_close_subwin(struct terminal *term, void *xxx, struct session *ses){
+void menu_close_subwin(union cba_t cba){
     struct subwin *sw;
     
 
-    sw = find_sw_ontop(ses);
+    sw = find_sw_ontop();
     if (!sw) return;
 
-    free_subwin(ses, sw);
-    sw_totop_next(ses, 0);
-    redraw_later(ses->term);
+    free_subwin(sw);
+    sw_totop_next( 0);
+    redraw_later();
     
 }
 
@@ -880,15 +975,15 @@ void do_unfinished(struct session *ses, char *remark){
     
     clear_tmpqsos(aband);
     clear_inputline(aband->il);
-    get_cw_qs(ctest->qs, aband->il->cdata); /* clears */
+    get_cw_qs(aband->il->cdata); /* clears */
     get_band_qs(aband, aband->il->cdata);
-    redraw_later(term);
+    redraw_later();
     
 
 }
 
     
-void menu_unfinished(struct terminal *term, void *xxx, struct session *ses){
+void menu_unfinished(union cba_t cba){
     static char remark[40];
     int max;
 
@@ -901,8 +996,8 @@ void menu_unfinished(struct terminal *term, void *xxx, struct session *ses){
     
     strcpy(remark, "");
 
-    input_field(ses->term, NULL, TEXT(T_UNFI_QSO), TEXT(T_N_REMARK), 
-               TEXT(T_OK), TEXT(T_CANCEL), ses, 
+    input_field(NULL, VTEXT(T_UNFI_QSO), VTEXT(T_N_REMARK), 
+               VTEXT(T_OK), VTEXT(T_CANCEL), NULL, 
                NULL, max, remark, 0, 0, NULL,
                (void (*)(void *, char *)) do_unfinished, NULL);
     
