@@ -2,19 +2,9 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
@@ -56,9 +46,12 @@
     CW destip:destport;srcip:srcport;call;locator;date      one C_W line
 
     DX DX de HL2IFR:    24900.2  BA4ED        AS-136                         0807Z
+    OP operator
     
 */
 
+void remove_conn_timer(union cba_t cba);
+
 struct net *net;
 char *ns_desc[]={"INIT", "SABM", "CONN", "WAIT", "DIST", "DEAD"};
 
@@ -75,8 +68,10 @@ struct net *init_net(void){
 }
 
 void free_net(struct net *net){
+    free_net_ifaces(net);
     free_net_tcp(net);
     free_net_udp(net);
+    g_free(net);
 }
 
 #define NET_DELIM " \t,"
@@ -127,9 +122,9 @@ int init_net_udp(struct net *net){
     getsockname(net->udpsock, (struct sockaddr *) &sin, &socklen);
     
     net->udptimer_period = UDP_ANNOUNCE;
-    net->udptimer_id = install_timer_(1, udp_timer, 
-                                     (void *) net);
-    set_handlers(net->udpsock, udp_read_handler, NULL, udp_exception_handler, NULL);
+    net->udptimer_id = install_timer(1, udp_timer, 
+                                     (union cba_t) net);
+    set_handlers(net->udpsock, udp_read_handler, NULL, udp_exception_handler, CBA0);
 
 /*    dbg("I'M %s:%d\n", inet_ntoa(net->my.sin_addr),
  *    ntohs(net->my.sin_port));*/
@@ -188,7 +183,7 @@ int init_net_tcp(struct net *net){
     }
 
     sock_debug(net->tcpsock, "listen init_net_tcp");
-    set_handlers(net->tcpsock, tcp_accept_handler, NULL, tcp_exception_handler, NULL);
+    set_handlers(net->tcpsock, tcp_accept_handler, NULL, tcp_exception_handler, CBA0);
     return 0;
 x:;
     sock_debug(net->tcpsock, "close net->tcpsock");
@@ -220,7 +215,9 @@ int init_net_ifaces(struct net *net){
         for (c=strtok_r(net_if_ignore, NET_DELIM, &token_ptr); c!=NULL; 
              c=strtok_r(NULL, NET_DELIM, &token_ptr)){
             dbg_bcast("adding iface '%s' for ignoring", c);
-            g_ptr_array_add(if_ignore, g_strdup(c));
+            c=g_strdup(c);
+/*            dbg("dup=%p\n", c);*/
+            g_ptr_array_add(if_ignore, c);
         }   
         g_free(net_if_ignore);
     }
@@ -258,7 +255,7 @@ int init_net_ifaces(struct net *net){
             gchar *c;
             c = (gchar *)g_ptr_array_index(if_ignore, ii);
             if (strcmp(c, ifaces[i].name)==0) {
-                dbg_bcast(TEXT(T_IGN_IFACE_S),ifaces[i].name);
+                dbg_bcast(VTEXT(T_IGN_IFACE_S),ifaces[i].name);
                 goto next_loop;
             }
         }
@@ -277,7 +274,7 @@ int init_net_ifaces(struct net *net){
             c = (gchar *)g_ptr_array_index(ip_ignore, ii);
             if (!inet_aton(c, &iad)) continue; /* invalid addr */
             if (iad.s_addr==sin->sin_addr.s_addr) {
-                dbg_bcast(TEXT(T_IGN_ADDR_S), inet_ntoa(sin->sin_addr));
+                dbg_bcast(VTEXT(T_IGN_ADDR_S), inet_ntoa(sin->sin_addr));
                 goto next_loop;
             }
         }
@@ -295,7 +292,7 @@ int init_net_ifaces(struct net *net){
         }
         j++;
 
-        dbg_bcast(TEXT(T_ADD_IFACE_SSD), ifaces[i].name, inet_ntoa(sin->sin_addr), NET_PORT);
+        dbg_bcast(VTEXT(T_ADD_IFACE_SSD), ifaces[i].name, inet_ntoa(sin->sin_addr), NET_PORT);
 next_loop:;
     }
 
@@ -314,7 +311,7 @@ next_loop:;
             sin->sin_addr.s_addr = iad.s_addr;
             sin->sin_port = htons(NET_PORT);
             
-            log_addf(TEXT(T_FORCED_ADDR), inet_ntoa(sin->sin_addr));
+            log_addf(VTEXT(T_FORCED_ADDR), inet_ntoa(sin->sin_addr));
             j++;
         }   
     }
@@ -347,7 +344,7 @@ void free_net_udp(struct net *net){
 }
 
 void free_conn(struct conn *conn){
-    set_handlers(conn->sock, NULL, NULL, NULL, NULL);
+    set_handlers(conn->sock, NULL, NULL, NULL, CBA0);
     sock_debug(conn->sock, "close free_conn()");
     close(conn->sock);
     tcp_kill(conn);
@@ -357,10 +354,11 @@ void free_conn(struct conn *conn){
     CONDGFREE(conn->operator);
     CONDGFREE(conn->remote_id);
     CONDGFREE(conn->remote_ac);
-   
+    /* look at udp_read_handler near to end */
 }
 
 void free_net_ifaces(struct net *net){
+    if (net->myid) g_free(net->myid);
 }
 
 
@@ -391,7 +389,7 @@ void free_net_tcp(struct net *net){
 
 /****************** SELECT's HANDLERS ********************************/
 
-void udp_read_handler(void *data){
+void udp_read_handler(union cba_t cba){
     char s[1024];
     struct sockaddr_in sin;
     socklen_t socklen;
@@ -437,7 +435,7 @@ void udp_read_handler(void *data){
     if (expire < now) return; /* ignore expired information */
     if (expire < now-NET_MAX_SKEW_NEG ||
         expire > now+NET_MAX_SKEW_POS) {
-        log_addf(TEXT(T_TIMESKEW), now-expire, inet_ntoa(master.sin_addr), ntohs(master.sin_port));
+        log_addf(VTEXT(T_TIMESKEW), now-expire, inet_ntoa(master.sin_addr), ntohs(master.sin_port));
     }
     if (expire > now+NET_MAX_SKEW_POS){
         dbg_bcast("ignoring exire in future, ignoring\n");
@@ -464,14 +462,17 @@ void udp_read_handler(void *data){
     if (cmp_sin(&master, &net->global)>0){ /* received master my slave */
         dbg_bcast("tucnak %s:%d means it is master but isn't true\n", inet_ntoa(net->global.sin_addr), ntohs(net->global.sin_port));
         kill_timer(net->udptimer_id);
-        net->udptimer_id = install_timer_(1, udp_timer, (void *) net);
+        net->udptimer_id = install_timer(1, udp_timer, (union cba_t) net);
     }
    
     /* is net->master "global" master? */
     if (net->master) {
         if (cmp_sin(&net->master->sin, &net->global)!=0) { /* net->master > net->global, disconnect */
             dbg_sock("disconnecting from %s:%d\n", inet_ntoa(net->master->sin.sin_addr), ntohs(net->master->sin.sin_port));
-            log_addf(TEXT(T_DISCONNECTING_SD), inet_ntoa(net->master->sin.sin_addr), ntohs(net->master->sin.sin_port));
+            log_addf(VTEXT(T_DISCONNECTING_SD), inet_ntoa(net->master->sin.sin_addr), ntohs(net->master->sin.sin_port));
+/*            CONDGFREE(net->master->operator);
+            CONDGFREE(net->master->remote_id);
+            CONDGFREE(net->master->remote_ac);*/
             free_conn(net->master);
             g_free(net->master);
             net->master = NULL;
@@ -490,14 +491,14 @@ void udp_read_handler(void *data){
     }
 }
 
-void udp_exception_handler(void *data){
+void udp_exception_handler(union cba_t cba){
 
     dbg("udp_exception_handler()\n");
 }
 
 
-void tcp_accept_handler(void *c){
-    int sock;
+void tcp_accept_handler(union cba_t cba){
+    int sock,i;
     struct sockaddr_in sin;
     socklen_t socklen;
     
@@ -511,7 +512,18 @@ void tcp_accept_handler(void *c){
     if (!socklen || sock<0) return;
 
     dbg_sock("Accepted socket %d %s:%d\n", sock, inet_ntoa(sin.sin_addr), ntohs(sin.sin_port));
-    log_addf(TEXT(T_ACCEPTED_SD), inet_ntoa(sin.sin_addr), ntohs(sin.sin_port));
+    dbg("Accepted socket %d %s:%d\n", sock, inet_ntoa(sin.sin_addr), ntohs(sin.sin_port));
+    log_addf(VTEXT(T_ACCEPTED_SD), inet_ntoa(sin.sin_addr), ntohs(sin.sin_port));
+
+    for (i=0;i<net->peers->len;i++){
+        union cba_t cba;
+
+        conn=(struct conn *)g_ptr_array_index(net->peers, i);
+        if (conn->sock!=sock) continue;
+        if (conn->timer) kill_timer(conn->timer);
+        cba.conn=conn;
+        remove_conn_timer(cba);
+    }
     
     conn = g_new0(struct conn, 1);
     conn->sin.sin_family      = AF_INET;
@@ -521,12 +533,13 @@ void tcp_accept_handler(void *c){
     tcp_set_state(conn, NS_CONNECTED);
     
     set_handlers(conn->sock, tcp_read_handler, 
-            NULL, tcp_exception_handler, (void *)conn);
+            NULL, tcp_exception_handler, (union cba_t)conn);
 
     g_ptr_array_add(net->peers, conn);
 
     dump_all_sources(ctest);
 
+    dbg("tcp_accept_handler: net_send_id() sock=%d\n", conn->sock);
     net_send_id();
     net_send_ac();
     net_send_operator();
@@ -534,12 +547,12 @@ void tcp_accept_handler(void *c){
 
 
 
-void tcp_connected_handler(void *c){
+void tcp_connected_handler(union cba_t cba){
     struct conn *conn;
     int ret, err;
     socklen_t optlen;
     
-    conn = (struct conn *) c;
+    conn = cba.conn;
 /*    dbg("tcp_connected_handler %s:%d\n", inet_ntoa(conn->sin.sin_addr),
  *    ntohs(conn->sin.sin_port));*/
 
@@ -554,11 +567,12 @@ void tcp_connected_handler(void *c){
         tcp_disconnect(conn);
         return;
     }
-    log_addf(TEXT(T_CONNECTED_SD), inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));
+    log_addf(VTEXT(T_CONNECTED_SD), inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));
     dump_all_sources(ctest);
     tcp_set_state(conn, NS_CONNECTED);
     set_handlers(conn->sock, tcp_read_handler, 
-            NULL, tcp_exception_handler, (void *)conn);
+            NULL, tcp_exception_handler, (union cba_t)conn);
+    dbg("tcp_connected_handler: net_send_id() sock=%d\n", conn->sock);
     net_send_id();
     net_send_ac();
     net_send_operator();
@@ -566,23 +580,25 @@ void tcp_connected_handler(void *c){
 }
 
 
-void tcp_read_handler(void *c){
-    struct conn *conn;
+void tcp_read_handler(union cba_t cba){
+    struct conn *conn;            
     char s[1030];
     int ret,err;
     gchar *d, *line;
+    char errstr[1030];
     
     
-    conn = (struct conn *) c;
+    conn = cba.conn;
 /*    dbg("tcp_read_handler %s:%d\n", inet_ntoa(conn->sin.sin_addr),
  *    ntohs(conn->sin.sin_port));*/
     
     ret=read(conn->sock, s, 1024);
     err=errno;
-/*    dbg("   read(%d) returns %d (%d) '%s'\n", conn->sock, ret, errno,
- *    strerror(errno));*/
+   /* dbg("   read(%d) returns %d (%d) '%s'\n", conn->sock, ret, err,
+     strerror_r(err, errstr, sizeof(errstr)));*/
     if (ret<=0) {
-        dbg_sock(" !!! read ERROR (%d/%d)", ret,err);
+        dbg_sock(" !!! read ERROR sock=%d err=%d %s\n", conn->sock, err, strerror_r(err, errstr, sizeof(errstr)));
+        dbg(" !!! read ERROR sock=%d err=%d %s\n", conn->sock, err, strerror_r(err, errstr, sizeof(errstr)));
         tcp_disconnect(conn);
         return;
     }
@@ -596,45 +612,45 @@ void tcp_read_handler(void *c){
     /* na tomhle to pada */
 /*    strcpy(s, "ID 192.168.1.132:55556\nAC 20060506;OK1KRQ;II. subregionalni zavod 2006;cegk\nLA 192.168.1.68:55555;1146999049;192.168.1.132:55555;1146991645;192.168.1.132:55556;1147001847;192.168.1.111:55555;1146932813;192.168.1.82:55555;1147003677;192.168.1.42:55555;1147002456;\n");*/
 /*    dbg("   s='%s'\n", s);*/
-    dbg_recv("tcp_read_handler() entry conn->rdbuf=%p '%s'\n", conn->rdbuf, conn->rdbuf);
+    dbg_recv("tcp_read_handler() entry conn->rdbuf=%p '%s'\n", conn->rdbuf, conn->rdbuf->str);
     while (1){
-        dbg_recv("tcp_read_handler() 1 conn->rdbuf=%p '%s'\n", conn->rdbuf, conn->rdbuf);
+        dbg_recv("tcp_read_handler() 1 conn->rdbuf=%p '%s'\n", conn->rdbuf, conn->rdbuf->str);
         if (!conn->rdbuf) raise(SIGSEGV);    
         d=index(conn->rdbuf->str, '\n');
         if (!d) break;
         line = g_strndup(conn->rdbuf->str, d - conn->rdbuf->str);
 /*        dbg("       line='%s'\n", line);*/
-        dbg_recv("tcp_read_handler() 2 conn->rdbuf=%p '%s'\n", conn->rdbuf, conn->rdbuf);
+        dbg_recv("tcp_read_handler() 2 conn->rdbuf=%p '%s'\n", conn->rdbuf, conn->rdbuf->str);
         g_string_erase(conn->rdbuf, 0, d - conn->rdbuf->str + 1);
-        dbg_recv("tcp_read_handler() 3 conn->rdbuf=%p '%s'\n", conn->rdbuf, conn->rdbuf);
+        dbg_recv("tcp_read_handler() 3 conn->rdbuf=%p '%s'\n", conn->rdbuf, conn->rdbuf->str);
 /*        dbg("   line='%s', remains '%s'\n", line, conn->rdbuf->str);*/
-        if (strlen(line)>0) rel_read(c, line);
+        if (strlen(line)>0) rel_read(conn, line);
         g_free(line);
     }
     dbg_recv("tcp_read_handler() exit conn->rdbuf=%p\n", conn->rdbuf);
-    
 }
 
 
-void tcp_write_handler(void *c){
+void tcp_write_handler(union cba_t cba){
     struct conn *conn;
     int towrite, written, err;
+    char errstr[1030];
 
-    conn = (struct conn *) c;
+    conn = cba.conn;
 /*    dbg("tcp_write_handler %s:%d\n", inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));*/
     
     if (!conn->wrbuf){
         set_handlers(conn->sock, tcp_read_handler, 
-                NULL, tcp_exception_handler, (void *)conn);
+                NULL, tcp_exception_handler, (union cba_t)conn);
         return;
     }
     towrite = strlen(conn->wrbuf->str);
     written = write(conn->sock, conn->wrbuf->str, towrite);
     err=errno;
-    dbg_send("s%d: tcp_write_handler '%s' (%d/%d/%d)", conn->sock, conn->wrbuf->str, towrite, written,err);
+    dbg_send("s%d: tcp_write_handler '%s' (%d/%d) %s", conn->sock, conn->wrbuf->str, towrite, written,err);
     /*dbg("    towrite=%d  written=%d \n", towrite, written); */
     if (written<=0){
-        dbg_sock("!!!!! written==%d !!!! disconnecting...\n"); 
+        dbg_sock("!!!!! ERROR %s\n", strerror_r(err, errstr, sizeof(errstr))); 
         tcp_disconnect(conn);
         return;
     }
@@ -642,7 +658,7 @@ void tcp_write_handler(void *c){
         g_string_free(conn->wrbuf, 1);
         conn->wrbuf = NULL;
         set_handlers(conn->sock, tcp_read_handler, 
-                NULL, tcp_exception_handler, (void *)conn);
+                NULL, tcp_exception_handler, (union cba_t)conn);
         return;
         
     }
@@ -651,10 +667,10 @@ void tcp_write_handler(void *c){
 }
 
 
-void tcp_exception_handler(void *c){
+void tcp_exception_handler(union cba_t cba){
     struct conn *conn;
-
-    conn = (struct conn *) c;
+    
+    conn = cba.conn;
     dbg("tcp_exception_handler %s:%d\n", inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port)); 
     
     tcp_disconnect(conn);
@@ -662,7 +678,7 @@ void tcp_exception_handler(void *c){
 
 /*************************** TIMERS *********************************/
 
-void udp_timer(void *data){
+void udp_timer(union cba_t cba){
     struct net *net;
     struct sockaddr_in sin;
     socklen_t socklen;
@@ -671,7 +687,7 @@ void udp_timer(void *data){
     time_t now;
     
 
-    net = (struct net *)data;
+    net = cba.net;
     
     now = time(NULL);
     if (net->global_expire < now){ /* global expired */
@@ -715,16 +731,16 @@ void udp_timer(void *data){
     }
     
     g_string_free(gs, 1);
-    net->udptimer_id = install_timer_(net->udptimer_period, udp_timer, 
-                                     (void *) net);
+    net->udptimer_id = install_timer(net->udptimer_period, udp_timer, 
+                                     (union cba_t) net);
     
 }
 
-void send_ack_timer(void *c){
+void send_ack_timer(union cba_t cba){
     struct conn *conn;
     char s[1026];
 
-    conn = (struct conn *) c;
+    conn = cba.conn;
     /*dbg("send_ack_timer %s:%d\n", inet_ntoa(conn->sin.sin_addr),
      * ntohs(conn->sin.sin_port));*/
     conn->timer = 0;
@@ -734,26 +750,27 @@ void send_ack_timer(void *c){
     tcp_set_state(conn, NS_WAIT_ACK);
 }
 
-void kill_conn_timer(void *c){
+void kill_conn_timer(union cba_t cba){
     struct conn *conn;
 
-    conn = (struct conn *) c;
+    conn = cba.conn;
 /*    dbg("kill_conn_timer %s:%d\n", inet_ntoa(conn->sin.sin_addr),
  *    ntohs(conn->sin.sin_port));*/
     conn->timer = 0;
     tcp_disconnect(conn);
 }
 
-void remove_conn_timer(void *c){
+void remove_conn_timer(union cba_t cba){
     struct conn *conn, *cx;
     int i;
 
-    conn = (struct conn *) c;
+    conn = cba.conn;
 /*    dbg("remove_conn_timer %s:%d\n", inet_ntoa(conn->sin.sin_addr),
  *    ntohs(conn->sin.sin_port));*/
     conn->timer = 0;
     
     if (net->master == conn) {
+        free_conn(net->master);
         g_free(net->master);
         net->master = NULL;
     }
@@ -762,6 +779,7 @@ void remove_conn_timer(void *c){
         cx = (struct conn *) g_ptr_array_index(net->peers, i);
         if (cx == conn) {
             g_ptr_array_remove_index(net->peers, i);
+            free_conn(conn);
             g_free(conn);
         }
     }
@@ -786,24 +804,43 @@ void tcp_set_state(struct conn *conn, enum net_state state){
         case NS_INIT:
             break;
         case NS_CONNECTING:
-            conn->timer = install_timer_(CONNECTION_TIMEOUT, kill_conn_timer, (void *)conn);
+            conn->timer = install_timer(CONNECTION_TIMEOUT, kill_conn_timer, (union cba_t)conn);
             break;
         case NS_CONNECTED:
-            conn->timer = install_timer_(SEND_ACK_TIMEOUT, send_ack_timer, (void *)conn);
+            conn->timer = install_timer(SEND_ACK_TIMEOUT, send_ack_timer, (union cba_t)conn);
             break;
         case NS_WAIT_ACK:
-            conn->timer = install_timer_(ACK_TIMEOUT, kill_conn_timer, (void *)conn);
+            conn->timer = install_timer(ACK_TIMEOUT, kill_conn_timer, (union cba_t)conn);
             break;
         case NS_DISCONNECTED:
-            conn->timer = install_timer_(REMOVE_TIMEOUT, remove_conn_timer, (void *)conn);
+            conn->timer = install_timer(REMOVE_TIMEOUT, remove_conn_timer, (union cba_t)conn);
             break;
         case NS_DEAD:
             break;    
     }
 }
 
+char *tcp_state_s(struct conn *conn){
+    switch (conn->state){
+        case NS_INIT:
+            return "INIT";
+        case NS_CONNECTING:
+            return "CONNECTING";
+        case NS_CONNECTED:
+            return "CONNECTED";
+        case NS_WAIT_ACK:
+            return "WAIT_ACK";
+        case NS_DISCONNECTED:
+            return "DISCONNECTED";
+        case NS_DEAD:
+            return "DEAD";
+    }
+    return "???";
+}
+
 void tcp_connect(struct conn *conn){
     int ret;
+    char errstr[1030];
     
 /*    dbg("tcp->connect %s:%d\n", inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));  */
    
@@ -816,12 +853,13 @@ void tcp_connect(struct conn *conn){
     }
     
     ret = connect(conn->sock, (struct sockaddr *) &conn->sin, sizeof(struct sockaddr_in));
-/*    dbg("connect returns %d (%d) '%s'\n", ret, errno, strerror(errno));*/
+/*    dbg("connect returns %d (%d) '%s'\n", ret, errno, strerror_r(errno));*/
     if (ret==0){
-        set_handlers(conn->sock, tcp_read_handler, NULL, tcp_exception_handler, (void *)conn);
+        set_handlers(conn->sock, tcp_read_handler, NULL, tcp_exception_handler, (union cba_t)conn);
         tcp_set_state(conn, NS_CONNECTED);
-        log_addf(TEXT(T_CONNECTED_SD), inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));
+        log_addf(VTEXT(T_CONNECTED_SD), inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));
         dump_all_sources(ctest);
+        dbg("tcp_connect: net_send_id() sock=%d\n", conn->sock);
         net_send_id();
         net_send_ac();
         net_send_operator();
@@ -830,10 +868,11 @@ void tcp_connect(struct conn *conn){
     if (ret<0){
         if (errno==EALREADY || errno==EINPROGRESS){
             tcp_set_state(conn, NS_CONNECTING);
-            set_handlers(conn->sock, NULL, tcp_connected_handler, tcp_exception_handler, (void *)conn);
-            log_addf(TEXT(T_CONNECTING_SD), inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));
+            set_handlers(conn->sock, NULL, tcp_connected_handler, tcp_exception_handler, (union cba_t)conn);
+            log_addf(VTEXT(T_CONNECTING_SD), inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));
             return;
-        }
+        }  
+        dbg_sock("connect error %s\n", strerror_r(errno, errstr, sizeof(errstr)));
         tcp_disconnect(conn);
     }
     
@@ -842,10 +881,11 @@ void tcp_connect(struct conn *conn){
 void tcp_disconnect(struct conn *conn){
 /*    dbg("tcp->disconnect %s:%d\n", inet_ntoa(conn->sin.sin_addr),
  *    ntohs(conn->sin.sin_port));*/
-    log_addf(TEXT(T_DISCONNECTED_SD), inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));
+    log_addf(VTEXT(T_DISCONNECTED_SD), inet_ntoa(conn->sin.sin_addr), ntohs(conn->sin.sin_port));
     tcp_set_state(conn, NS_DISCONNECTED);
-    set_handlers(conn->sock, NULL, NULL, NULL, NULL);
+    set_handlers(conn->sock, NULL, NULL, NULL, CBA0);
     dbg_sock("s%d: close tcp_disconnect", conn->sock);
+    dbg("tcp_disconnect(%d)\n", conn->sock);
     dump_all_sources(ctest);
     close(conn->sock);
     if (conn->wrbuf) g_string_free(conn->wrbuf, 1);
@@ -856,7 +896,7 @@ void tcp_disconnect(struct conn *conn){
 void tcp_kill(struct conn *conn){
     /*dbg("tcp->disconnect %s:%d\n", inet_ntoa(conn->sin.sin_addr),
      * ntohs(conn->sin.sin_port));*/
-    set_handlers(conn->sock, NULL, NULL, NULL, NULL);
+    set_handlers(conn->sock, NULL, NULL, NULL, CBA0);
     tcp_set_state(conn, NS_DISCONNECTED);
     dbg_sock("s%d: close tcp_kill", conn->sock);
     dump_all_sources(ctest);
@@ -878,6 +918,7 @@ int cmp_sin(struct sockaddr_in *a, struct sockaddr_in *b){
 void rel_write(struct conn *conn, gchar *s){
     int towrite, written,err;
     GString *oldwrbuf;
+    char errstr[1030];
     
     oldwrbuf=conn->wrbuf;
     if (!conn->wrbuf) conn->wrbuf = g_string_sized_new(1024);
@@ -897,9 +938,10 @@ void rel_write(struct conn *conn, gchar *s){
     if (written<0){
         if (err==EAGAIN){
             set_handlers(conn->sock, tcp_read_handler, 
-                    tcp_write_handler, tcp_exception_handler, (void *)conn);
+                    tcp_write_handler, tcp_exception_handler, (union cba_t)conn);
             return;
         }
+        dbg_sock("error writing %s\n", strerror_r(err, errstr, sizeof(errstr)));
         tcp_disconnect(conn);
         return;
     }
@@ -911,26 +953,33 @@ void rel_write(struct conn *conn, gchar *s){
     }
     g_string_erase(conn->wrbuf, 0, written);
     set_handlers(conn->sock, tcp_read_handler, 
-            tcp_write_handler, tcp_exception_handler, (void *)conn);
+            tcp_write_handler, tcp_exception_handler, (union cba_t)conn);
                 
 }
 
 void rel_write_all(gchar *s){
     int i;
     struct conn *conn;
+    char ss[256];
     
+    strncpy(ss, s, 2);
+    ss[2]='\0';
+
     if (net->master && conn_prod_state(net->master)) {
+        /*dbg("rel_write_all('%s') %d(%d)", ss, net->master->sock, tcp_state_s(net->master));*/
         rel_write(net->master, s);
         return;
     }
     
+    /*dbg("rel_write_all('%s') ",ss);*/
     for (i=0; i<net->peers->len; i++){
         conn = (struct conn *) g_ptr_array_index(net->peers, i);    
         if (conn_prod_state(conn)) {
+            /*dbg("%d(%s) ", conn->sock,tcp_state_s(conn) );*/
             rel_write(conn, s);
         }
     }
-    
+/*    dbg("\n"); */
 }
 
 /* writes to all almost conn */
@@ -1111,7 +1160,7 @@ void rel_read(struct conn *conn, gchar *line){
         if (!conn->is_same_ctest) return;
         conn->replicating=0;
         recalc_all_stats(ctest); /* include statsfifo */
-        check_autosave();
+        /*check_autosave();*/
         return;
     }
     
@@ -1182,9 +1231,7 @@ void rel_read(struct conn *conn, gchar *line){
         net->allpeers=g_strdup(c);
 
         if (!net->peerfunc) return;
-        do_peer_menu(term,
-                (void (*)(struct terminal*, int no, struct session *ses)) net->peerfunc,
-                gses);
+        do_peer_menu((void (*)(union cba_t cba)) net->peerfunc);
         net->peerfunc=NULL;
         return;
     }
@@ -1246,7 +1293,7 @@ void rel_read(struct conn *conn, gchar *line){
         if (strlen(items[2])==0) goto x_cf;
         
         if (strcmp(items[2],"eof")==0){
-            log_addf(TEXT(T_LOADED_CFG_SSSS),cfg->pcall, cfg->pwwlo, cfg->padr1,cfg->padr2);
+            log_addf(VTEXT(T_LOADED_CFG_SSSS),cfg->pcall, cfg->pwwlo, cfg->padr1,cfg->padr2);
             goto x_cf;
         }
 
@@ -1312,14 +1359,14 @@ x_cf:;
             gchar *s;
             int ret;
 
-            log_addf(TEXT(T_LOADED_CW));
+            log_addf(VTEXT(T_LOADED_CW));
             
             s = g_strconcat(getenv("HOME"), "/tucnak/tucnakcw", NULL);
             ret=save_cw_into_file(cw, s);        
             if (ret){
-                errbox(TEXT(T_CANT_WRITE), ret);
+                errbox(VTEXT(T_CANT_WRITE), ret);
             }else{
-                log_addf(TEXT(T_SAVED_S), s);
+                log_addf(VTEXT(T_SAVED_S), s);
             }
             g_free(s);
             goto x_cw;
@@ -1343,7 +1390,7 @@ x_cw:;
         d=g_strconcat("DX ",c,"\n",NULL);
         rel_write_almost_all(conn, d);
         g_free(d);
-        redraw_later(term);
+        redraw_later();
         return;
     }
 
@@ -1433,55 +1480,55 @@ x_gr:
         }
 
         sp = get_spypeer_by_peerid(b->spypeers, items[1]);
-        if (!sp) goto x_wt;       /* we don't want spy info from items[1] */
+        if (sp) {       /* we want spy info from items[1] */
           
-        switch(atoi(items[2])){
-            case WT_CLEAR:
-                clear_spypeer(sp);
-                break;
-            case WT_CALLSIGN:
-                CONDGFREE(sp->callsign);
-                sp->callsign=g_strdup(items[3]);
-                break;
-            case WT_RSTS:
-                CONDGFREE(sp->rsts);
-                sp->rsts=g_strdup(items[3]);
-                break;
-            case WT_RSTR:
-                CONDGFREE(sp->rstr);
-                sp->rstr=g_strdup(items[3]);
-                break;
-            case WT_QSONRS:
-                CONDGFREE(sp->qsonrs);
-                sp->qsonrs=g_strdup(items[3]);
-                break;
-            case WT_QSONRR:
-                CONDGFREE(sp->qsonrr);
-                sp->qsonrr=g_strdup(items[3]);
-                break;
-            case WT_EXC:
-                CONDGFREE(sp->exc);
-                sp->exc=g_strdup(items[3]);
-                break;
-            case WT_LOCATOR:
-                CONDGFREE(sp->locator);
-                sp->locator=g_strdup(items[3]);
-                break;
-            case WT_OPERATOR:
-                CONDGFREE(sp->operator);
-                sp->operator=g_strdup(items[3]);
-                break;
-            case WT_REMARK:
-                CONDGFREE(sp->remark);
-                sp->remark=g_strdup(items[3]);
-                break;
-                
+            switch(atoi(items[2])){
+                case WT_CLEAR:
+                    clear_spypeer(sp);
+                    break;
+                case WT_CALLSIGN:
+                    CONDGFREE(sp->callsign);
+                    sp->callsign=g_strdup(items[3]);
+                    break;
+                case WT_RSTS:
+                    CONDGFREE(sp->rsts);
+                    sp->rsts=g_strdup(items[3]);
+                    break;
+                case WT_RSTR:
+                    CONDGFREE(sp->rstr);
+                    sp->rstr=g_strdup(items[3]);
+                    break;
+                case WT_QSONRS:
+                    CONDGFREE(sp->qsonrs);
+                    sp->qsonrs=g_strdup(items[3]);
+                    break;
+                case WT_QSONRR:
+                    CONDGFREE(sp->qsonrr);
+                    sp->qsonrr=g_strdup(items[3]);
+                    break;
+                case WT_EXC:
+                    CONDGFREE(sp->exc);
+                    sp->exc=g_strdup(items[3]);
+                    break;
+                case WT_LOCATOR:
+                    CONDGFREE(sp->locator);
+                    sp->locator=g_strdup(items[3]);
+                    break;
+                case WT_OPERATOR:
+                    CONDGFREE(sp->operator);
+                    sp->operator=g_strdup(items[3]);
+                    break;
+                case WT_REMARK:
+                    CONDGFREE(sp->remark);
+                    sp->remark=g_strdup(items[3]);
+                    break;
+            }
         }
         
         d=g_strconcat("WT ",c,"\n",NULL);
         rel_write_almost_all(conn, d);
         g_free(d);
-        redraw_later(term);
+        redraw_later();
         
 x_wt:
         g_strfreev(items);
@@ -1506,13 +1553,14 @@ x_wt:
             goto x_pt;
         }
         sp = get_spypeer_by_peerid(b->spypeers, items[1]);
-        if (!sp) goto x_pt;       /* we don't want spy info from items[1] */
-        sp->peertx = atoi(items[2]);
+        if (sp) {      /* we want spy info from items[1] */
+            sp->peertx = atoi(items[2]);
+        }
         
         d=g_strconcat("PT ",c,"\n",NULL);
         rel_write_almost_all(conn, d);
         g_free(d);
-        redraw_later(term);
+        redraw_later();
         
 x_pt:
         g_strfreev(items);
@@ -1537,15 +1585,15 @@ x_pt:
             goto x_wi;
         }
         sp = get_spypeer_by_peerid(b->spypeers, items[1]);
-        if (!sp) goto x_wi;       /* we don't want spy info from items[1] */
-
-        CONDGFREE(sp->inputline);
-        sp->inputline = g_strdup(items[2]);
+        if (sp) {       /* we want spy info from items[1] */
+            CONDGFREE(sp->inputline);
+            sp->inputline = g_strdup(items[2]);
+        }
         
         d=g_strconcat("WI ",c,"\n",NULL);
         rel_write_almost_all(conn, d);
         g_free(d);
-        redraw_later(term);
+        redraw_later();
         
 x_wi:
         g_strfreev(items);
@@ -1595,9 +1643,7 @@ x_wi:
         net->allpeers=g_strdup(c);
 
         if (!net->peerfunc) return;
-        do_peer_operators_menu(term,
-                (void (*)(struct terminal*, int no, struct session *ses)) net->peerfunc,
-                gses);
+        do_peer_operators_menu((void (*)(union cba_t cba)) net->peerfunc);
         net->peerfunc=NULL;
         return;
     }
@@ -1629,7 +1675,7 @@ x_wi:
             wkd_tmpqso(b, WT_LOCATOR, b->tmpqsos[0].locator);
             wkd_tmpqso(b, WT_REMARK, b->tmpqsos[0].remark);
             wkd_tmpqso(b, WT_OPERATOR, b->operator);
-            peer_tx(b, ctest->last_cq_timer_id>0?2:ctest->tx);
+            peer_tx(b, gses->last_cq_timer_id>0?2:gses->tx);
             send_inputline(b);
             goto x_rt;  /* noone has same myid, don't need to route */
         }
@@ -1641,6 +1687,21 @@ x_rt:
         g_strfreev(items);
         return;
     }
+    
+    if (strncmp(line, "OP", 2)==0){ /* operator */
+        gchar *d;
+
+        if (strlen(line)<4) return; 
+        c=line+3;
+
+        CONDGFREE(conn->operator);
+        conn->operator=g_strdup(c);
+        
+        d=g_strconcat("OP ",c,"\n",NULL);
+        rel_write_almost_all(conn, d);
+        g_free(d);
+
+    }
 
 }
 
@@ -1720,7 +1781,6 @@ void compare_remote_with_me(gpointer key, gpointer value, gpointer data){
         rel_write(sconn, c);
 
         if (strlen(c)>0 && c[strlen(c)-1]=='\n') c[strlen(c)]='\0';
-        dbg_qsos("replicating2: ser_id=%d '%s'", c);
         dump_qso(q,"replicating2\n");
         
         g_free(c);
@@ -1739,8 +1799,7 @@ void qso_from_net(struct conn *conn, gchar *str){
     gchar *date_str, *time_str, *callsign_or_error, *rsts, *qsonrs;
     gchar *rstr, *qsonrr, *exc, *locator, *source, *operator, *callsign, *remark;
     int found,sortit;
-    time_t now;
-    struct tm *utc;
+    int same_km=0;
 
     
     items=g_strsplit(str, ";", 0);
@@ -1758,10 +1817,6 @@ void qso_from_net(struct conn *conn, gchar *str){
         remark=items[17];
     }
 
-    time(&now);
-    utc=gmtime(&now);
-    
-    
    /* dbg("qso_from_net '%s'\n", str);*/
     /*dbg("qso_from_net sock=%d %d %s %-10s %s | ", */
             /*conn->sock, stamp, qsonrs, callsign, source);*/
@@ -1812,7 +1867,8 @@ void qso_from_net(struct conn *conn, gchar *str){
                 eq->remark   = g_strdup("");    
                 eq->error    = 1;
                 add_qso(b, eq);
-                update_stats(b->stats, b, eq);
+                update_stats(b, b->stats, eq);
+				minute_stats(b);
                 dump_qso(eq, "error");
             }
             if (!some_replicating(net)) check_autosave();
@@ -1834,8 +1890,21 @@ void qso_from_net(struct conn *conn, gchar *str){
     q->band = b;
     STORE_STR   (q,date_str); 
     STORE_STR   (q,time_str); 
-    if (strcasecmp(callsign_or_error, "ERROR")==0) q->error = 1;
-    else q->error=0;
+    
+    if (q->locator){
+        same_km=!strcasecmp(q->locator, locator);
+    }else{
+        same_km=2; /*new qso or contest without locator*/
+    }
+
+    if (strcasecmp(callsign_or_error, "ERROR")==0){
+        if (!q->error) same_km=0;
+        q->error = 1;
+    }else{
+        if (q->error) same_km=0;
+        q->error=0;
+    }
+    
     
     STORE_INT   (q,mode);
     STORE_STR   (q,rsts); 
@@ -1893,21 +1962,22 @@ void qso_from_net(struct conn *conn, gchar *str){
     write_qso_to_swap(b, q); 
 
     if (!ctest->redraw_timer_id)
-        ctest->redraw_timer_id = install_timer_(DELAY_AFTER_REPLICATION, timer_redraw, NULL);
+        ctest->redraw_timer_id = install_timer(DELAY_AFTER_REPLICATION, timer_redraw, CBA0);
     
 add:;    
-    DIRTY_BAND(b); 
+    dirty_band(b); 
     if (!found){
 /*      dbg("adding\n");*/
         add_qso(b, q);
         if (!q->error) invalidate_tmpqso(b, q);
-        update_stats(b->stats, b, q);
+        update_stats(b, b->stats, q);
+		minute_stats(b);
 #ifdef HAVE_SDL
-	    if (b==aband) gfx_add_qso(q);   /*plot new QSO only if it is on active band*/
+	    if (b==aband) map_add_qso(q);   /*plot new QSO only if it is on active band*/
 #endif	  
     }else{
 #ifdef HAVE_SDL
-	  	gfx_reload(gfx);
+	  	if (!same_km) map_clear_qso(q);
 #endif		
 /*      dbg("dont adding\n");*/
     }
@@ -1968,11 +2038,20 @@ void net_send_ac(){
 void net_send_operator(void){
     int i;
 
-    if (!ctest) return;
+    if (!ctest) {
+        if (cfg->operator && *cfg->operator) {
+            gchar *c;
+
+            c=g_strdup_printf("OP %s\n", cfg->operator);
+            rel_write_all(c);
+            g_free(c);
+        }
+        return;
+    }
     for (i=0;i<ctest->bands->len;i++){
         struct band *b;
         b=(struct band *)g_ptr_array_index(ctest->bands, i);
-        wkd_tmpqso(aband, WT_OPERATOR, b->operator);
+        wkd_tmpqso(b, WT_OPERATOR, b->operator);
     }
 }    
 
@@ -2037,23 +2116,26 @@ void net_test_same_contest(struct conn *conn, gchar *ac_text){
 }
 
 
-void timer_redraw(void *data){
+void timer_redraw(union cba_t cba){
     struct band *b;
     int i;
     
     if (ctest){
+        ST_START;
         for (i=0; i<ctest->bands->len; i++){
             b = g_ptr_array_index(ctest->bands, i);
-            recalc_stats(b->stats, b);
+            recalc_stats(b);
             
             if (ctest->qsoused && b->qsos->len+1 != atoi(b->tmpqsos[0].qsonrs)){
                 g_free(b->tmpqsos[0].qsonrs);
                 b->tmpqsos[0].qsonrs = g_strdup_printf("%03d", b->qsos->len+1);
+                wkd_tmpqso(b, WT_QSONRS, b->tmpqsos[0].qsonrs);
             }
         }
+        ST_STOP;
     }
     
-    redraw_later(term);
+    redraw_later();
     ctest->redraw_timer_id = 0;
     
     
@@ -2107,7 +2189,7 @@ gchar *get_timer_str(struct conn *conn){
     return s;
 }
 
-void send_config_request(struct terminal *term, int no, struct session *ses){
+void send_config_request(int no){
     char *peerid,*c;
     struct conn *gw;
     gchar **items;
@@ -2129,7 +2211,7 @@ found:
     if (cmp_sin(&net->global, &net->my)==0) { /* i'm master */
         gw=find_conn_by_remote_id(peerid);
         if (!gw) {
-            log_addf(TEXT(T_PEER_NOT_FOUND), peerid);
+            log_addf(VTEXT(T_PEER_NOT_FOUND), peerid);
             goto x;
         }
     }else{
@@ -2147,7 +2229,7 @@ x:;
     g_strfreev(items);
 }
 
-void send_cwdb_request(struct terminal *term, int no, struct session *ses){
+void send_cwdb_request(int no){
     char *peerid,*c;
     struct conn *gw;
     gchar **items;
@@ -2169,7 +2251,7 @@ found:
     if (cmp_sin(&net->global, &net->my)==0) { /* i'm master */
         gw=find_conn_by_remote_id(peerid);
         if (!gw) {
-            log_addf(TEXT(T_PEER_NOT_FOUND), peerid);
+            log_addf(VTEXT(T_PEER_NOT_FOUND), peerid);
             goto x;
         }
     }else{
