@@ -3,19 +3,9 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
@@ -29,6 +19,14 @@
 #include <gpm.h>
 #endif
 
+#if defined(HAVE_SYS_KD_H)
+#include <sys/kd.h>
+#endif 
+
+#if defined(HAVE_SYS_VFS_H)
+#include <sys/vfs.h>
+#endif 
+
 int is_safe_in_shell(char c)
 {
     return c == '@' || c == '+' || c == '-' || c == '.' || (c >= '0' && c <= '9') || (c >= 'A' && c <= 'Z') || c == '_' || (c >= 'a' && c <= 'z');
@@ -78,22 +76,28 @@ int get_e(char *env)
 
 #if defined(UNIX) || defined(BEOS)
 
-void sigwinch(void *s)
+void sigwinch(union cba_t cba)
 {
-    ((void (*)(void))s)();
+    ((void (*)(void))cba.void_)();
 }
 
-void handle_terminal_resize(int fd, void (*fn)(void))
+void handle_terminal_resize(int fd, void (*fn)(union cba_t))
 {
-    install_signal_handler(SIGWINCH, sigwinch, fn, 0);
+#ifdef HAVE_SDL    
+    if (sdl) return;
+#endif
+    install_signal_handler(SIGWINCH, sigwinch, (union cba_t)(void*)fn, 0);
 }
 
 void unhandle_terminal_resize(int fd)
 {
-    install_signal_handler(SIGWINCH, NULL, NULL, 0);
+#ifdef HAVE_SDL    
+    if (sdl) return;
+#endif
+    install_signal_handler(SIGWINCH, NULL, CBA0, 0);
 }
 
-int get_terminal_size(int fd, int *x, int *y)
+int term_get_terminal_size(int fd, int *x, int *y)
 {
     struct winsize ws;
     if (!x || !y) return -1;
@@ -156,7 +160,7 @@ void winch(void *s)
     ((void (*)())s)();
 }
 
-void handle_terminal_resize(int fd, void (*fn)())
+void handle_terminal_resize(int fd, void (*fn)(union cba_t))
 {
     if (!is_xterm()) return;
     if (!winch_thread_running) {
@@ -164,12 +168,12 @@ void handle_terminal_resize(int fd, void (*fn)())
         winch_thread_running = 1;
         _beginthread((void (*)(void *))winch_thread, NULL, 0x32000, NULL);
     }
-    set_handlers(winch_pipe[0], winch, NULL, NULL, fn);
+    set_handlers(winch_pipe[0], winch, NULL, NULL, (union cba_t)(void*)fn);
 }
 
 void unhandle_terminal_resize(int fd)
 {
-    set_handlers(winch_pipe[0], NULL, NULL, NULL, NULL);
+    set_handlers(winch_pipe[0], NULL, NULL, NULL, CBA0);
 }
 
 int get_terminal_size(int fd, int *x, int *y)
@@ -328,15 +332,20 @@ void set_clipboard_text(char *data)
     /* !!! FIXME */
 }
 
-void set_window_title_(char *title)
+void set_window_title(char *title)
 {
-    if (is_xterm()){
-        if (!title) title="";
-        printf("\033]0;%s\007", title);
-        fflush(stdout);
+#ifdef HAVE_SDL    
+    if (!sdl)
+#endif
+    {
+        if (is_xterm()){
+            if (!title) title="";
+            printf("\033]0;%s\007", title);
+            fflush(stdout);
+        }
     }
 #ifdef HAVE_SDL
-	gfx_set_title(title);
+	sdl_set_title(title);
 #endif	
     /* !!! FIXME */
 }
@@ -487,7 +496,7 @@ char *get_window_title()
 #endif
 }
 
-void set_window_title_(char *title)
+void set_window_title(char *title)
 {
 #ifndef OS2_DEBUG
     static PTIB tib;
@@ -1098,7 +1107,14 @@ void want_draw() {}
 void done_draw() {}
 #endif
 
-int get_output_handle() { return 1; }
+int get_output_handle() { 
+#ifdef HAVE_SDL    
+    if (sdl)
+        return -1;
+    else
+#endif
+        return 1; 
+}
 
 #if defined(OS2)
 
@@ -1106,7 +1122,14 @@ int get_ctl_handle() { return get_input_handle(); }
 
 #else
 
-int get_ctl_handle() { return 0; }
+int get_ctl_handle() { 
+#ifdef HAVE_SDL    
+    if (sdl) 
+        return -1;
+    else
+#endif
+        return 0; 
+}
 
 #endif
 
@@ -1130,32 +1153,16 @@ int get_input_handle()
     return fd[0];
 }
 
-#elif defined(WIN32)
-
-int get_input_handle()
-{
-    int fd[2] ;
-    static int ti = -1, tp = -1;
-    int pid ;
-
-    if (ti != -1) return ti;
-    if (c_pipe (fd) < 0) return 0;
-    ti = fd[0] ;
-    tp = fd[1] ;
-    if (!(pid = fork()))
-    {
-        input_function (tp) ;
-    }
-    else
-        set_proc_id (pid) ;
-    return ti ;
-}
-
 #else
 
 int get_input_handle()
 {
-    return 0;
+#ifdef HAVE_SDL    
+    if (sdl) 
+        return -1;
+    else
+#endif
+        return 0;
 }
 
 #endif /* defined(HAVE_BEGINTHREAD) && defined(HAVE_READ_KBD) */
@@ -1182,12 +1189,15 @@ struct gpm_mouse_spec {
     void *data;
 };
 
-void gpm_mouse_in(struct gpm_mouse_spec *gms)
+void gpm_mouse_in(union cba_t cba)
 {
     Gpm_Event gev;
     struct event ev;
+    struct gpm_mouse_spec *gms;
+        
+    gms=cba.gpm_mouse_spec;
     if (Gpm_GetEvent(&gev) <= 0) {
-        set_handlers(gms->h, NULL, NULL, NULL, NULL);
+        set_handlers(gms->h, NULL, NULL, NULL, CBA0);
         return;
     }
     ev.ev = EV_MOUSE;
@@ -1219,14 +1229,14 @@ void *handle_mouse(int cons, void (*fn)(void *, char *, int), void *data)
     gms->h = h;
     gms->fn = fn;
     gms->data = data;
-    set_handlers(h, (void (*)(void *))gpm_mouse_in, NULL, NULL, gms);
+    set_handlers(h, gpm_mouse_in, NULL, NULL, (union cba_t)gms);
     return gms;
 }
 
 void unhandle_mouse(void *h)
 {
     struct gpm_mouse_spec *gms = h;
-    set_handlers(gms->h, NULL, NULL, NULL, NULL);
+    set_handlers(gms->h, NULL, NULL, NULL, CBA0);
     Gpm_Close();
     mem_free(gms);
 }
@@ -1264,12 +1274,12 @@ int get_system_env()
 
 #endif
 
-void exec_new_links(struct terminal *term, char *xterm, char *exe, char *param)
+void exec_new_links(char *xterm, char *exe, char *param)
 {
     char *str;
     if (!(str = mem_alloc(strlen(xterm) + 1 + strlen(exe) + 1 + strlen(param) + 1))) return;
     sprintf(str, "%s %s %s", xterm, exe, param);
-    exec_on_terminal(term, str, "", 2);
+    exec_on_terminal(str, "", 2);
     mem_free(str);
 }
 
@@ -1277,37 +1287,37 @@ void open_in_new_twterm(struct terminal *term, char *exe, char *param)
 {
     char *twterm;
     if (!(twterm = getenv("TUCNAK_TWTERM"))) twterm = "twterm -e";
-    exec_new_links(term, twterm, exe, param);
+    exec_new_links(twterm, exe, param);
 }
 
 void open_in_new_xterm(struct terminal *term, char *exe, char *param)
 {
     char *xterm;
     if (!(xterm = getenv("TUCNAK_XTERM"))) xterm = "xterm -e";
-    exec_new_links(term, xterm, exe, param);
+    exec_new_links(xterm, exe, param);
 }
 
 void open_in_new_screen(struct terminal *term, char *exe, char *param)
 {
-    exec_new_links(term, "screen", exe, param);
+    exec_new_links("screen", exe, param);
 }
 
 #ifdef OS2
 void open_in_new_vio(struct terminal *term, char *exe, char *param)
 {
-    exec_new_links(term, "cmd /c start /c /f /win", exe, param);
+    exec_new_links("cmd /c start /c /f /win", exe, param);
 }
 
 void open_in_new_fullscreen(struct terminal *term, char *exe, char *param)
 {
-    exec_new_links(term, "cmd /c start /c /f /fs", exe, param);
+    exec_new_links("cmd /c start /c /f /fs", exe, param);
 }
 #endif
 
 #ifdef BEOS
 void open_in_new_be(struct terminal *term, char *exe, char *param)
 {
-    exec_new_links(term, "Terminal", exe, param);
+    exec_new_links("Terminal", exe, param);
 }
 #endif
 
@@ -1366,14 +1376,14 @@ int can_open_os_shell(int environment)
 }
 
 
-int mkdir_p(const char *s,mode_t mode) {
+/*int mkdir_p(const char *s,mode_t mode) {
   struct stat st;
   gchar *dir,*c;
   
   if (!stat(s,&st))
     if (S_ISDIR(st.st_mode)) return(0);
     
-  dir = g_strdup(s);
+  dir = g_strdup((char *)s);
   c=strrchr(dir,'/');
   if (c==NULL) {
       g_free(dir);
@@ -1387,7 +1397,7 @@ int mkdir_p(const char *s,mode_t mode) {
   mkdir_p(dir,mode);
   g_free(dir);
   return(mkdir(s,mode));
-}
+} */
 
 
 
@@ -1565,3 +1575,81 @@ int do_mount(gchar *mountcmd){
    	return 0;
     
 }
+
+static int sound_fd=-1;
+
+int init_sound(){
+    int ret,i;
+    int fds[2];
+
+    if (pipe(fds)) internal("Can't create pipe");
+    /*dbg("sound pipe %d->%d\n", fds[1], fds[0]);*/
+
+    ret=fork();
+    if (ret<0){
+        internal("Can't fork");
+    }
+    if (ret) {  /* parent */
+        close(fds[0]);
+        sound_fd=fds[1];
+        return 0; 
+    }
+    /* child */
+    close(fds[1]);
+    close(0);
+    dup2(fds[0], 0);
+    close(fds[0]);
+    for (i=3;i<1024;i++) close(i);
+    execlp("soundwrapper", "soundwrapper", NULL);
+	exit(-1);
+}
+
+int free_sound(){
+    int cmd, ret;
+    
+    if (sound_fd<0) return -1;
+    cmd=-1;
+    ret=write(sound_fd, &cmd, sizeof(int));
+    if (ret!=sizeof(int)){
+        char errstr[1030];
+        dbg("error writing sound_fd=%d (error %d) '%s'\n", sound_fd, errno, strerror_r(errno, errstr, sizeof(errstr)));      
+    }
+    close(sound_fd);
+    sound_fd=-1;
+    return 0;
+}
+
+int sound(int freq){
+    int ret;
+
+    if (sound_fd<0) {
+/*        dbg("sound_fd is closed\n");*/
+        return -1;
+    }
+    
+    ret=write(sound_fd, &freq, sizeof(int));
+    if (ret!=sizeof(int)){
+        char errstr[1030];
+        
+        dbg("error writing sound_fd=%d (error %d) '%s'\n", sound_fd, errno, strerror_r(errno, errstr, sizeof(errstr)));      
+        close(sound_fd);
+        sound_fd=-1;
+        return -1;
+    }
+    return 0;
+}
+
+double fdf(int fd){
+#if defined(HAVE_SYS_VFS_H)
+    struct statfs sf;
+    double ret;
+
+    if (fstatfs(fd, &sf)) return -1.0;
+
+    ret=(double)sf.f_bsize*(double)sf.f_bavail;
+/*    dbg("bsize=%ld, bavail=%ld, ret=%f\n", sf.f_bsize, sf.f_bavail, ret);*/
+    return ret;
+#else
+    return -1.0;
+#endif    
+}
