@@ -2,19 +2,9 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
@@ -23,17 +13,30 @@
 struct contest *ctest;
 struct band *aband;
 
-int init_ctest(){
+char *mode_msg[]={
+    "unspecified",
+    "SSB",
+    "CW",
+    "SSBs CWr",
+    "CWs SSBr",
+    "AM",
+    "FM",
+    "RTTY",
+    "SSTV",
+    "ATV"   /* 9 */
+};
 
+
+int init_ctest(){
     /*dbg("init_ctest\n");*/
     if (ctest) return 0;
 
     ctest = g_new0(struct contest, 1);
     ctest->bands = g_ptr_array_new();
     ctest->bystamp = g_hash_table_new(g_str_hash, g_str_equal);
-    ctest->hicalls=g_hash_table_new(g_str_hash, g_str_equal);    
+
 /*    ctest->serial = 1;*/
-    ctest->qs = g_ptr_array_new();
+	ctest->minute_timer_id=install_timer(60000, timer_minute_stats_all, CBA0);
     return 0;
 }
 
@@ -41,19 +44,28 @@ int new_ctest(char *tdate){
     struct stat st;
     int counter;
     time_t now;
-    struct tm *tm;
+    struct tm tm;
     gchar *c;
     int ret;
     static char s[1024]; /* for msg_box */
-
+    int nowdate_int, ctestdate_int;
 
     now = time(NULL);
-    tm = gmtime(&now);
+    gmtime_r(&now, &tm);
     
-    if (tdate) 
+    if (tdate){ 
         ctest->cdate = g_strdup(tdate);
-    else
-        ctest->cdate = g_strdup_printf("%4d%02d%02d", tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday );
+        nowdate_int = (tm.tm_year+1900)*10000 + (tm.tm_mon+1)*100 + tm.tm_mday;
+        ctestdate_int = atoi(ctest->cdate);
+        if (/*nowdate_int >= ctestdate_int && */nowdate_int <= ctestdate_int + 2) 
+            ctest->recording = 1;
+        else
+            ctest->recording = 0;
+    }else{
+        ctest->cdate = g_strdup_printf("%4d%02d%02d", tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday );
+        ctest->recording = 1;
+    }
+    
     
     ctest->directory = g_strdup_printf("%s/tucnak/%s",
             getenv("HOME"), ctest->cdate);
@@ -69,7 +81,7 @@ int new_ctest(char *tdate){
         
     if (mkdir_p(ctest->directory,0770)){
 
-        g_snprintf(s,1000, TEXT(T_CANT_CREATE_HOME_S),ctest->directory);
+        g_snprintf(s,1000, VTEXT(T_CANT_CREATE_HOME_S),ctest->directory);
         errbox(s,errno);
         free_ctest();
         return -1;
@@ -79,15 +91,17 @@ int new_ctest(char *tdate){
     c = g_strdup_printf("%s/desc",ctest->directory);
     ctest->descfile = fopen(c,"wt");
     if (!ctest->descfile){
-        g_snprintf(s,1000, "%s %s", TEXT(T_CANT_WRITE),c);
+        g_snprintf(s,1000, "%s %s", VTEXT(T_CANT_WRITE),c);
         errbox(s,errno);
+        g_free(c);
         free_ctest();
         return -1;
     }
+    g_free(c);
     setvbuf(ctest->descfile, NULL, _IONBF, 0);
     
     if (lockf(fileno(ctest->descfile), F_TLOCK, 0)){
-        g_snprintf(s,1000, "%s %s", TEXT(T_CANT_LOCK),c);
+        g_snprintf(s,1000, "%s %s", VTEXT(T_CANT_LOCK),c);
         errbox(s,errno);
         free_ctest();
         return -1;
@@ -106,18 +120,20 @@ int new_ctest(char *tdate){
     c=g_strdup_printf("%s/log", ctest->directory);
     ctest->logfile = fopen(c, "at");
     if (!ctest->logfile){
-        g_snprintf(s,1000, TEXT(T_CANT_OPEN_S),c);
+        g_snprintf(s,1000, VTEXT(T_CANT_OPEN_S),c);
+        g_free(c);
+        msg_box(NULL, VTEXT(T_ERROR), AL_CENTER, s, NULL, 1, VTEXT(T_OK), NULL, B_ENTER | B_ESC);
         g_free(c);
-        msg_box(term, NULL, TEXT(T_ERROR), AL_CENTER, s, NULL, 1, TEXT(T_OK), NULL, B_ENTER | B_ESC);
         free_ctest();
         return -1;
     }
+    g_free(c);
     setvbuf(ctest->logfile, NULL, _IONBF, 0);
 
     
-    set_ctest_title(term);
+    set_ctest_title();
 #ifdef HAVE_SDL
-	gfx_reload(gfx);
+	maps_reload();
 #endif	
     
     return 0; 
@@ -163,57 +179,17 @@ int load_ctest_from_mem(struct contest *ctest, gchar *datedir, GHashTable *hash)
     STORE_HASH_INT(ctest, wwlused);
     STORE_HASH_INT(ctest, wwlbonu);
     STORE_HASH_INT(ctest, wwlmult);
-    STORE_HASH_INT(ctest, modesep);
     
-    set_ctest_title(term);
-#ifdef HAVE_SDL
-	gfx_reload(gfx);
-#endif	
+    set_ctest_title();
+/*#ifdef HAVE_SDL
+	maps_reload();
+#endif	*/
    
     return 0; 
 }
 
 
 
-void free_ctest(){
-    int i;
-
-    if (!ctest) return;
-
-    dump_all_sources(ctest);
-    
-    if (ctest->last_cq_timer_id) {
-        kill_timer(ctest->last_cq_timer_id);
-        ctest->last_cq_timer_id = 0;
-    }
-    
-
-    for (i=0;i<ctest->bands->len; i++){
-        free_band(g_ptr_array_index(ctest->bands,i));
-    }
-
-    if (ctest->descfile)  fclose(ctest->descfile);
-    if (ctest->logfile)   fclose(ctest->logfile);
-    if (ctest->directory) g_free(ctest->directory);
-    if (ctest->cdate)     g_free(ctest->cdate);
-    free_hash(ctest->hicalls);
-    /*TODO free ctest->bystamp*/
-    g_ptr_array_free(ctest->bands, TRUE);
-    g_ptr_array_free_all(ctest->qs);
-    if (ctest->redraw_timer_id) kill_timer(ctest->redraw_timer_id);
-    g_free(ctest);
-    ctest=NULL;
-    aband=NULL;
-    
-    set_ctest_title(term);
-    cq_abort(1);
-#ifdef HAVE_SDL
-	gfx_reload(gfx);
-#endif	
-}
-
-
-
 struct band *init_band(struct config_band *confb, 
                        GHashTable *opt_band){
     struct band *b;
@@ -229,7 +205,7 @@ struct band *init_band(struct config_band *confb,
 
     b=g_new0(struct band,1);
 
-    b->qsos  = g_ptr_array_new();
+    b->qsos = g_ptr_array_new();
     b->mode  = MOD_SSB_SSB;
     b->bandmulti = 1;
     b->spymode = SM_INPUTLINE;
@@ -240,7 +216,11 @@ struct band *init_band(struct config_band *confb,
         b->readonly = confb->readonly;
 
         b->psect = confb->psect; /* gint */
-        b->operator = fixsemi(g_strdup(get_raw_call(s,ctest->pcall)));
+        if (cfg->operator && *cfg->operator){
+            b->operator = fixsemi(g_strdup(cfg->operator));
+        }else{
+            b->operator = fixsemi(g_strdup(get_raw_call(s,ctest->pcall)));
+        }
         b->stxeq = g_strdup(confb->stxeq);
         b->spowe = g_strdup(confb->spowe);
         b->srxeq = g_strdup(confb->srxeq);
@@ -308,6 +288,8 @@ struct band *init_band(struct config_band *confb,
     }
 
     b->stats = init_stats();
+    b->tmpstats = init_stats();
+    b->stats_mutex = g_mutex_new();
     
     c = g_strdup_printf("%s/%c.swp",ctest->directory,b->bandchar);
             
@@ -320,7 +302,7 @@ struct band *init_band(struct config_band *confb,
     
     b->swap = fopen(c,"at");
     if (!b->swap){
-        log_addf(TEXT(T_CANT_APPEND_S),c);
+        log_addf(VTEXT(T_CANT_APPEND_S),c);
         g_free(c);
         free_band(b);
         return NULL;
@@ -333,10 +315,11 @@ struct band *init_band(struct config_band *confb,
     b->qs = g_ptr_array_new();
     b->oqs = g_ptr_array_new();
     b->unfi = init_fifo(100);        
-    b->statsfifo = init_fifo(1000);        
+    b->statsfifo1 = init_fifo(1000);        
     
     g_ptr_array_add(ctest->bands, b);
-    ctest->qrv|=1<<(toupper(b->bandchar)-'A');
+    ctest->qrv|=1<<(upcase(b->bandchar)-'A');
+    aband->tmplocqso.band=aband;
     
 /*    init_spypeer(b->spypeers, "127.0.0.1:55555");
     init_spypeer(b->spypeers, "127.0.0.1:55556");*/
@@ -345,27 +328,6 @@ struct band *init_band(struct config_band *confb,
 }
 
 
-/* called ONLY when ctest is freeed. bands remain in memory */
-void free_band(struct band *b){
-    struct event ev = {EV_ABORT, 0,0,0};
-
-    clear_tmpqsos(b);
-    if (b->tmpqsos[0].qsonrs) g_free(b->tmpqsos[0].qsonrs);
-    if (b->unres) g_free(b->unres);
-    if (b->stats) free_stats(b->stats);
-    if (b->swap) fclose(b->swap);
-    inputln_func(b->il,&ev);
-    if (b->qs)  g_ptr_array_free_all(b->qs);
-    if (b->oqs) g_ptr_array_free_all(b->oqs);
-    if (b->swapfifo)  free_fifo(b->swapfifo);
-    if (b->unfi)      free_fifo(b->unfi);
-    if (b->statsfifo) free_fifo(b->statsfifo);
-    if (b->ctrlsp)    g_free(b->ctrlsp);
-    free_spypeers(b->spypeers);
-    g_free(b);
-}
-
-
 struct band *find_band_by_pband(char *pband){
     int i;
     struct band *b;
@@ -449,7 +411,6 @@ void clear_tmpqsos(struct band *b){
     } 
     CONDGFREE(b->unres);
     CONDGFREE(b->qrv_str);
-
     wkd_tmpqso(aband, WT_CLEAR, "");
     if (ctest->qsoused){
         c=g_strdup_printf("%03d",b->qsos->len+1);
@@ -460,15 +421,17 @@ void clear_tmpqsos(struct band *b){
 
     default_rst_to_tmpqsos(b);
     ssbd_callsign(ssbd, "");
+
 }
 
 void default_rst_to_tmpqsos(struct band *b){
     gchar *c;
     int i;
     
+	if (!ctest) return;
     for(i=0;i<TMP_QSOS;i++)  CLEAR_TMPQSO_STRING(b,rsts);
 
-/*    if (!ctest->qsoused) return;*/
+    /*if (!ctest->qsoused) return;*/
     
     if (b->mode == MOD_CW_CW)
         c=g_strdup(cfg->default_rst);
@@ -477,6 +440,7 @@ void default_rst_to_tmpqsos(struct band *b){
        
     ADD_TMPQSO_STRING(b,rsts,c,1,udummy);
     wkd_tmpqso(b, WT_RSTS, c);
+    g_free(c);
 }
        
 #define APPEND_Q(item) \
@@ -503,9 +467,10 @@ void add_qso(struct band *b, struct qso *q){
     q->band = b; /* don't dealloc! */
     /*q->serial = ctest->serial++;*/
     
+    stats_thread_kill(b);
     g_ptr_array_add(b->qsos,q);
     add_qso_to_index(q, 0);
-    DIRTY_BAND(b);
+    dirty_band(b);
 }
 
 struct qso *get_qso(struct band *b, gint i){
@@ -537,7 +502,7 @@ struct qso *get_qso_by_id(struct band *b, gchar *source, gint ser_id){
     struct qso *qso;
     int i;
             
-  	/* TODO id's in hash */
+	/* TODO id's in hash */
 	/*dbg("get_qso_by_id(src=%s, ser_id=%d) \n", source, ser_id);*/
     for(i=0;i<b->qsos->len; i++){
         qso = g_ptr_array_index(b->qsos, i);
@@ -568,15 +533,15 @@ struct qso *get_qso_by_qsonr(struct band *b, gint qsonrs){
 void add_error(struct band *b, gchar *remark){
     struct qso *eq;
     time_t now;
-    struct tm *utc;
+    struct tm utc;
 
     if (!ctest && !b) return;
     if (b->readonly) return;
 
-    log_addf("added ERROR for band %c, qsonr about %d\n", b->bandchar, b->qsos->len);
+    log_addf("added ERROR for band %c, qsonr %d\n", b->bandchar, b->qsos->len+1);
     
     time(&now);
-    utc=gmtime(&now);
+    gmtime_r(&now, &utc);
            
     eq = g_new0(struct qso, 1);
     eq->source   = g_strdup(net->myid);
@@ -585,8 +550,8 @@ void add_error(struct band *b, gchar *remark){
         eq->date_str = g_strdup(b->tmpqsos[0].date_str);
         eq->time_str = g_strdup(b->tmpqsos[0].time_str);
     }else{
-       eq->date_str = g_strdup_printf("%4d%02d%02d",1900+utc->tm_year, 1+utc->tm_mon, utc->tm_mday);
-       eq->time_str = g_strdup_printf("%02d%02d",utc->tm_hour, utc->tm_min);    
+       eq->date_str = g_strdup_printf("%4d%02d%02d",1900+utc.tm_year, 1+utc.tm_mon, utc.tm_mday);
+       eq->time_str = g_strdup_printf("%02d%02d",utc.tm_hour, utc.tm_min);    
     }
     eq->callsign = g_strdup(b->tmpqsos[0].callsign&&b->tmpqsos[0].ucallsign?b->tmpqsos[0].callsign:"ERROR");    
     eq->rsts     = g_strdup(b->tmpqsos[0].rsts?b->tmpqsos[0].rsts:"");        
@@ -600,7 +565,9 @@ void add_error(struct band *b, gchar *remark){
     eq->ser_id   = -1;  /* computed by add_qso_to_index */
     
     add_qso(b, eq);
-    update_stats(b->stats, b, eq);
+    stats_thread_join(b);
+    update_stats(b, b->stats, eq);
+	minute_stats(b);
     recalc_statsfifo(b);
     
     if (b->qsos->len+1 != atoi(b->tmpqsos[0].qsonrs)){
@@ -613,7 +580,7 @@ void add_error(struct band *b, gchar *remark){
     wkd_tmpqso(aband, WT_QSONRS, TMPQ.qsonrs);
 
     replicate_qso(NULL, eq);
-    redraw_later(term);
+    redraw_later();
     check_autosave();
 
 }
@@ -625,7 +592,7 @@ void qso_mark_as_error(struct band *b, gint i){
     qso = g_ptr_array_index(b->qsos, i);
     qso->error = 1;
     qso->qsop = 0;
-    DIRTY_BAND(b);
+    dirty_band(b);
     replicate_qso(NULL, qso);
 }
 
@@ -647,7 +614,7 @@ int add_swap(struct band *band, gchar *s){
 
 
 
-void activate_band(struct session *ses, struct band *b){
+void activate_band(struct band *b){
     struct event ev={EV_RESIZE, 0, 0, 0};
     
     /*dbg("activate_band(ses=%p,b=%p) %c\n",ses,b);*/
@@ -658,21 +625,20 @@ void activate_band(struct session *ses, struct band *b){
     
     il_unset_focus(aband->il);
     aband=b;
-    sw_default_func(ses->ontop, &ev, 1);
-    ses->ontop->check_bounds(ses->ontop);
-    sw_unset_focus(ses);
+    sw_all_func(&ev, 1);
+/*    sw_default_func(gses->ontop, &ev, 1);*/
+    gses->ontop->check_bounds(gses->ontop);
+    sw_unset_focus();
     il_set_focus(aband->il);
     get_band_qs(aband, aband->il->cdata);
     get_oband_qs(aband, aband->il->cdata);
-    get_cw_qs(ctest->qs, aband->il->cdata);
+    get_cw_qs(aband->il->cdata);
     
-    if (ses->ontop && ses->ontop->type == SWT_STAT)
+    if (gses->ontop && gses->ontop->type == SWT_STAT)
         recalc_statsfifo(b);
 
-    redraw_later(ses->term);
-#ifdef HAVE_SDL
-	gfx_reload(gfx);
-#endif	
+    redraw_later();
+	maps_reload();
 }
 
 
@@ -747,7 +713,7 @@ int add_qsos_from_swap(struct band *b, FILE *f){
         compute_qrbqtf(q);
         
         add_qso(b, q);
-        update_stats(b->stats, b, q);
+        update_stats(b, q);
     }
     check_autosave();
     
@@ -784,8 +750,8 @@ gchar *get_latests_str(){
     g_hash_table_foreach(ctest->bystamp, foreach_add_latests_str, gs);
     /*dbg("get_latests_str='%s'\n", gs->str);*/
     g_string_append(gs, "\n");
-    c = gs->str;
-    g_string_free(gs, 0);
+    c = g_strdup(gs->str);
+    g_string_free(gs, 1);
     return c;
 }
 
@@ -856,12 +822,6 @@ void add_qso_to_index(struct qso *q, int qsort_if_needed){
         ia = g_index_array_new();
         g_hash_table_insert(ctest->bystamp, g_strdup(q->source), ia);
     }
-    /*dbg_qsos("  add_qso_to_index: ia->len=%d q->ser_id=%d", ia->len, q->ser_id);*/
-    /*while (q->ser_id > ia->len) {
-        dbg_qsos("  adding NULL item to GIndexArray[ia->len]");
-        g_index_array_add(ia, NULL);
-    } */
-
         
 	if (q->ser_id==-1) q->ser_id=ia->len; /* ia is NOT sorted by ser_id! */
     g_index_array_add(ia, q);
@@ -884,16 +844,21 @@ void remove_qso_from_index(struct qso *q){
 }
 
 void compute_qrbqtf(struct qso *q){
-    double qtf;
+    double qtf, qtfrad;
     
     if (!q) return;
     /* CHANGE look at add_tmpqso_locator */    
-    qrbqtf(ctest->pwwlo, q->locator, &q->qrb, &qtf, NULL);
+    qrbqtf(ctest->pwwlo, q->locator, &q->qrb, &qtf, NULL, 2);
     q->qtf = (int) (qtf+0.5);
     if (q->qrb < 0.1) {
         q->qrb = 0;
         q->qtf = 0;
     }
+    qtfrad=qtf*MY_PI/180.0;
+    q->kx =   q->qrb*sin(qtfrad);
+    q->ky = - q->qrb*cos(qtfrad);
+    /*dbg("compute_qrbqtf(%s) %s->%s qrb=%f qtf=%d kx=%d ky=%d\n", 
+            q->callsign, ctest->pwwlo, q->locator, q->qrb, q->qtf, q->kx, q->ky); */
 }
 
 
@@ -932,7 +897,7 @@ char *my_strstr(const char *phaystack, const char *pneedle){
 
     b = *needle;
     if (b != '\0'){
-        if (b != '.'){
+        if (b != '.' && b != '?'){
             haystack--;             /* possible ANSI violation */
             do{
                 c = *++haystack;
@@ -955,30 +920,30 @@ char *my_strstr(const char *phaystack, const char *pneedle){
                 a = *++haystack;
                 if (a == '\0') goto ret0;
 shloop:;
-            }while (a != b && b != '.');
+            }while (a != b && b != '.' && b != '?');
 
 jin:;     
             a = *++haystack;
             if (a == '\0') goto ret0;
-            if (c!='.' && a != c)    goto shloop;
+            if (c!='.' && c!='?' && a != c)    goto shloop;
 
             rhaystack = haystack-- + 1;
             rneedle = needle;
             a = *rneedle;
 
-            if (*rhaystack == a || a=='.')
+            if (*rhaystack == a || a=='.' || a=='?')
                 do{
                     if (a == '\0') goto foundneedle;
                     ++rhaystack;
                     a = *++needle;
-                    if (a!='.'){
+                    if (a!='.' && a!='?'){
                         if (*rhaystack != a) break;
                     }
                     if (a == '\0') goto foundneedle;
                         
                     ++rhaystack;
                     a = *++needle;
-                }while (*rhaystack == a || a == '.');
+                }while (*rhaystack == a || a == '.' || a=='?');
 
             needle = rneedle;       /* took the register-poor approach */
 
@@ -1063,21 +1028,21 @@ GPtrArray *get_band_qs(struct band *band, gchar *str){
     struct qs *qs;
     struct qso *qso;
 
-   /* ST_START;*/
+    /*ST_START;*/
     g_ptr_array_free_items(band->qs);
 
     len = strlen(str);
     if (len<2) return NULL;
     
     qs = g_new0(struct qs, 1);
-    qs->result = band->qs;
+    qs->result1 = band->qs;
     qs->str = str;
 
 
     for (i=0;i<band->qsos->len;i++){
         qso = (struct qso *)g_ptr_array_index(band->qsos, i);
         if (qso->error || qso->dupe) continue;
-        if (qs->result->len == term->y - QSONR_HEIGHT - 4 - cfg->loglines - band->spypeers->len - 1) break;
+        if (qs->result1->len == term->y - QSONR_HEIGHT - 4 - cfg->loglines - band->spypeers->len - 1) break;
         
         if (qso->callsign && my_strstr(qso->callsign, qs->str)){
             if (qso->locator)
@@ -1085,7 +1050,7 @@ GPtrArray *get_band_qs(struct band *band, gchar *str){
             else
                 c=g_strdup(qso->callsign);
 /*            dbg(",%s", c);*/
-            g_ptr_array_add(qs->result, c);
+            g_ptr_array_add(qs->result1, c);
             continue;
         }
         
@@ -1095,8 +1060,8 @@ GPtrArray *get_band_qs(struct band *band, gchar *str){
             else
                 c=g_strdup_printf("%-10s %-6s%5d/%3d", "", qso->locator, (int)qso->qrb, qso->qtf);
 /*            dbg(",%s", c);*/
-            g_ptr_array_add(qs->result, c);
-            continue;
+            g_ptr_array_add(qs->result1, c);
+            continue;                 
         }
         
     }
@@ -1123,7 +1088,7 @@ GPtrArray *get_oband_qs(struct band *oband, gchar *str){
     if (len<2) return NULL;
     
     qs = g_new0(struct qs, 1);
-    qs->result = oband->oqs;
+    qs->result1 = oband->oqs;
     qs->str = str;
 
     for (j=0;j<ctest->bands->len;j++){
@@ -1135,31 +1100,32 @@ GPtrArray *get_oband_qs(struct band *oband, gchar *str){
         for (i=0;i<band->qsos->len;i++){
             qso = (struct qso *)g_ptr_array_index(band->qsos, i);
             if (qso->error || qso->dupe) continue;
-            if (qs->result->len == term->y - QSONR_HEIGHT - 4 - cfg->loglines - oband->spypeers->len - 1) break;
+            if (qs->result1->len == term->y - QSONR_HEIGHT - 4 - cfg->loglines - oband->spypeers->len - 1) break;
             /* FIXME contest without locator */
             if ((qso->callsign && my_strstr(qso->callsign, qs->str))||
                 (qso->locator  && my_strstr(qso->locator, qs->str))){    
                 
                 int j;
 
-                for (j=0;j<qs->result->len; j++){
+                for (j=0;j<qs->result1->len; j++){
                     char d[20], *pd, *line;
                     
-                    safe_strncpy(d, line=g_ptr_array_index(qs->result,j), 10);
+                    safe_strncpy(d, line=g_ptr_array_index(qs->result1,j), 10);
                     pd = index(d, ' ');
                     if (pd) *pd='\0';
                     
                     if (strcmp(d, qso->callsign)==0){ 
 /*                        dbg("add %s %c\n", line, band->bandchar);*/
-                        c = g_strdup_printf("%s%c", line, toupper(band->bandchar));
-                        g_ptr_array_index(qs->result, j) = c;
+                        c = g_strdup_printf("%s%c", line, upcase(band->bandchar));
+                        g_ptr_array_index(qs->result1, j) = c;
                         g_free(line);
                         goto cont;
                     }
                 }
 /*                dbg("ins %s %s %c\n", qso->callsign, qso->locator, band->bandchar);*/
-                c=g_strdup_printf("%-10s %-6s %c", qso->callsign, qso->locator, toupper(band->bandchar));
-                g_ptr_array_add(qs->result, c);
+                c=g_strdup_printf("%-10s %-6s %c", qso->callsign, qso->locator, upcase(band->bandchar));
+                set_mem_comment(c, c, strlen(c));
+                g_ptr_array_add(qs->result1, c);
                 goto cont;
             }
 cont:;                
@@ -1182,6 +1148,7 @@ gchar *find_wwl_by_oband(struct band *oband, gchar *call){
     if (ctest->bands->len<2) return NULL;
     if (!ctest->wwlused) return NULL;
     
+    ST_START;
     for (j=0;j<ctest->bands->len;j++){
         struct band *band;
         
@@ -1196,6 +1163,7 @@ gchar *find_wwl_by_oband(struct band *oband, gchar *call){
             return qso->locator; /* can be NULL */
         }
     }
+    ST_STOP;
     return NULL;
 }
 
@@ -1213,7 +1181,6 @@ char *get_raw_call(char *buf, char *call){
         safe_strncpy0(buf, c, 20);  
         return buf;
     }
-   
     safe_strncpy0(buf, call, 20);  
     return buf;
 }
@@ -1230,8 +1197,8 @@ void dump_qso(struct qso *q, char *desc){
         dbg_qsos("  NULL");
         return;
     }
-   /* dbg_qsos("  %8s %c%03d %s:%d.%05d  %s", q->callsign, toupper(q->band->bandchar), atoi(q->qsonrs), q->source, q->ser_id, q->stamp%100000, desc);*/
-    dbg_qsos("  %8s %c%03d %s:%d.%05d  (qsop=%d error=%d dupe=%d) %s", q->callsign, toupper(q->band->bandchar), atoi(q->qsonrs), q->source, q->ser_id, q->stamp%100000, q->qsop, q->error, q->dupe,  desc);
+   /* dbg_qsos("  %8s %c%03d %s:%d.%05d  %s", q->callsign, upcase(q->band->bandchar), atoi(q->qsonrs), q->source, q->ser_id, q->stamp%100000, desc);*/
+    dbg_qsos("  %8s %c%03d %s:%d.%05d  (qsop=%d error=%d dupe=%d) %s", q->callsign, upcase(q->band->bandchar), atoi(q->qsonrs), q->source, q->ser_id, q->stamp%100000, q->qsop, q->error, q->dupe,  desc);
 }
 
 void invalidate_tmpqso(struct band *b, struct qso *q){
@@ -1244,8 +1211,170 @@ void invalidate_tmpqso(struct band *b, struct qso *q){
         get_raw_call(s, b->tmpqsos[i].callsign);
         if (strcasecmp(s, raw)==0){
             b->tmpqsos[i].ucallsign=0;   
-            redraw_later(term);
+            redraw_later();
         }
     }
 }
 
+void foreach_free_bystamp(gpointer key, gpointer value, gpointer data){
+    GIndexArray *ia;
+
+    ia = (GIndexArray *)value;
+    g_index_array_free(ia, TRUE);
+    g_free(key);
+
+}
+    
+void free_ctest(){
+    int i;
+
+    if (!ctest) return;
+
+
+	kill_timer(ctest->minute_timer_id);
+	
+    dump_all_sources(ctest);
+    
+    if (gses->last_cq_timer_id) {
+        kill_timer(gses->last_cq_timer_id);
+        gses->last_cq_timer_id = 0;
+    }
+    
+
+    for (i=0;i<ctest->bands->len; i++){
+        free_band(g_ptr_array_index(ctest->bands,i));
+    }
+
+    if (ctest->descfile)  fclose(ctest->descfile);
+    if (ctest->logfile)   fclose(ctest->logfile);
+    CONDGFREE(ctest->directory);
+    CONDGFREE(ctest->cdate);
+    CONDGFREE(ctest->tname);
+    CONDGFREE(ctest->tdate);
+    CONDGFREE(ctest->pcall);
+    CONDGFREE(ctest->padr1);
+    CONDGFREE(ctest->padr2);
+    CONDGFREE(ctest->pclub);
+    CONDGFREE(ctest->pwwlo);
+    CONDGFREE(ctest->pexch);
+    CONDGFREE(ctest->default_rs);
+    CONDGFREE(ctest->default_rst);
+    
+    CONDGFREE(ctest->rname);
+    CONDGFREE(ctest->rcall);
+    CONDGFREE(ctest->radr1);
+    CONDGFREE(ctest->radr2);
+    CONDGFREE(ctest->rpoco);
+    CONDGFREE(ctest->rcity);
+    CONDGFREE(ctest->rcoun);
+    CONDGFREE(ctest->rphon);
+    CONDGFREE(ctest->rhbbs);
+    
+    CONDGFREE(ctest->directory);
+/*    CONDGFREE(ctest->pband);
+    CONDGFREE(ctest->);
+    CONDGFREE(ctest->);
+    CONDGFREE(ctest->);
+  */
+
+    g_hash_table_foreach(ctest->bystamp, foreach_free_bystamp, NULL); 
+    g_ptr_array_free(ctest->bands, TRUE);
+    if (ctest->redraw_timer_id) kill_timer(ctest->redraw_timer_id);
+    g_free(ctest);
+    ctest=NULL;
+    aband=NULL;
+    
+    set_ctest_title();
+    cq_abort(0);
+    if (!terminate) maps_reload();
+}
+
+
+
+/* called ONLY when ctest is freeed. bands remain in memory */
+void free_band(struct band *b){
+    int i;
+    struct event ev = {EV_ABORT, 0,0,0};
+
+    stats_thread_kill(b);
+    clear_tmpqsos(b);
+    if (b->tmpqsos[0].qsonrs) g_free(b->tmpqsos[0].qsonrs);
+    if (b->tmpqsos[0].rsts)   g_free(b->tmpqsos[0].rsts);
+
+    for (i=0;i<b->qsos->len;i++){
+        struct qso *qso;
+        qso=g_ptr_array_index(b->qsos, i);
+        free_qso(qso);
+    }
+    
+    CONDGFREE(b->pband);
+    CONDGFREE(b->operator);
+    CONDGFREE(b->stxeq);
+    CONDGFREE(b->spowe);
+    CONDGFREE(b->srxeq);
+    CONDGFREE(b->sante);
+    CONDGFREE(b->santh);
+    CONDGFREE(b->mope1);
+    CONDGFREE(b->mope2);
+    CONDGFREE(b->remarks);
+	CONDGFREE(b->skedqrg);
+    /*CONDGFREE(b->);
+    CONDGFREE(b->);
+    CONDGFREE(b->);
+    CONDGFREE(b->);*/
+    
+    
+    
+    inputln_func(b->il,&ev);
+    g_free(b->il);
+    if (b->unres) g_free(b->unres);
+    if (b->stats) free_stats(b->stats);
+    if (b->tmpstats) free_stats(b->tmpstats);
+    g_mutex_free(b->stats_mutex);
+    if (b->swap) fclose(b->swap);
+    if (b->qs)  g_ptr_array_free_all(b->qs);
+    if (b->oqs) g_ptr_array_free_all(b->oqs);
+    if (b->swapfifo)  free_fifo(b->swapfifo);
+    if (b->unfi)      free_fifo(b->unfi);
+    if (b->statsfifo1) free_fifo(b->statsfifo1);
+    if (b->ctrlsp)    g_free(b->ctrlsp);
+    free_spypeers(b->spypeers);
+    CONDGFREE(b->tmplocqso.locator);
+    g_free(b);
+}
+
+
+void free_qso(struct qso *qso){
+    CONDGFREE(qso->source);
+    CONDGFREE(qso->operator);
+    CONDGFREE(qso->date_str);
+    CONDGFREE(qso->time_str);
+    CONDGFREE(qso->callsign);
+    CONDGFREE(qso->rsts);
+    CONDGFREE(qso->rstr);
+    CONDGFREE(qso->qsonrs);
+    CONDGFREE(qso->qsonrr);
+    CONDGFREE(qso->exc);
+    CONDGFREE(qso->locator);
+    CONDGFREE(qso->remark);
+
+    g_free(qso);
+}
+
+
+void dirty_band(struct band *band) {
+    dbg_qsos("DIRTY_BAND %c",band->bandchar);
+    band->dirty_save = 1;
+    band->dirty_stats = 1;
+    band->dirty_statsf = 1;
+};
+
+enum modes get_mode(void){
+	if (aband) return aband->mode;
+	else return gses->mode;
+}
+
+void set_mode(enum modes mode){
+	if (aband) aband->mode=mode;
+	else gses->mode=mode;
+}
