@@ -2,31 +2,21 @@
     Tucnak - VHF contest log
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
  
 #include "header.h"
 
-#define RC_MAIN "/etc/tucnakrc"
 #define RC_USER "/tucnak/tucnakrc"
 #define RC_LOCAL "/tucnak/tucnakrc.local"
 
 
 struct config *cfg;
+int opt_g, opt_i, opt_t, debug_keyboard=0;
     
 enum {
     SYMBOL_OBSOLETE     = G_TOKEN_LAST + 10000,
@@ -105,51 +95,63 @@ enum {
     SYMBOL_TRACE_QSOS   = G_TOKEN_LAST + 77,
     SYMBOL_LANGUAGE     = G_TOKEN_LAST + 78,
 
-    SYMBOL_CWDA_HOST     = G_TOKEN_LAST + 80,
-    SYMBOL_CWDA_UDP      = G_TOKEN_LAST + 81,
-    SYMBOL_CWDA_DEVICE_S = G_TOKEN_LAST + 82,
-    SYMBOL_CWDA_SPEED    = G_TOKEN_LAST + 83,
-    SYMBOL_CWDA_WEIGHT   = G_TOKEN_LAST + 84,
-    SYMBOL_CWDA_SPKDEV   = G_TOKEN_LAST + 85,
-    SYMBOL_CWDA_SPKVOL   = G_TOKEN_LAST + 86,
-    SYMBOL_CWDA_PTTDELAY = G_TOKEN_LAST + 87,
-#if 0    
-    SYMBOL_CWDA_DEVICE   = G_TOKEN_LAST + 88,/* obsolete */
-    SYMBOL_CWDA_SPEAKER  = G_TOKEN_LAST + 89,/* obsolete */
-    SYMBOL_CWDA_PORT     = G_TOKEN_LAST + 90,/* obsolete */
-#endif
+    SYMBOL_CWDA_TYPE     = G_TOKEN_LAST + 80,
+    SYMBOL_CWDA_DEVICE   = G_TOKEN_LAST + 81,
+    SYMBOL_CWDA_HOST     = G_TOKEN_LAST + 82,
+    SYMBOL_CWDA_UDP      = G_TOKEN_LAST + 83,
+    SYMBOL_CWDA_SPEED    = G_TOKEN_LAST + 84,
+    SYMBOL_CWDA_WEIGHT   = G_TOKEN_LAST + 85,
+    SYMBOL_CWDA_SPK      = G_TOKEN_LAST + 86,
+	SYMBOL_CWDA_VID      = G_TOKEN_LAST + 87,
+	SYMBOL_CWDA_PID      = G_TOKEN_LAST + 88,
 
-	SYMBOL_SSBD_HOST       = G_TOKEN_LAST + 100,
-	SYMBOL_SSBD_UDP        = G_TOKEN_LAST + 101,
-	SYMBOL_SSBD_DSP        = G_TOKEN_LAST + 102,
-	SYMBOL_SSBD_RECORD     = G_TOKEN_LAST + 103,
+    SYMBOL_SSBD_TYPE       = G_TOKEN_LAST + 100,
+	SYMBOL_SSBD_RECORD     = G_TOKEN_LAST + 101,
+	SYMBOL_SSBD_MAXMIN     = G_TOKEN_LAST + 102,
+	SYMBOL_SSBD_DISKFREE   = G_TOKEN_LAST + 103,
 	SYMBOL_SSBD_FORMAT     = G_TOKEN_LAST + 104,
 	SYMBOL_SSBD_CHANNELS   = G_TOKEN_LAST + 105,
 	SYMBOL_SSBD_SAMPLERATE = G_TOKEN_LAST + 106,
-	SYMBOL_SSBD_PIPE       = G_TOKEN_LAST + 107,
-	SYMBOL_SSBD_COMMAND    = G_TOKEN_LAST + 108,
-	SYMBOL_SSBD_TEMPLATE   = G_TOKEN_LAST + 109,
-	SYMBOL_SSBD_MIXER      = G_TOKEN_LAST + 110,
-	SYMBOL_SSBD_RECSRC     = G_TOKEN_LAST + 111,
-    
-	SYMBOL_TERM 			 = G_TOKEN_LAST + 120,
-	SYMBOL_TERM_MODE 		 = G_TOKEN_LAST + 121,
-	SYMBOL_TERM_M11_HACK	 = G_TOKEN_LAST + 122,
-	SYMBOL_TERM_RESTRICT_852 = G_TOKEN_LAST + 123,
-	SYMBOL_TERM_BLOCK_CURSOR = G_TOKEN_LAST + 124,
-	SYMBOL_TERM_COL			 = G_TOKEN_LAST + 125,
-	SYMBOL_TERM_UTF_8_IO     = G_TOKEN_LAST + 126,
-	SYMBOL_TERM_CHARSET		 = G_TOKEN_LAST + 127,
+	SYMBOL_SSBD_TEMPLATE   = G_TOKEN_LAST + 107,
 
-    SYMBOL_ROTAR            = G_TOKEN_LAST + 130,
-	SYMBOL_ROT_TYPE	        = G_TOKEN_LAST + 131,
-	SYMBOL_ROT_PORT         = G_TOKEN_LAST + 132,
-	SYMBOL_ROT_SADDR        = G_TOKEN_LAST + 133,
-	SYMBOL_ROT_TIMEOUT_MS   = G_TOKEN_LAST + 134,
-	SYMBOL_ROT_FILENAME     = G_TOKEN_LAST + 135,
-	SYMBOL_ROT_HOSTNAME     = G_TOKEN_LAST + 136,
-	SYMBOL_ROT_DESC         = G_TOKEN_LAST + 137,
-	SYMBOL_ROT_BEAMWIDTH    = G_TOKEN_LAST + 138,
+    SYMBOL_SSBD_PCM_PLAY   = G_TOKEN_LAST + 108,
+    SYMBOL_SSBD_PCM_REC    = G_TOKEN_LAST + 109,
+    SYMBOL_SSBD_PERIOD_T   = G_TOKEN_LAST + 110,
+    SYMBOL_SSBD_BUFFER_T   = G_TOKEN_LAST + 111,
+    SYMBOL_SSBD_ALSA_MIXER = G_TOKEN_LAST + 112,
+    SYMBOL_SSBD_ALSA_SRC   = G_TOKEN_LAST + 113,
+
+	SYMBOL_SSBD_DSP        = G_TOKEN_LAST + 114,
+    SYMBOL_SSBD_MAXFRAG    = G_TOKEN_LAST + 115,
+	SYMBOL_SSBD_MIXER      = G_TOKEN_LAST + 116,
+	SYMBOL_SSBD_OSS_SRC    = G_TOKEN_LAST + 117,
+	SYMBOL_SSBD_RECSRC     = G_TOKEN_LAST + 118,
+
+    SYMBOL_SSBD_HOST       = G_TOKEN_LAST + 119,
+	SYMBOL_SSBD_UDP        = G_TOKEN_LAST + 120,
+
+    /* unused */
+	SYMBOL_SSBD_PIPE       = G_TOKEN_LAST + 121,
+	SYMBOL_SSBD_COMMAND    = G_TOKEN_LAST + 122,
+    
+	SYMBOL_TERM 			 = G_TOKEN_LAST + 130,
+	SYMBOL_TERM_MODE 		 = G_TOKEN_LAST + 131,
+	SYMBOL_TERM_M11_HACK	 = G_TOKEN_LAST + 132,
+	SYMBOL_TERM_RESTRICT_852 = G_TOKEN_LAST + 133,
+	SYMBOL_TERM_BLOCK_CURSOR = G_TOKEN_LAST + 134,
+	SYMBOL_TERM_COL			 = G_TOKEN_LAST + 135,
+	SYMBOL_TERM_UTF_8_IO     = G_TOKEN_LAST + 136,
+	SYMBOL_TERM_CHARSET		 = G_TOKEN_LAST + 137,
+
+    SYMBOL_ROTAR            = G_TOKEN_LAST + 140,
+	SYMBOL_ROT_TYPE	        = G_TOKEN_LAST + 141,
+	SYMBOL_ROT_PORT         = G_TOKEN_LAST + 142,
+	SYMBOL_ROT_SADDR        = G_TOKEN_LAST + 143,
+	SYMBOL_ROT_TIMEOUT_MS   = G_TOKEN_LAST + 144,
+	SYMBOL_ROT_FILENAME     = G_TOKEN_LAST + 145,
+	SYMBOL_ROT_HOSTNAME     = G_TOKEN_LAST + 146,
+	SYMBOL_ROT_DESC         = G_TOKEN_LAST + 147,
+	SYMBOL_ROT_BEAMWIDTH    = G_TOKEN_LAST + 148,
 	
 	
     SYMBOL_SW                = G_TOKEN_LAST + 150,
@@ -161,6 +163,10 @@ enum {
     SYMBOL_LOGLINES          = G_TOKEN_LAST + 160,
     SYMBOL_SKEDSHIFT         = G_TOKEN_LAST + 161,
     SYMBOL_TAKEOFF           = G_TOKEN_LAST + 162,
+    SYMBOL_OPERATOR          = G_TOKEN_LAST + 163,
+    SYMBOL_GLOBAL_OPERATOR   = G_TOKEN_LAST + 164,
+	SYMBOL_GFX_X             = G_TOKEN_LAST + 165,
+	SYMBOL_GFX_Y             = G_TOKEN_LAST + 166,
     
     SYMBOL_MY_LAST     		 = G_TOKEN_LAST + 170
 };
@@ -242,27 +248,43 @@ void add_symbols(GScanner *scanner){
     g_scanner_add_symbol(scanner,"trace_qsos",       GINT_TO_POINTER(SYMBOL_TRACE_QSOS)); 
     g_scanner_add_symbol(scanner,"language",         GINT_TO_POINTER(SYMBOL_LANGUAGE)); 
     
+    g_scanner_add_symbol(scanner,"cwda_type",      GINT_TO_POINTER(SYMBOL_CWDA_TYPE)); 
+    g_scanner_add_symbol(scanner,"cwda_device_s",  GINT_TO_POINTER(SYMBOL_CWDA_DEVICE)); 
     g_scanner_add_symbol(scanner,"cwda_hostname",  GINT_TO_POINTER(SYMBOL_CWDA_HOST)); 
     g_scanner_add_symbol(scanner,"cwda_udp_port",  GINT_TO_POINTER(SYMBOL_CWDA_UDP)); 
-    g_scanner_add_symbol(scanner,"cwda_device_s",  GINT_TO_POINTER(SYMBOL_CWDA_DEVICE_S)); 
     g_scanner_add_symbol(scanner,"cwda_speed",     GINT_TO_POINTER(SYMBOL_CWDA_SPEED)); 
-    g_scanner_add_symbol(scanner,"cwda_weight",     GINT_TO_POINTER(SYMBOL_CWDA_WEIGHT)); 
-    g_scanner_add_symbol(scanner,"cwda_spkdev",     GINT_TO_POINTER(SYMBOL_CWDA_SPKDEV)); 
-    g_scanner_add_symbol(scanner,"cwda_spkvol",     GINT_TO_POINTER(SYMBOL_CWDA_SPKVOL)); 
-    g_scanner_add_symbol(scanner,"cwda_pttdelay",     GINT_TO_POINTER(SYMBOL_CWDA_PTTDELAY)); 
+    g_scanner_add_symbol(scanner,"cwda_weight",    GINT_TO_POINTER(SYMBOL_CWDA_WEIGHT)); 
+    g_scanner_add_symbol(scanner,"cwda_spk",       GINT_TO_POINTER(SYMBOL_CWDA_SPK)); 
+    g_scanner_add_symbol(scanner,"cwda_vid",       GINT_TO_POINTER(SYMBOL_CWDA_VID)); 
+    g_scanner_add_symbol(scanner,"cwda_pid",       GINT_TO_POINTER(SYMBOL_CWDA_PID)); 
     
-	g_scanner_add_symbol(scanner,"ssbd_hostname",   GINT_TO_POINTER(SYMBOL_SSBD_HOST)); 
-	g_scanner_add_symbol(scanner,"ssbd_udp_port",   GINT_TO_POINTER(SYMBOL_SSBD_UDP)); 
-	g_scanner_add_symbol(scanner,"ssbd_dsp",        GINT_TO_POINTER(SYMBOL_SSBD_DSP)); 
+	g_scanner_add_symbol(scanner,"ssbd_type",       GINT_TO_POINTER(SYMBOL_SSBD_TYPE)); 
 	g_scanner_add_symbol(scanner,"ssbd_record",     GINT_TO_POINTER(SYMBOL_SSBD_RECORD)); 
+	g_scanner_add_symbol(scanner,"ssbd_maxmin",     GINT_TO_POINTER(SYMBOL_SSBD_MAXMIN)); 
+	g_scanner_add_symbol(scanner,"ssbd_diskfree",   GINT_TO_POINTER(SYMBOL_SSBD_DISKFREE)); 
 	g_scanner_add_symbol(scanner,"ssbd_format",     GINT_TO_POINTER(SYMBOL_SSBD_FORMAT)); 
 	g_scanner_add_symbol(scanner,"ssbd_channels",   GINT_TO_POINTER(SYMBOL_SSBD_CHANNELS)); 
 	g_scanner_add_symbol(scanner,"ssbd_samplerate", GINT_TO_POINTER(SYMBOL_SSBD_SAMPLERATE)); 
-	g_scanner_add_symbol(scanner,"ssbd_pipe",       GINT_TO_POINTER(SYMBOL_SSBD_PIPE)); 
-	g_scanner_add_symbol(scanner,"ssbd_command",    GINT_TO_POINTER(SYMBOL_SSBD_COMMAND)); 
 	g_scanner_add_symbol(scanner,"ssbd_template",   GINT_TO_POINTER(SYMBOL_SSBD_TEMPLATE)); 
+
+	g_scanner_add_symbol(scanner,"ssbd_pcm_play",   GINT_TO_POINTER(SYMBOL_SSBD_PCM_PLAY)); 
+	g_scanner_add_symbol(scanner,"ssbd_pcm_rec",    GINT_TO_POINTER(SYMBOL_SSBD_PCM_REC)); 
+	g_scanner_add_symbol(scanner,"ssbd_period_time",GINT_TO_POINTER(SYMBOL_SSBD_PERIOD_T)); 
+	g_scanner_add_symbol(scanner,"ssbd_buffer_time",GINT_TO_POINTER(SYMBOL_SSBD_BUFFER_T)); 
+	g_scanner_add_symbol(scanner,"ssbd_alsa_mixer", GINT_TO_POINTER(SYMBOL_SSBD_ALSA_MIXER)); 
+	g_scanner_add_symbol(scanner,"ssbd_alsa_src",   GINT_TO_POINTER(SYMBOL_SSBD_ALSA_SRC)); 
+
+	g_scanner_add_symbol(scanner,"ssbd_dsp",        GINT_TO_POINTER(SYMBOL_SSBD_DSP)); 
+	g_scanner_add_symbol(scanner,"ssbd_maxfrag",    GINT_TO_POINTER(SYMBOL_SSBD_MAXFRAG)); 
 	g_scanner_add_symbol(scanner,"ssbd_mixer",      GINT_TO_POINTER(SYMBOL_SSBD_MIXER)); 
+	g_scanner_add_symbol(scanner,"ssbd_oss_src",    GINT_TO_POINTER(SYMBOL_SSBD_OSS_SRC)); 
 	g_scanner_add_symbol(scanner,"ssbd_recsrc",     GINT_TO_POINTER(SYMBOL_SSBD_RECSRC)); 
+
+	g_scanner_add_symbol(scanner,"ssbd_hostname",   GINT_TO_POINTER(SYMBOL_SSBD_HOST)); 
+	g_scanner_add_symbol(scanner,"ssbd_udp_port",   GINT_TO_POINTER(SYMBOL_SSBD_UDP)); 
+
+	g_scanner_add_symbol(scanner,"ssbd_command",    GINT_TO_POINTER(SYMBOL_SSBD_COMMAND)); 
+	g_scanner_add_symbol(scanner,"ssbd_pipe",       GINT_TO_POINTER(SYMBOL_SSBD_PIPE)); 
 	
     g_scanner_add_symbol(scanner,"term",   		 		GINT_TO_POINTER(SYMBOL_TERM)); 
     g_scanner_add_symbol(scanner,"term_mode",			GINT_TO_POINTER(SYMBOL_TERM_MODE)); 
@@ -292,11 +314,18 @@ void add_symbols(GScanner *scanner){
 	g_scanner_add_symbol(scanner,"loglines",         GINT_TO_POINTER(SYMBOL_LOGLINES)); 
 	g_scanner_add_symbol(scanner,"skedshift",        GINT_TO_POINTER(SYMBOL_SKEDSHIFT)); 
 	g_scanner_add_symbol(scanner,"takeoff",          GINT_TO_POINTER(SYMBOL_TAKEOFF)); 
+	g_scanner_add_symbol(scanner,"operator",         GINT_TO_POINTER(SYMBOL_OPERATOR)); 
+	g_scanner_add_symbol(scanner,"global_operator",  GINT_TO_POINTER(SYMBOL_GLOBAL_OPERATOR)); 
+	g_scanner_add_symbol(scanner,"gfx_x",  			 GINT_TO_POINTER(SYMBOL_GFX_X)); 
+	g_scanner_add_symbol(scanner,"gfx_y",  			 GINT_TO_POINTER(SYMBOL_GFX_Y)); 
     
 	/* obsolete items */
     g_scanner_add_symbol(scanner,"cwda_device",    GINT_TO_POINTER(SYMBOL_OBSOLETE));
     g_scanner_add_symbol(scanner,"cwda_port",      GINT_TO_POINTER(SYMBOL_OBSOLETE));
     g_scanner_add_symbol(scanner,"cwda_speaker",   GINT_TO_POINTER(SYMBOL_OBSOLETE));
+    g_scanner_add_symbol(scanner,"cwda_spkdev",   GINT_TO_POINTER(SYMBOL_OBSOLETE));
+    g_scanner_add_symbol(scanner,"cwda_spkvol",   GINT_TO_POINTER(SYMBOL_OBSOLETE));
+    g_scanner_add_symbol(scanner,"cwda_pttdelay",   GINT_TO_POINTER(SYMBOL_OBSOLETE));
     g_scanner_add_symbol(scanner,"rota_type",   		    GINT_TO_POINTER(SYMBOL_OBSOLETE)); 
 	g_scanner_add_symbol(scanner,"rota_port",   		    GINT_TO_POINTER(SYMBOL_OBSOLETE)); 
 	g_scanner_add_symbol(scanner,"rota_saddr",   		    GINT_TO_POINTER(SYMBOL_OBSOLETE)); 
@@ -379,24 +408,41 @@ gint free_rc(void){
         FREE_IF_EXISTS(band,ok_section_multi);
         FREE_IF_EXISTS(band,adifband);
         FREE_IF_EXISTS(band,skedqrg);
+        g_free(band);
     }
     
-    g_ptr_array_free_all(cfg->bands);
+    g_ptr_array_free(cfg->bands, TRUE);
     
     for (i=0;i<cfg->cqs->len; i++){
         free_cq(g_ptr_array_index(cfg->cqs,i));
     }
     g_ptr_array_free(cfg->cqs, TRUE); /* YES, TRUE */
 
-	/*TODO ostatni gchar *cfg->  ???*/
+    FREE_IF_EXISTS(cfg,startband);
+    FREE_IF_EXISTS(cfg,operator);
+
+    FREE_IF_EXISTS(cfg,cwda_device);
+    FREE_IF_EXISTS(cfg,cwda_hostname);
+    
+    FREE_IF_EXISTS(cfg,ssbd_pcm_play);
+    FREE_IF_EXISTS(cfg,ssbd_pcm_rec);
+    FREE_IF_EXISTS(cfg,ssbd_hostname);
+    FREE_IF_EXISTS(cfg,ssbd_dsp);
+    FREE_IF_EXISTS(cfg,ssbd_template);
+    FREE_IF_EXISTS(cfg,ssbd_mixer);
+    FREE_IF_EXISTS(cfg,ssbd_alsa_mixer);
+    FREE_IF_EXISTS(cfg,ssbd_alsa_src);
+    FREE_IF_EXISTS(cfg,ssbd_oss_src);
+
     
     for (i=0;i<cfg->crotars->len; i++){
         rot = g_ptr_array_index(cfg->crotars, i);
 	    FREE_IF_EXISTS(rot,rot_filename);
 	    FREE_IF_EXISTS(rot,rot_hostname);
 	    FREE_IF_EXISTS(rot,rot_desc);
+        g_free(rot);
     }
-    g_ptr_array_free_all(cfg->bands);
+    g_ptr_array_free(cfg->crotars, TRUE);
     
 	
     for (i=0;i<cfg->sws->len;i++){
@@ -622,31 +668,47 @@ guint read_one_token(GScanner *scanner){
         STORE_RC_INT      (cfg->band, SYMBOL_QRG_MAX, qrg_max);
         STORE_RC_STRING   (cfg->band, SYMBOL_SKEDQRG, skedqrg);
 
+        STORE_RC_INT      (cfg, SYMBOL_CWDA_TYPE,    cwda_type);
+        STORE_RC_STRING   (cfg, SYMBOL_CWDA_DEVICE,  cwda_device);
         STORE_RC_STRING   (cfg, SYMBOL_CWDA_HOST,    cwda_hostname);
         STORE_RC_INT      (cfg, SYMBOL_CWDA_UDP,     cwda_udp_port);
-        STORE_RC_STRING   (cfg, SYMBOL_CWDA_DEVICE_S,  cwda_device_s);
         STORE_RC_INT      (cfg, SYMBOL_CWDA_SPEED,   cwda_speed);
         STORE_RC_INT      (cfg, SYMBOL_CWDA_WEIGHT,  cwda_weight);
-        STORE_RC_INT      (cfg, SYMBOL_CWDA_SPKDEV,  cwda_spkdev);
-        STORE_RC_INT      (cfg, SYMBOL_CWDA_SPKVOL,  cwda_spkvol);
-        STORE_RC_INT      (cfg, SYMBOL_CWDA_PTTDELAY,cwda_pttdelay);
+        STORE_RC_INT      (cfg, SYMBOL_CWDA_SPK,     cwda_spk);
+        STORE_RC_INT      (cfg, SYMBOL_CWDA_VID,     cwda_vid);
+        STORE_RC_INT      (cfg, SYMBOL_CWDA_PID,     cwda_pid);
 #if 0        
         STORE_RC_INT      (cfg, SYMBOL_CWDA_DEVICE,  cwda_device);  /* obsolete */
         STORE_RC_INT      (cfg, SYMBOL_CWDA_SPEAKER, cwda_speaker); /* obsolete */
         STORE_RC_INT      (cfg, SYMBOL_CWDA_PORT,    cwda_port);    /* obsolete */
 #endif        
-		STORE_RC_STRING   (cfg, SYMBOL_SSBD_HOST,       ssbd_hostname);
-        STORE_RC_INT      (cfg, SYMBOL_SSBD_UDP,        ssbd_udp_port);
-		STORE_RC_STRING   (cfg, SYMBOL_SSBD_DSP,        ssbd_dsp);
+		STORE_RC_INT      (cfg, SYMBOL_SSBD_TYPE,       ssbd_type);
         STORE_RC_INT      (cfg, SYMBOL_SSBD_RECORD,     ssbd_record);
+        STORE_RC_INT      (cfg, SYMBOL_SSBD_MAXMIN,     ssbd_maxmin);
+        STORE_RC_INT      (cfg, SYMBOL_SSBD_DISKFREE,   ssbd_diskfree);
         STORE_RC_INT      (cfg, SYMBOL_SSBD_FORMAT,     ssbd_format);
         STORE_RC_INT      (cfg, SYMBOL_SSBD_CHANNELS,   ssbd_channels);
         STORE_RC_INT      (cfg, SYMBOL_SSBD_SAMPLERATE, ssbd_samplerate);
+        STORE_RC_STRING   (cfg, SYMBOL_SSBD_TEMPLATE, 	ssbd_template);
+
+		STORE_RC_STRING   (cfg, SYMBOL_SSBD_PCM_PLAY,   ssbd_pcm_play);
+		STORE_RC_STRING   (cfg, SYMBOL_SSBD_PCM_REC,    ssbd_pcm_rec);
+        STORE_RC_INT      (cfg, SYMBOL_SSBD_PERIOD_T,   ssbd_period_time);
+        STORE_RC_INT      (cfg, SYMBOL_SSBD_BUFFER_T,   ssbd_buffer_time);
+		STORE_RC_STRING   (cfg, SYMBOL_SSBD_ALSA_MIXER, ssbd_alsa_mixer);
+		STORE_RC_STRING   (cfg, SYMBOL_SSBD_ALSA_SRC,   ssbd_alsa_src);
+        
+		STORE_RC_STRING   (cfg, SYMBOL_SSBD_DSP,        ssbd_dsp);
+        STORE_RC_INT      (cfg, SYMBOL_SSBD_MAXFRAG, 	ssbd_maxfrag);
+        STORE_RC_STRING   (cfg, SYMBOL_SSBD_MIXER, 	    ssbd_mixer);
+		STORE_RC_STRING   (cfg, SYMBOL_SSBD_OSS_SRC,    ssbd_oss_src);
+        STORE_RC_INT      (cfg, SYMBOL_SSBD_RECSRC, 	ssbd_recsrc);
+
+		STORE_RC_STRING   (cfg, SYMBOL_SSBD_HOST,       ssbd_hostname);
+        STORE_RC_INT      (cfg, SYMBOL_SSBD_UDP,        ssbd_udp_port);
+
 /*		STORE_RC_INT      (cfg, SYMBOL_SSBD_PIPE, 		ssbd_pipe);
 		STORE_RC_STRING   (cfg, SYMBOL_SSBD_COMMAND, 	ssbd_command);*/
-        STORE_RC_STRING   (cfg, SYMBOL_SSBD_TEMPLATE, 	ssbd_template);
-        STORE_RC_STRING   (cfg, SYMBOL_SSBD_MIXER, 	    ssbd_mixer);
-        STORE_RC_INT      (cfg, SYMBOL_SSBD_RECSRC, 	ssbd_recsrc);
 
         case SYMBOL_ROTAR:
             cfg->crotar = get_config_rotar_by_number(cfg->crotars, scanner->value.v_int);
@@ -697,9 +759,12 @@ guint read_one_token(GScanner *scanner){
                 toff->to=atoi(items[1]);
                 toff->value=atoi(items[2]);
                 g_ptr_array_add(cfg->takeoff, toff); 
-            
             }
             break;
+        STORE_RC_STRING(cfg, SYMBOL_OPERATOR, operator);
+        STORE_RC_INT   (cfg, SYMBOL_GLOBAL_OPERATOR, global_operator);
+        STORE_RC_INT   (cfg, SYMBOL_GFX_X, gfx_x);
+        STORE_RC_INT   (cfg, SYMBOL_GFX_Y, gfx_y);
         
 
     }
@@ -712,15 +777,18 @@ guint read_one_token(GScanner *scanner){
     return G_TOKEN_NONE;
 }
 
-gint read_rc_file(gchar *filename){
+gint read_rc_file(gchar *filename, const char *text){
    FILE *f;
    GScanner *scanner;
    guint expected;
    gint errors;
 
 /*  dbg("read_rc_file(%s)\n", filename);*/
-   f=fopen(filename,"rt");
-   if (!f) return -1;
+   f=NULL;
+   if (filename){
+       f=fopen(filename,"rt");
+       if (!f) return -1;
+   }
 
    scanner = g_scanner_new(NULL);
    
@@ -729,13 +797,20 @@ gint read_rc_file(gchar *filename){
 
    add_symbols(scanner);
    
-   g_scanner_input_file(scanner, fileno(f));
-   scanner->input_name = filename;
+   if (filename){
+       g_scanner_input_file(scanner, fileno(f));
+       scanner->input_name = filename;
+   }else{
+       g_scanner_input_text(scanner, text, strlen(text));
+       filename="internal_default.rc";
+       scanner->input_name = filename;
+   }
  
    errors=0;
    do{
        expected = read_one_token(scanner);
-       if (expected == G_TOKEN_SYMBOL)  errors++;
+       if (expected == G_TOKEN_SYMBOL)  
+		   errors++;
        
        /*dbg("expected=%d\n",expected);*/
        g_scanner_peek_next_token(scanner);
@@ -745,10 +820,10 @@ gint read_rc_file(gchar *filename){
 
 
    g_scanner_destroy(scanner);    
-   fclose(f); 
+   if (f) fclose(f); 
 /*   dbg("call %s\n", cfg->call);*/
    if (errors){
-       log_addf(TEXT(T_ERR_RC_DSS),errors, filename);
+       log_addf(VTEXT(T_ERR_RC_DSS),errors, filename);
        dbg("Found %d error%s while reading %s\n",errors, errors>1?"s":"",filename);
        return errors;
    }
@@ -788,7 +863,7 @@ gint read_rc_line(gchar *str){
 
    g_scanner_destroy(scanner);    
    if (errors){
-       log_addf(TEXT(T_ERR_RC_DS),errors );
+       log_addf(VTEXT(T_ERR_RC_DS),errors );
    }
    return 0;
 }
@@ -798,18 +873,18 @@ void terminate_bh(void *xxx){
     terminate=1;
 }
 
-void show_rc_error(void *xxx){
-    msg_box(term, NULL, 
-            TEXT(T_ERROR), AL_CENTER, 
-            TEXT(T_CANT_LOAD_CFG), NULL, 1, 
-            TEXT(T_CANCEL), terminate_bh, B_ENTER | B_ESC);
+void show_rc_error(union cba_t cba){
+    msg_box(NULL, 
+            VTEXT(T_ERROR), AL_CENTER, 
+            VTEXT(T_CANT_LOAD_CFG), NULL, 1, 
+            VTEXT(T_CANCEL), terminate_bh, B_ENTER | B_ESC);
 }
 
-void show_rc_warning(void *xxx){
-    msg_box(term, NULL, 
-            TEXT(T_WARNING), AL_CENTER, 
-            TEXT(T_ERR_RC), NULL, 1, 
-            TEXT(T_CANCEL), NULL, B_ENTER | B_ESC);
+void show_rc_warning(union cba_t cba){
+    msg_box(NULL, 
+            VTEXT(T_WARNING), AL_CENTER, 
+            VTEXT(T_ERR_RC), NULL, 1, 
+            VTEXT(T_CANCEL), NULL, B_ENTER | B_ESC);
 }
 
 gint read_rc_files(void){
@@ -818,28 +893,44 @@ gint read_rc_files(void){
     
     cfg->loglines=3;
     cfg->skedshift=5;
+    cfg->global_operator=1;
+    
+    
     
     fail1=0;
     
     user = g_strconcat(getenv("HOME"), RC_USER, NULL);
-    fail2=read_rc_file(user);
+    fail2=read_rc_file(user, NULL);
     g_free(user);      
     
     if (fail2==-1){  /* reads global config only if no user config exists */
-      fail1=read_rc_file(RC_MAIN);
+      fail1=read_rc_file(NULL, defrc);
     }
     
     local = g_strconcat(getenv("HOME"), RC_LOCAL, NULL);
-    fail3=read_rc_file(local);
+    fail3=read_rc_file(local, NULL);
     g_free(local);      
     
     
     
     if (!cfg->pcall || !cfg->pwwlo) 
-        register_bottom_half(show_rc_error, NULL);
+        register_bottom_half(show_rc_error, CBA0);
     else
-        if (fail1>0 || fail2>0 || fail3>0) register_bottom_half(show_rc_warning, NULL);
+        if (fail1>0 || fail2>0 || fail3>0) register_bottom_half(show_rc_warning, CBA0);
         
+	if (cfg->gfx_x<=0) cfg->gfx_x=106*FONT_W;
+	if (cfg->gfx_y<=0) cfg->gfx_y=42*FONT_H; 
+        
+    if (!cfg->ssbd_pcm_play)    cfg->ssbd_pcm_play  =g_strdup("hw:0,0");
+    if (!cfg->ssbd_pcm_rec)     cfg->ssbd_pcm_rec   =g_strdup("hw:0,0");
+    if (!cfg->ssbd_alsa_mixer)  cfg->ssbd_alsa_mixer=g_strdup("hw:0");
+    if (!cfg->ssbd_alsa_src)    cfg->ssbd_alsa_src  =g_strdup("Mic");
+    if (!cfg->cwda_vid)         cfg->cwda_vid = 0xa600;
+    if (!cfg->cwda_pid)         cfg->cwda_pid = 0xe110;
+        
+#ifdef __CYGWIN__    
+    cfg->ssbd_type=0; /* Alsa not available under cygwin */
+#endif        
         
     
     return 0;
@@ -858,6 +949,8 @@ void save_rc_string(GString *gs){
     g_string_sprintfa(gs, "pcall = \"%s\"\n", U(cfg->pcall));
     g_string_sprintfa(gs, "pwwlo = \"%s\"\n", U(cfg->pwwlo));
     g_string_sprintfa(gs, "pexch = \"%s\"\n", U(cfg->pexch));
+	g_string_sprintfa(gs, "operator = \"%s\"\n", U(cfg->operator));
+	g_string_sprintfa(gs, "global_operator = %d\n", cfg->global_operator);
     g_string_sprintfa(gs, "\n");
     g_string_sprintfa(gs, "padr1 = \"%s\"\n", S(cfg->padr1));
     g_string_sprintfa(gs, "padr2 = \"%s\"\n", S(cfg->padr2));
@@ -898,30 +991,42 @@ void save_rc_string(GString *gs){
     g_string_sprintfa(gs, "trace_qsos  = %d\n", cfg->trace_qsos);
     g_string_sprintfa(gs, "language= %d\n", current_language);
     g_string_sprintfa(gs, "\n");
+    g_string_sprintfa(gs, "cwda_type     = %d\n", cfg->cwda_type);
+    g_string_sprintfa(gs, "cwda_device_s = \"%s\"\n", S(cfg->cwda_device));
     g_string_sprintfa(gs, "cwda_hostname = \"%s\"\n", U(cfg->cwda_hostname));
     g_string_sprintfa(gs, "cwda_udp_port = %d\n", cfg->cwda_udp_port);
-    g_string_sprintfa(gs, "cwda_device_s = \"%s\"\n", S(cfg->cwda_device_s));
     g_string_sprintfa(gs, "cwda_speed    = %d\n", cfg->cwda_speed);
     g_string_sprintfa(gs, "cwda_weight   = %d\n", cfg->cwda_weight);
-    g_string_sprintfa(gs, "cwda_spkdev   = %d\n", cfg->cwda_spkdev);
-    g_string_sprintfa(gs, "cwda_spkvol   = %d\n", cfg->cwda_spkvol);
-    g_string_sprintfa(gs, "cwda_pttdelay = %d\n", cfg->cwda_pttdelay);
-    /*g_string_sprintfa(gs, "cwda_device   = %d\n", cfg->cwda_device);*/
-    /*g_string_sprintfa(gs, "cwda_speaker  = %d\n", cfg->cwda_speaker);*/
-    /*g_string_sprintfa(gs, "cwda_port     = 0x%x\n", cfg->cwda_port);*/
+    g_string_sprintfa(gs, "cwda_spk      = %d\n", cfg->cwda_spk);
+    g_string_sprintfa(gs, "cwda_vid      = 0x%04x\n", cfg->cwda_vid);
+    g_string_sprintfa(gs, "cwda_pid      = 0x%04x\n", cfg->cwda_pid);
     g_string_sprintfa(gs, "\n");
-    g_string_sprintfa(gs, "ssbd_hostname   = \"%s\"\n", S(cfg->ssbd_hostname));
-    g_string_sprintfa(gs, "ssbd_udp_port   = %d\n", cfg->ssbd_udp_port);
-    g_string_sprintfa(gs, "ssbd_dsp        = \"%s\"\n", S(cfg->ssbd_dsp));
+    g_string_sprintfa(gs, "ssbd_type       = %d\n", cfg->ssbd_type);
     g_string_sprintfa(gs, "ssbd_record     = %d\n", cfg->ssbd_record);
+    g_string_sprintfa(gs, "ssbd_maxmin     = %d\n", cfg->ssbd_maxmin);
+    g_string_sprintfa(gs, "ssbd_diskfree   = %d\n", cfg->ssbd_diskfree);
     g_string_sprintfa(gs, "ssbd_format     = 0x%x\n", cfg->ssbd_format);
     g_string_sprintfa(gs, "ssbd_channels   = %d\n", cfg->ssbd_channels);
     g_string_sprintfa(gs, "ssbd_samplerate = %d\n", cfg->ssbd_samplerate);
+    g_string_sprintfa(gs, "ssbd_template   = \"%s\"\n", S(cfg->ssbd_template));
+    g_string_sprintfa(gs, "\n");
+    g_string_sprintfa(gs, "ssbd_dsp        = \"%s\"\n", S(cfg->ssbd_dsp));
+    g_string_sprintfa(gs, "ssbd_maxfrag    = %d\n", cfg->ssbd_maxfrag);
+    g_string_sprintfa(gs, "ssbd_mixer      = \"%s\"\n", S(cfg->ssbd_mixer));
+    g_string_sprintfa(gs, "ssbd_oss_src    = \"%s\"\n", S(cfg->ssbd_oss_src));
+    /*g_string_sprintfa(gs, "ssbd_recsrc     = 0x%x\n", cfg->ssbd_recsrc);*/
+    g_string_sprintfa(gs, "\n");
+    g_string_sprintfa(gs, "ssbd_pcm_play    = \"%s\"\n", S(cfg->ssbd_pcm_play));
+    g_string_sprintfa(gs, "ssbd_pcm_rec     = \"%s\"\n", S(cfg->ssbd_pcm_rec));
+    g_string_sprintfa(gs, "ssbd_period_time = %d\n", cfg->ssbd_period_time);
+    g_string_sprintfa(gs, "ssbd_buffer_time = %d\n", cfg->ssbd_buffer_time);
+    g_string_sprintfa(gs, "ssbd_alsa_mixer  = \"%s\"\n", S(cfg->ssbd_alsa_mixer));
+    g_string_sprintfa(gs, "ssbd_alsa_src    = \"%s\"\n", S(cfg->ssbd_alsa_src));
+    g_string_sprintfa(gs, "\n");
+    g_string_sprintfa(gs, "ssbd_hostname   = \"%s\"\n", S(cfg->ssbd_hostname));
+    g_string_sprintfa(gs, "ssbd_udp_port   = %d\n", cfg->ssbd_udp_port);
 /*    g_string_sprintfa(gs, "ssbd_pipe       = %d\n", cfg->ssbd_pipe);
     g_string_sprintfa(gs, "ssbd_command    = \"%s\"\n", cfg->ssbd_command);*/
-    g_string_sprintfa(gs, "ssbd_template   = \"%s\"\n", S(cfg->ssbd_template));
-    g_string_sprintfa(gs, "ssbd_mixer      = \"%s\"\n", S(cfg->ssbd_mixer));
-    g_string_sprintfa(gs, "ssbd_recsrc     = 0x%x\n", cfg->ssbd_recsrc);
     g_string_sprintfa(gs, "\n\n");
     
     for (i=0; i<cfg->crotars->len; i++){
@@ -943,6 +1048,8 @@ void save_rc_string(GString *gs){
 	g_string_sprintfa(gs, "loglines        = %d\n", cfg->loglines);
 	g_string_sprintfa(gs, "skedshift       = %d\n", cfg->skedshift);
     g_string_sprintfa(gs, "startband       = \"%s\"\n", S(cfg->startband));
+	g_string_sprintfa(gs, "gfx_x           = %d\n", cfg->gfx_x);
+	g_string_sprintfa(gs, "gfx_y           = %d\n", cfg->gfx_y);
     for (i=0; i<cfg->takeoff->len;i++){
         struct takeoff *toff;
         toff=(struct takeoff*)g_ptr_array_index(cfg->takeoff, i);
@@ -1109,17 +1216,17 @@ char *parse_options(int argc, char **argv){
             {0, 0, 0, 0}
         };
 
-        c = getopt_long (argc, argv, "dhv?", long_options, &option_index);
+        c = getopt_long (argc, argv, "dghiktv?", long_options, &option_index);
         if (c == -1) break;
 
         switch (c){
             case 0:
-                printf ("option %s", long_options[option_index].name);
+/*              printf ("option %s", long_options[option_index].name);*/
                 if (optarg) printf (" with arg %s", optarg);
                 printf ("\n");
                 break;
 
-           case 'd':
+            case 'd':
                 if (optarg) {
                     debug_type=1;
                     debug_filename=optarg;
@@ -1128,17 +1235,33 @@ char *parse_options(int argc, char **argv){
                 }
                 break;
 
-           case 'h':     
-           case '?':
-                printf ("option h\n");
+            case 'g':
+                opt_g++;
+                break;
+                
+            case 'h':     
+            case '?':
+                /*printf ("option h\n");*/
+                break;
+
+            case 'i':
+                opt_i++;
+                break;
+
+            case 'k':
+                debug_keyboard=1;
+                break;
+                            
+            case 't':
+                opt_t++;
                 break;
            
-           case 'v':
-                printf ("option v\n");
+            case 'v':
+                /*printf ("option v\n");*/
                 break;
 
 
-           default:
+            default:
                 printf ("?? getopt returned character code 0%o ??\n", c);
         }
     }
