@@ -3,19 +3,9 @@
     Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>
     and authors of web browser Links 0.96
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public
-    License along with this library; if not, write to the Free
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    This program is free software; you can redistribute it and/or                                                        
+    modify it under the terms of the GNU General Public License                                                          
+    version 2 as published by the Free Software Foundation.
 
 */
 
@@ -26,6 +16,8 @@ struct terminal *term;
 
 int hard_write(int fd, char *p, int l)
 {
+    if (fd<0) return -1;
+    
     int w = 1;
     int t = 0;
     while (l > 0 && w) {
@@ -77,7 +69,7 @@ void set_cwd(char *path)
     if (path) while (chdir(path) && errno == EINTR) ;
 }
 
-struct list_head terminals = {&terminals, &terminals};
+/*struct list_head terminals = {&terminals, &terminals};*/
 
 void alloc_term_screen(struct terminal *term, int x, int y)
 {
@@ -95,14 +87,14 @@ void alloc_term_screen(struct terminal *term, int x, int y)
     }
 }
 
-void in_term(struct terminal *);
-void destroy_terminal(struct terminal *);
+void in_term(union cba_t);
 void check_if_no_terminal(void);
 
-void clear_terminal(struct terminal *term)
+void clear_terminal(void)
 {
-    fill_area(term, 0, 0, term->x, term->y, ' ');
-    set_cursor(term, 0, 0, 0, 0);
+    if (!term) return;
+    fill_area(0, 0, term->x, term->y, ' ');
+    set_cursor(0, 0, 0, 0);
 }
 
 void redraw_terminal_ev(struct terminal *term, int e)
@@ -110,34 +102,38 @@ void redraw_terminal_ev(struct terminal *term, int e)
     struct window *win;
     struct event ev = {0, 0, 0, 0};
     
+    if (!term) return;
     ev.ev = e;
     ev.x = term->x;
     ev.y = term->y;
-    clear_terminal(term);
+    clear_terminal();
     term->redrawing = 2;
     foreachback(win, term->windows) win->handler(win, &ev, 0);
     term->redrawing = 0;
     
 }
 
-void redraw_terminal(struct terminal *term)
+void redraw_terminal(union cba_t cba)
 {
-  /*  dbg("redraw_terminal\n");*/
+    /*if (!term) return;*/
     redraw_terminal_ev(term, EV_REDRAW);
 }
 
-void redraw_terminal_all(struct terminal *term)
+void redraw_terminal_all(void)
 {
+    if (!term) return;
     redraw_terminal_ev(term, EV_RESIZE);
 }
 
-void redraw_later(struct terminal *term)
+void redraw_later(void)
 {
-    register_bottom_half((void (*)(void *))redraw_terminal, (void *)term);
+    if (!term) return;
+    register_bottom_half(redraw_terminal, CBA0);
 }
 
-void erase_screen(struct terminal *term)
+void erase_screen(void)
 {
+    if (!term) return;
     if (!term->master || !is_blocked()) {
         if (term->master) want_draw();
         hard_write(term->fdout, "\033[2J\033[1;1H", 10);
@@ -145,18 +141,19 @@ void erase_screen(struct terminal *term)
     }
 }
 
-void redraw_terminal_cls(struct terminal *term)
+void redraw_terminal_cls()
 {
-    erase_screen(term);
+    if (!term) return;
+    erase_screen();
     alloc_term_screen(term, term->x, term->y);
-    redraw_terminal_all(term);
+    redraw_terminal_all();
 }
 
-void cls_redraw_all_terminals()
+/*void cls_redraw_all_terminals()
 {
     struct terminal *term;
     foreach(term, terminals) redraw_terminal_cls(term);
-}
+} */
 
 void redraw_from_window(struct window *win)
 {
@@ -190,7 +187,7 @@ void redraw_below_window(struct window *win)
     term->redrawing = tr;
 }
 
-void add_window_at_pos(struct terminal *term, void (*handler)(struct window *, struct event *, int), void *data, struct window *at)
+void add_window_at_pos(void (*handler)(struct window *, struct event *, int), void *data, struct window *at)
 {
     struct event ev = {EV_INIT, 0, 0, 0};
     struct window *win;
@@ -208,9 +205,9 @@ void add_window_at_pos(struct terminal *term, void (*handler)(struct window *, s
     win->handler(win, &ev, 0);
 }
 
-void add_window(struct terminal *term, void (*handler)(struct window *, struct event *, int), void *data)
+void add_window(void (*handler)(struct window *, struct event *, int), void *data)
 {
-    add_window_at_pos(term, handler, data, (struct window *)&term->windows);
+    add_window_at_pos(handler, data, (struct window *)&term->windows);
 }
 
 void delete_window(struct window *win)
@@ -219,7 +216,7 @@ void delete_window(struct window *win)
     win->handler(win, &ev, 1);
     del_from_list(win);
     if (win->data) mem_free(win->data);
-    redraw_terminal(win->term);
+    redraw_terminal(CBA0);
     mem_free(win);
 }
 
@@ -247,7 +244,7 @@ void get_parent_ptr(struct window *win, int *x, int *y)
     }
 }
 
-struct window *get_root_window(struct terminal *term)
+struct window *get_root_window()
 {
     if (list_empty(term->windows)) {
         internal("terminal has no windows");
@@ -288,15 +285,15 @@ void empty_window_handler(struct window *win, struct event *ev, int fwd)
     if (n->next != n) n->handler(n, ev, fwd);
 }
 
-void add_empty_window(struct terminal *term, void (*fn)(void *), void *data)
+/*void add_empty_window(void (*fn)(void *), void *data)
 {
     struct ewd *ewd;
     if (!(ewd = mem_alloc(sizeof(struct ewd)))) return;
     ewd->fn = fn;
     ewd->data = data;
     ewd->b = 0;
-    add_window(term, empty_window_handler, ewd);
-}
+    add_window(empty_window_handler, ewd);
+} */
 
 void free_term_specs()
 {
@@ -329,9 +326,9 @@ struct term_spec *new_term_spec(char *term)
 
 void sync_term_specs()
 {
-    struct terminal *term;
-    foreach (term, terminals) term->spec = get_term_spec(term->term);
-}
+    if (!term) return;
+    term->spec = get_term_spec(term->term);
+} 
 
 struct terminal *init_term(int fdin, int fdout, 
         void (*root_window)(struct window *, struct event *, int))
@@ -339,7 +336,7 @@ struct terminal *init_term(int fdin, int fdout,
     struct terminal *term;
     struct window *win;
     if (!(term = mem_alloc(sizeof (struct terminal)))) {
-        check_if_no_terminal();
+        terminate=1;
         return NULL;
     }
     memset(term, 0, sizeof(struct terminal));
@@ -365,16 +362,16 @@ struct terminal *init_term(int fdin, int fdout,
     init_list(term->windows);
     if (!(win = mem_alloc(sizeof (struct window)))) {
         mem_free(term);
-        check_if_no_terminal();
+        terminate=1;
         return NULL;
     }
     win->handler = root_window;
     win->data = NULL;
     win->term = term;
     add_to_list(term->windows, win);
-    /*alloc_term_screen(term, 80, 25);*/
-    add_to_list(terminals, term);
-    set_handlers(fdin, (void (*)(void *))in_term, NULL, (void (*)(void *))destroy_terminal, term);
+    if (fdin!=-1){
+        set_handlers(fdin, in_term, NULL, destroy_terminal, CBA0);
+    }
     return term;
 }
 
@@ -404,47 +401,85 @@ static void term_send_ucs(struct terminal *term, struct event *ev, unicode_val u
 
 #define get_bool_opt(s, opt) (s->opt == 1 ? 1 : 0)
 
-void in_term(struct terminal *term)
+void in_term(union cba_t cba)
 {
     struct event *ev;
     int r;
-    char *iq;
+    char *iq, *c;
     if (!(iq = mem_realloc(term->input_queue, term->qlen + ALLOC_GR))) {
-        destroy_terminal(term);
+        destroy_terminal(CBA0);
         return;
     }
     term->input_queue = iq;
     if ((r = read(term->fdin, iq + term->qlen, ALLOC_GR)) <= 0) {
         if (r == -1 && errno != ECONNRESET) error("ERROR: error %d on terminal: could not read event", errno);
-        destroy_terminal(term);
+        destroy_terminal(CBA0);
         return;
     }
+#ifdef HAVE_SDL    
+    if (sdl){
+        LOCK(sdl->eventpipestate);
+        sdl->eventpipestate-=r;
+/*        dbg("r %d\n", sdl->eventpipestate);*/
+        UNLOCK(sdl->eventpipestate);
+    }
+#endif
+
+    
     term->qlen += r;
     test_queue:
     if (term->qlen < sizeof(struct event)) return;
     ev = (struct event *)iq;
     
-    dbg("in_term [%d,%d,%d,%d]\n",ev->ev,ev->x,ev->y,ev->b);
+  /*  dbg("in_term [%d,%d,%d,%d]\n",ev->ev,ev->x,ev->y,ev->b);*/
+/*    dbg("in_term qlen=%d\n", term->qlen);*/
+
+    if (term->qlen > sizeof(struct event) &&    /* more than one event */
+        ev->ev==EV_KBD && ev->x==KBD_TAB){       /* processing TAB keypress */
+        for (c=iq+sizeof(struct event); c<iq+term->qlen;c+=sizeof(struct event)){
+            ev = (struct event *)c;
+            /*dbg("in_term c=%d [%d,%d,%d,%d]\n",(c-iq)/sizeof(struct event),ev->ev,ev->x,ev->y,ev->b);*/
+            if (ev->ev!=EV_KBD || ev->x!=KBD_TAB) continue;
+            ev->ev=EV_SKIP;
+            
+        }
+    }
+    
+    
+    ev = (struct event *)iq;
     
     r = sizeof(struct event);
-    if (ev->ev != EV_INIT && ev->ev != EV_RESIZE && ev->ev != EV_REDRAW && ev->ev != EV_KBD && ev->ev != EV_MOUSE && ev->ev != EV_ABORT) {
+    if (ev->ev != EV_INIT && ev->ev != EV_RESIZE && ev->ev != EV_REDRAW && ev->ev != EV_KBD && ev->ev != EV_MOUSE && ev->ev != EV_ABORT && ev->ev!=EV_SDLRESIZE && ev->ev!=EV_SKIP) {
         error("ERROR: error on terminal: bad event %d", ev->ev);
         goto mm;
     }
     if (ev->ev == EV_INIT) {
-        int init_len;
-        if (term->qlen < sizeof(struct event) + MAX_TERM_LEN + MAX_CWD_LEN + 2 * sizeof(int)) return;
-        init_len = *(int *)(iq + sizeof(struct event) + MAX_TERM_LEN + MAX_CWD_LEN + sizeof(int));
-        if (term->qlen < sizeof(struct event) + MAX_TERM_LEN + MAX_CWD_LEN + 2 * sizeof(int) + init_len) return;
-        memcpy(term->term, iq + sizeof(struct event), MAX_TERM_LEN);
-        term->term[MAX_TERM_LEN - 1] = 0;
-        memcpy(term->cwd, iq + sizeof(struct event) + MAX_TERM_LEN, MAX_CWD_LEN);
-        term->cwd[MAX_CWD_LEN - 1] = 0;
-        term->environment = *(int *)(iq + sizeof(struct event) + MAX_TERM_LEN + MAX_CWD_LEN);
-        ev->b = (long)(iq + sizeof(struct event) + MAX_TERM_LEN + MAX_CWD_LEN + sizeof(int));
-        r = sizeof(struct event) + MAX_TERM_LEN + MAX_CWD_LEN + 2 * sizeof(int) + init_len;
+        char *ts;
+        int xwin;
+        
+        xwin = is_xterm() * ENV_XWIN + can_twterm() * ENV_TWIN + (!!getenv("STY")) * ENV_SCREEN + get_system_env();
+        if (!(ts = getenv("TERM"))) ts = "";
+        strcpy(term->term, ts);
+    
+        if (!(ts = get_cwd()) && !(ts = stracpy(""))) goto neni_pamet;
+        strcpy(term->cwd, ts);
+        mem_free(ts);
+        neni_pamet:;
+        term->environment=xwin;
+        
         sync_term_specs();
     }
+    if (ev->ev == EV_SDLRESIZE){
+        dbg("EV_SDLRESIZE(%d,%d) (%d,%d)\n", ev->x, ev->y, ev->mx, ev->my);
+#ifdef HAVE_SDL        
+        if (sdl) {
+			cfg->gfx_x=ev->mx;
+			cfg->gfx_y=ev->my;
+			sdl_setvideomode(ev->mx, ev->my, 0);
+		}
+#endif
+        resize_terminal(CBA0);
+    }
     if (ev->ev == EV_REDRAW || ev->ev == EV_RESIZE || ev->ev == EV_INIT) {
         struct window *win;
         send_redraw:
@@ -453,8 +488,8 @@ void in_term(struct terminal *term)
             goto mm;
         }
         alloc_term_screen(term, ev->x, ev->y);
-        clear_terminal(term);
-        erase_screen(term);
+        clear_terminal();
+        erase_screen();
         term->redrawing = 1;
         foreachback(win, term->windows) win->handler(win, ev, 0);
         term->redrawing = 0;
@@ -511,7 +546,7 @@ void in_term(struct terminal *term)
 			term_send_ucs(term, ev, UCS_NO_CHAR);
 		}
     }
-    if (ev->ev == EV_ABORT) destroy_terminal(term);
+    if (ev->ev == EV_ABORT) destroy_terminal(CBA0);
     /*redraw_screen(term);*/
     mm:
     if (term->qlen == r) term->qlen = 0;
@@ -630,13 +665,13 @@ char frame_restrict[48] = {
 	cx++;                                                                   \
 }                                                                           \
 
-void redraw_all_terminals()
+/*void redraw_all_terminals()
 {
     struct terminal *term;
     foreach(term, terminals) redraw_screen(term);
-}
+}*/
 
-void redraw_screen(struct terminal *term)
+void term_redraw_screen()
 {
     int x, y, p = 0;
     int cx = -1, cy = -1;
@@ -692,37 +727,40 @@ void redraw_screen(struct terminal *term)
     term->dirty = 0;
 }
 
-void destroy_terminal(struct terminal *term)
+void destroy_terminal(union cba_t cba)
 {
+    if (!term) return;
+    terminate=1;
     while ((term->windows.next) != &term->windows) delete_window(term->windows.next);
     /*if (term->cwd) mem_free(term->cwd);*/
     if (term->title) mem_free(term->title);
     mem_free(term->screen);
+    term->screen=NULL;
     mem_free(term->last_screen);
-    set_handlers(term->fdin, NULL, NULL, NULL, NULL);
+    term->screen=NULL;
+    set_handlers(term->fdin, NULL, NULL, NULL, CBA0);
     mem_free(term->input_queue);
     if (term->blocked != -1) {
         close(term->blocked);
-        set_handlers(term->blocked, NULL, NULL, NULL, NULL);
+        set_handlers(term->blocked, NULL, NULL, NULL, CBA0);
     }
-    del_from_list(term);
     close(term->fdin);
     if (term->fdout != 1) {
         if (term->fdout != term->fdin) close(term->fdout);
     } else {
-        unhandle_terminal_signals(term);
+        unhandle_terminal_signals();
         free_all_itrms();
-#ifndef NO_FORK_ON_EXIT
+/*#ifndef NO_FORK_ON_EXIT
         if (!list_empty(terminals)) {
             if (fork()) exit(0);
         }
-#endif
+#endif */
     }
     mem_free(term);
-    check_if_no_terminal();
+    term=NULL;
 }
 
-void destroy_all_terminals()
+/*void destroy_all_terminals()
 {
     struct terminal *term;
     while ((void *)(term = terminals.next) != &terminals) destroy_terminal(term);
@@ -732,83 +770,91 @@ void check_if_no_terminal(void)
 {
     if (!list_empty(terminals)) return;
     terminate = 1;
+} */
+
+void set_char(int x, int y, unsigned c)
+{
+    term->dirty = 1;
+    if (x >= 0 && x < term->x && y >= 0 && y < term->y) term->screen[x + term->x * y] = c;
 }
 
-void set_char(struct terminal *t, int x, int y, unsigned c)
+unsigned get_char(int x, int y)
 {
-    t->dirty = 1;
-    if (x >= 0 && x < t->x && y >= 0 && y < t->y) t->screen[x + t->x * y] = c;
-}
-
-unsigned get_char(struct terminal *t, int x, int y)
-{
-    if (x >= t->x) x = t->x - 1;
+    if (x >= term->x) x = term->x - 1;
     if (x < 0) x = 0;
-    if (y >= t->y) y = t->y - 1;
+    if (y >= term->y) y = term->y - 1;
     if (y < 0) y = 0;
-    return t->screen[x + t->x * y];
+    return term->screen[x + term->x * y];
 }
 
-void set_color_(struct terminal *t, int x, int y, unsigned c)
+void set_color(int x, int y, unsigned c)
 {
-    t->dirty = 1;
-    if (x >= 0 && x < t->x && y >= 0 && y < t->y) t->screen[x + t->x * y] = (t->screen[x + t->x * y] & 0x80ff) | (c & ~0x80ff);
+    term->dirty = 1;
+    if (x >= 0 && x < term->x && y >= 0 && y < term->y) term->screen[x + term->x * y] = (term->screen[x + term->x * y] & 0x80ff) | (c & ~0x80ff);
 }
 
-void set_only_char(struct terminal *t, int x, int y, unsigned c)
+void set_only_char(int x, int y, unsigned c)
 {
-    t->dirty = 1;
-    if (x >= 0 && x < t->x && y >= 0 && y < t->y) t->screen[x + t->x * y] = (t->screen[x + t->x * y] & ~0x80ff) | (c & 0x80ff);
+    term->dirty = 1;
+    if (x >= 0 && x < term->x && y >= 0 && y < term->y) term->screen[x + term->x * y] = (term->screen[x + term->x * y] & ~0x80ff) | (c & 0x80ff);
 }
 
-void set_line(struct terminal *t, int x, int y, int l, chr *line)
+void set_line(int x, int y, int l, chr *line)
 {
     int i;
-    t->dirty = 1;
-    for (i = x >= 0 ? 0 : -x; i < (x+l <= t->x ? l : t->x-x); i++)
-        t->screen[x+i + t->x * y] = line[i];
+    term->dirty = 1;
+    for (i = x >= 0 ? 0 : -x; i < (x+l <= term->x ? l : term->x-x); i++)
+        term->screen[x+i + term->x * y] = line[i];
 }
 
-void set_line_color(struct terminal *t, int x, int y, int l, unsigned c)
+void set_line_color(int x, int y, int l, unsigned c)
 {
     int i;
-    t->dirty = 1;
-    for (i = x >= 0 ? 0 : -x; i < (x+l <= t->x ? l : t->x-x); i++)
-        t->screen[x+i + t->x * y] = (t->screen[x+i + t->x * y] & 0x80ff) | (c & ~0x80ff);
+    term->dirty = 1;
+    for (i = x >= 0 ? 0 : -x; i < (x+l <= term->x ? l : term->x-x); i++)
+        term->screen[x+i + term->x * y] = (term->screen[x+i + term->x * y] & 0x80ff) | (c & ~0x80ff);
 }
 
-void fill_area(struct terminal *t, int x, int y, int xw, int yw, unsigned c)
+void fill_area(int x, int y, int xw, int yw, unsigned c)
 {
     int i,j;
-    t->dirty = 1;
-    for (j = y >= 0 ? 0 : -y; j < yw && y+j < t->y; j++)
-        for (i = x >= 0 ? 0 : -x; i < xw && x+i < t->x; i++) 
-            t->screen[x+i + t->x*(y+j)] = c;
+    term->dirty = 1;
+    for (j = y >= 0 ? 0 : -y; j < yw && y+j < term->y; j++)
+        for (i = x >= 0 ? 0 : -x; i < xw && x+i < term->x; i++) 
+            term->screen[x+i + term->x*(y+j)] = c;
+}
+
+void fill_lastarea(int x, int y, int xw, int yw, unsigned c)
+{
+    int i,j;
+    for (j = y >= 0 ? 0 : -y; j < yw && y+j < term->y; j++)
+        for (i = x >= 0 ? 0 : -x; i < xw && x+i < term->x; i++) 
+            term->last_screen[x+i + term->x*(y+j)] = c;
 }
 
 int p1[] = { 218, 191, 192, 217, 179, 196 };
 int p2[] = { 201, 187, 200, 188, 186, 205 };
 
-void draw_frame(struct terminal *t, int x, int y, int xw, int yw, unsigned c, int w)
+void draw_frame(int x, int y, int xw, int yw, unsigned c, int w)
 {
     int *p = w > 1 ? p2 : p1;
     c |= ATTR_FRAME;
-    set_char(t, x, y, c+p[0]);
-    set_char(t, x+xw-1, y, c+p[1]);
-    set_char(t, x, y+yw-1, c+p[2]);
-    set_char(t, x+xw-1, y+yw-1, c+p[3]);
-    fill_area(t, x, y+1, 1, yw-2, c+p[4]);
-    fill_area(t, x+xw-1, y+1, 1, yw-2, c+p[4]);
-    fill_area(t, x+1, y, xw-2, 1, c+p[5]);
-    fill_area(t, x+1, y+yw-1, xw-2, 1, c+p[5]);
+    set_char(x, y, c+p[0]);
+    set_char(x+xw-1, y, c+p[1]);
+    set_char(x, y+yw-1, c+p[2]);
+    set_char(x+xw-1, y+yw-1, c+p[3]);
+    fill_area(x, y+1, 1, yw-2, c+p[4]);
+    fill_area(x+xw-1, y+1, 1, yw-2, c+p[4]);
+    fill_area(x+1, y, xw-2, 1, c+p[5]);
+    fill_area(x+1, y+yw-1, xw-2, 1, c+p[5]);
 }
 
-void print_text(struct terminal *t, int x, int y, int l, char *text, unsigned c)
+void print_text(int x, int y, int l, char *text, unsigned c)
 {
-    for (; l-- && *text; text++, x++) set_char(t, x, y,((unsigned char)*text) + c);
+    for (; l-- && *text; text++, x++) set_char(x, y,((unsigned char)*text) + c);
 }
 
-void set_cursor(struct terminal *term, int x, int y, int altx, int alty)
+void set_cursor(int x, int y, int altx, int alty)
 {
     term->dirty = 1;
     if (term->spec->block_cursor) x = altx, y = alty;
@@ -830,23 +876,23 @@ void exec_thread(char *path, int p)
     if (path[1 + strlen(path + 1) + 1]) unlink(path + 1 + strlen(path + 1) + 1);
 }
 
-void close_handle(void *p)
+void close_handle(union cba_t cba)
 {
-    int h = (vint)p;
+    int h = cba.int_;
     close(h);
-    set_handlers(h, NULL, NULL, NULL, NULL);
+    set_handlers(h, NULL, NULL, NULL, CBA0);
 }
 
-void unblock_terminal(struct terminal *term)
+void unblock_terminal(union cba_t cba)
 {
-    close_handle((void *)(vint)term->blocked);
+    close_handle((union cba_t)term->blocked);
     term->blocked = -1;
-    set_handlers(term->fdin, (void (*)(void *))in_term, NULL, (void (*)(void *))destroy_terminal, term);
+    set_handlers(term->fdin, in_term, NULL, destroy_terminal, CBA0);
     unblock_itrm(term->fdin);
-    redraw_terminal_cls(term);
+    redraw_terminal_cls();
 }
 
-void exec_on_terminal(struct terminal *term, char *path, char *delete, int fg)
+void exec_on_terminal(char *path, char *delete, int fg)
 {
     if (path && !*path) return;
     if (!path) path="";
@@ -875,11 +921,11 @@ void exec_on_terminal(struct terminal *term, char *path, char *delete, int fg)
             mem_free(param);
             if (fg == 1) {
                 term->blocked = blockh;
-                set_handlers(blockh, (void (*)(void *))unblock_terminal, NULL, (void (*)(void *))unblock_terminal, term);
-                set_handlers(term->fdin, NULL, NULL, (void (*)(void *))destroy_terminal, term);
+                set_handlers(blockh, unblock_terminal, NULL, unblock_terminal, CBA0);
+                set_handlers(term->fdin, NULL, NULL, destroy_terminal, CBA0);
                 /*block_itrm(term->fdin);*/
             } else {
-                set_handlers(blockh, close_handle, NULL, close_handle, (void *)(vint)blockh);
+                set_handlers(blockh, close_handle, NULL, close_handle, (union cba_t)blockh);
             }
         }
     } else {
@@ -901,35 +947,38 @@ void exec_on_terminal(struct terminal *term, char *path, char *delete, int fg)
     }
 }
 
-void do_terminal_function(struct terminal *term, char code, char *data)
+void do_terminal_function(char code, char *data)
 {
     char *x_data;
     if (!(x_data = mem_alloc(strlen(data) + 2))) return;
     x_data[0] = code;
     strcpy(x_data + 1, data);
-    exec_on_terminal(term, NULL, x_data, 0);
+    exec_on_terminal(NULL, x_data, 0);
     mem_free(x_data);
 }
 
-void set_terminal_title(struct terminal *term, char *title)
+void set_terminal_title(char *title)
 {
     if (term->title && !strcmp(title, term->title)) return;
     if (term->title) mem_free(term->title);
     term->title = stracpy(title);
-    do_terminal_function(term, TERM_FN_TITLE, title);
+#ifdef HAVE_SDL    
+    if (!sdl) 
+#endif
+        do_terminal_function(TERM_FN_TITLE, title);
 }
     
 
-void set_ctest_title(struct terminal *term){
+void set_ctest_title(){
     gchar *c;
 
     if (ctest){
-        c = g_strdup_printf(" %s %s - %s %s", ctest->pcall, ctest->tname, PACKAGE, VERSION);
-        set_terminal_title(term, c);
+        c = g_strdup_printf(" %s %s - %s %s", ctest->pcall, ctest->tname, PACKAGE_NAME, PACKAGE_VERSION);
+        set_terminal_title(c);
         g_free(c);
     }else{
-        c = g_strdup_printf(" %s %s", PACKAGE, VERSION);
-        set_terminal_title(term, c);
+        c = g_strdup_printf(" %s %s", PACKAGE_NAME, PACKAGE_VERSION);
+        set_terminal_title(c);
         g_free(c);
     }
 }
